@@ -874,6 +874,8 @@ def readFiles(files: Iterable[str]) -> Tuple[JumbledFunctions, JumbledFunctions,
 							 or match("gstinterfaces010","gstaudio")\
 							 or match("gstreamer010","gstreamer")	\
 							 or match("appindicator","appindicator3")\
+							 or match("libgl",        "libegl")     \
+							 or match("libgl",        "libglx")     \
 							 \
 							 or match("libc",        "tcmallocminimal") \
 							 or match("libc",        "ldlinux") 	\
