@@ -5,10 +5,8 @@
 
 #include "debug.h"
 #include "box64context.h"
-#include "dynarec.h"
+#include "box64cpu.h"
 #include "emu/x64emu_private.h"
-#include "emu/x64run_private.h"
-#include "x64run.h"
 #include "x64emu.h"
 #include "box64stack.h"
 #include "callback.h"
@@ -135,7 +133,8 @@ uintptr_t dynarec64_DB(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int nin
             INST_NAME("FNINIT");
             MESSAGE(LOG_DUMP, "Need Optimization (FNINIT)\n");
             x87_purgecache(dyn, ninst, 0, x1, x2, x3);
-            CALL(reset_fpu, -1);
+            CALL(const_reset_fpu, -1);
+            NATIVE_RESTORE_X87PC();
             break;
         case 0xE8:
         case 0xE9:
@@ -149,10 +148,12 @@ uintptr_t dynarec64_DB(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int nin
             SETFLAGS(X_ALL, SF_SET);
             v1 = x87_get_st(dyn, ninst, x1, x2, 0, X87_COMBINE(0, nextop&7));
             v2 = x87_get_st(dyn, ninst, x1, x2, nextop&7, X87_COMBINE(0, nextop&7));
-            if(ST_IS_F(0)) {
-                FCMPS(v1, v2);
-            } else {
-                FCMPD(v1, v2);
+            IFX(X_CF|X_PF|X_ZF) {
+                if(ST_IS_F(0)) {
+                    FCMPS(v1, v2);
+                } else {
+                    FCMPD(v1, v2);
+                }
             }
             FCOMI(x1, x2);
             break;
@@ -168,10 +169,12 @@ uintptr_t dynarec64_DB(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int nin
             SETFLAGS(X_ALL, SF_SET);
             v1 = x87_get_st(dyn, ninst, x1, x2, 0, X87_COMBINE(0, nextop&7));
             v2 = x87_get_st(dyn, ninst, x1, x2, nextop&7, X87_COMBINE(0, nextop&7));
-            if(ST_IS_F(0)) {
-                FCMPS(v1, v2);
-            } else {
-                FCMPD(v1, v2);
+            IFX(X_CF|X_PF|X_ZF) {
+                if(ST_IS_F(0)) {
+                    FCMPS(v1, v2);
+                } else {
+                    FCMPD(v1, v2);
+                }
             }
             FCOMI(x1, x2);
             break;
@@ -194,7 +197,7 @@ uintptr_t dynarec64_DB(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int nin
                 v1 = x87_get_st(dyn, ninst, x1, x2, 0, NEON_CACHE_ST_D);
                 addr = geted(dyn, addr, ninst, nextop, &wback, x2, &fixedaddress, &unscaled, 0xfff<<2, 3, rex, NULL, 0, 0);
                 s0 = fpu_get_scratch(dyn, ninst);
-                if(arm64_frintts) {
+                if(cpuext.frintts) {
                     FRINT32ZD(s0, v1);
                     FCVTZSwD(x5, s0);
                     STW(x5, wback, fixedaddress);
@@ -224,7 +227,7 @@ uintptr_t dynarec64_DB(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int nin
                 u8 = x87_setround(dyn, ninst, x1, x2, x4); // x1 have the modified RPSCR reg
                 addr = geted(dyn, addr, ninst, nextop, &wback, x2, &fixedaddress, &unscaled, 0xfff<<2, 3, rex, NULL, 0, 0);
                 s0 = fpu_get_scratch(dyn, ninst);
-                if(arm64_frintts) {
+                if(cpuext.frintts) {
                     FRINT32XD(s0, v1);
                     FCVTZSwD(x5, s0);
                     STW(x5, wback, fixedaddress);
@@ -254,7 +257,7 @@ uintptr_t dynarec64_DB(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int nin
                 u8 = x87_setround(dyn, ninst, x1, x2, x4); // x1 have the modified RPSCR reg
                 addr = geted(dyn, addr, ninst, nextop, &wback, x2, &fixedaddress, &unscaled, 0xfff<<2, 3, rex, NULL, 0, 0);
                 s0 = fpu_get_scratch(dyn, ninst);
-                if(arm64_frintts) {
+                if(cpuext.frintts) {
                     FRINT32XD(s0, v1);
                     FCVTZSwD(x5, s0);
                     STW(x5, wback, fixedaddress);
@@ -310,7 +313,7 @@ uintptr_t dynarec64_DB(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int nin
                         X87_PUSH_EMPTY_OR_FAIL(dyn, ninst, 0);
                         // sync top
                         x87_reflectcount(dyn, ninst, x3, x4);
-                        CALL(native_fld, -1);
+                        CALL(const_native_fld, -1);
                         // go back with the top & stack counter
                         x87_unreflectcount(dyn, ninst, x3, x4);
                     }
@@ -328,7 +331,7 @@ uintptr_t dynarec64_DB(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int nin
                         addr = geted(dyn, addr, ninst, nextop, &ed, x1, &fixedaddress, NULL, 0, 0, rex, NULL, 0, 0);
                         if(ed!=x1) {MOVx_REG(x1, ed);}
                         x87_reflectcount(dyn, ninst, x3, x4);
-                        CALL(native_fstp, -1);
+                        CALL(const_native_fstp, -1);
                         x87_unreflectcount(dyn, ninst, x3, x4);
                     } else {
                         // Painfully long, straight conversion from the C code, shoud be optimized
