@@ -1,6 +1,10 @@
 #ifndef __DYNAREC_HELPER__H_
 #define __DYNAREC_HELPER__H_
 
+#include "env.h"
+#include "pe_tools.h"
+#include "debug.h"
+
 /* Box64 Strong Memory Model Emulation
  *
  * Definition of a SEQ:
@@ -32,6 +36,8 @@
 #define STRONGMEM_LAST_WRITE 1 // The level of a barrier before the last guest memory store will be put
 #define STRONGMEM_SEQ_WRITE  3 // The level of a barrier at every third memory store will be  put
 
+#define STRONGMEM_LEVEL() ((box64_wine && VolatileRangesContains(ip)) ? 0 : BOX64DRENV(dynarec_strongmem))
+
 #if STEP == 1
 
 #define SMWRITE()                                                \
@@ -43,21 +49,26 @@
         dyn->insts[ninst].will_write = 1;                        \
     } while (0)
 
-#define SMWRITELOCK(lock)              \
-    do {                               \
-        dyn->insts[ninst].lock = lock; \
-        SMWRITE();                     \
+#define SMWRITELOCK(lock)                   \
+    do {                                    \
+        if(lock) dyn->insts[ninst].lock = 1;\
+        SMWRITE();                          \
     } while (0)
 
-#define SMWRITE2()                                                   \
-    do {                                                             \
-        if (BOX64DRENV(dynarec_strongmem) >= STRONGMEM_SIMD_WRITE) { \
-            dyn->smwrite = 1;                                        \
-            dyn->insts[ninst].will_write = 2;                        \
-        }                                                            \
+#define SMWRITE2()                                       \
+    do {                                                 \
+        if (STRONGMEM_LEVEL() >= STRONGMEM_SIMD_WRITE) { \
+            dyn->smwrite = 1;                            \
+            dyn->insts[ninst].will_write = 2;            \
+        }                                                \
+    } while (0)
+
+#define SMREAD()                                               \
+    do {                                                       \
+        /* Mark that current opcode read from guest memory. */ \
+        dyn->insts[ninst].will_read = 1;                       \
     } while (0)
 
-#define SMREAD()
 #define SMREADLOCK(lock)
 #define WILLWRITE()
 #define WILLWRITELOCK(lock)
@@ -71,7 +82,7 @@
 #define SMEND()                                                                                \
     do {                                                                                       \
         /* If there is any guest memory write, which is a SEQ, then compute the last_write. */ \
-        if (dyn->smwrite && (BOX64DRENV(dynarec_strongmem) >= STRONGMEM_LAST_WRITE)) {         \
+        if (dyn->smwrite && (STRONGMEM_LEVEL() >= STRONGMEM_LAST_WRITE)) {                     \
             int i = ninst;                                                                     \
             while (i >= 0 && !dyn->insts[i].will_write)                                        \
                 --i;                                                                           \
@@ -84,19 +95,24 @@
 
 #else
 
-// An opcode writes guest memory, this need to be put after the STORE instruction manually.
-#define SMWRITE()                                                     \
-    do {                                                              \
-        /* Put a barrier at every third memory write. */              \
-        if (BOX64DRENV(dynarec_strongmem) >= STRONGMEM_SEQ_WRITE) {   \
-            if (++dyn->smwrite >= 3 /* Every third memory write */) { \
-                DMB_ISH();                                            \
-                dyn->smwrite = 1;                                     \
-            }                                                         \
-        } else {                                                      \
-            /* Mark that current sequence writes to guest memory. */  \
-            dyn->smwrite = 1;                                         \
-        }                                                             \
+// An opcode writes guest memory, this need to be put after the STORE instruction manually. It will also end the SEQ automaticaly on last_write immediatly
+#define SMWRITE()                                                         \
+    do {                                                                  \
+        if (dyn->insts[ninst].last_write) {                               \
+            dyn->smwrite = 1;                                             \
+            SMEND();                                                      \
+        } else {                                                          \
+            /* Put a barrier at every third memory write. */              \
+            if (STRONGMEM_LEVEL() >= STRONGMEM_SEQ_WRITE) {               \
+                if (++dyn->smwrite >= 3 /* Every third memory write */) { \
+                    DMB_ISH();                                            \
+                    dyn->smwrite = 1;                                     \
+                }                                                         \
+            } else {                                                      \
+                /* Mark that current sequence writes to guest memory. */  \
+                dyn->smwrite = 1;                                         \
+            }                                                             \
+        }                                                                 \
     } while (0)
 
 // Similar to SMWRITE, but checks lock.
@@ -110,10 +126,10 @@
     } while (0)
 
 // Similar to SMWRITE, but for SIMD instructions.
-#define SMWRITE2()                                                 \
-    do {                                                           \
-        if (BOX64DRENV(dynarec_strongmem) >= STRONGMEM_SIMD_WRITE) \
-            SMWRITE();                                             \
+#define SMWRITE2()                                     \
+    do {                                               \
+        if (STRONGMEM_LEVEL() >= STRONGMEM_SIMD_WRITE) \
+            SMWRITE();                                 \
     } while (0)
 
 // An opcode reads guest memory, this need to be put before the LOAD instruction manually.
@@ -130,22 +146,22 @@
     } while (0)
 
 // An opcode will write memory, this will be put before the STORE instruction automatically.
-#define WILLWRITE()                                                                                            \
-    do {                                                                                                       \
-        if (BOX64DRENV(dynarec_strongmem) >= dyn->insts[ninst].will_write && dyn->smwrite == 0) {              \
-            /* Will write but never written, this is the start of a SEQ, put a barrier. */                     \
-            if (BOX64ENV(dynarec_weakbarrier))                                                                 \
-                DMB_ISHLD();                                                                                   \
-            else                                                                                               \
-                DMB_ISH();                                                                                     \
-        } else if (BOX64DRENV(dynarec_strongmem) >= STRONGMEM_LAST_WRITE && BOX64ENV(dynarec_weakbarrier) != 2 \
-            && dyn->insts[ninst].last_write) {                                                                 \
-            /* Last write, put a barrier */                                                                    \
-            if (BOX64ENV(dynarec_weakbarrier))                                                                 \
-                DMB_ISHST();                                                                                   \
-            else                                                                                               \
-                DMB_ISH();                                                                                     \
-        }                                                                                                      \
+#define WILLWRITE()                                                                                \
+    do {                                                                                           \
+        if (STRONGMEM_LEVEL() >= dyn->insts[ninst].will_write && dyn->smwrite == 0) {              \
+            /* Will write but never written, this is the start of a SEQ, put a barrier. */         \
+            if (BOX64ENV(dynarec_weakbarrier))                                                     \
+                DMB_ISHLD();                                                                       \
+            else                                                                                   \
+                DMB_ISH();                                                                         \
+        } else if (STRONGMEM_LEVEL() >= STRONGMEM_LAST_WRITE && BOX64ENV(dynarec_weakbarrier) != 2 \
+            && dyn->insts[ninst].last_write) {                                                     \
+            /* Last write, put a barrier */                                                        \
+            if (BOX64ENV(dynarec_weakbarrier))                                                     \
+                DMB_ISHST();                                                                       \
+            else                                                                                   \
+                DMB_ISH();                                                                         \
+        }                                                                                          \
     } while (0)
 
 // Similar to WILLWRITE, but checks lock.
@@ -167,7 +183,7 @@
 // Will be put at the end of the SEQ
 #define SMEND()                                                         \
     do {                                                                \
-        if (BOX64DRENV(dynarec_strongmem)) {                            \
+        if (STRONGMEM_LEVEL()) {                                        \
             /* It's a SEQ, put a barrier here. */                       \
             if (dyn->smwrite) {                                         \
                 /* Check if the next instruction has a end loop mark */ \
