@@ -55,9 +55,10 @@ void native_clflush(x64emu_t* emu, void* p);
 void native_ud(x64emu_t* emu);
 void native_br(x64emu_t* emu);
 void native_priv(x64emu_t* emu);
-void native_singlestep(x64emu_t* emu);
+void native_gpf(x64emu_t* emu);
 void native_int3(x64emu_t* emu);
 void native_int(x64emu_t* emu, int num);
+void native_wineint(x64emu_t* emu, int num);
 void native_div0(x64emu_t* emu);
 
 // Caches transformation (for loops) // Specific, need to be written par backend
@@ -74,16 +75,6 @@ uintptr_t fakeed(dynarec_native_t* dyn, uintptr_t addr, int ninst, uint8_t nexto
 // return Ib on a mod/rm opcode without emitting anything
 uint8_t geted_ib(dynarec_native_t* dyn, uintptr_t addr, int ninst, uint8_t nextop);
 
-// Is what pointed at addr a native call? And if yes, to what function?
-int isNativeCall(dynarec_native_t* dyn, uintptr_t addr, int is32bits, uintptr_t* calladdress, uint16_t* retn);
-
-// AVX utilities
-void avx_mark_zero(dynarec_native_t* dyn, int ninst, int reg);
-int is_avx_zero(dynarec_native_t* dyn, int ninst, int reg);
-int is_avx_zero_unset(dynarec_native_t* dyn, int ninst, int reg);
-void avx_mark_zero_reset(dynarec_native_t* dyn, int ninst);
-void avx_unmark_zero(dynarec_native_t* dyn, int ninst, int reg);
-
 typedef struct register_mapping_s {
     const char* name;
     const char* native;
