@@ -5,11 +5,9 @@
 
 #include "debug.h"
 #include "box64context.h"
-#include "dynarec.h"
+#include "box64cpu.h"
 #include "emu/x64emu_private.h"
-#include "emu/x64run_private.h"
 #include "la64_emitter.h"
-#include "x64run.h"
 #include "x64emu.h"
 #include "box64stack.h"
 #include "callback.h"
@@ -32,7 +30,7 @@ void emit_xor8(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
         SET_DFNONE();
     }
 
-    IFXA (X_ALL, la64_lbt) {
+    IFXA (X_ALL, cpuext.lbt) {
         X64_XOR_B(s1, s2);
     }
 
@@ -43,7 +41,7 @@ void emit_xor8(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
         ST_B(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) return;
+    if (cpuext.lbt) return;
 
     CLEAR_FLAGS(s3);
     IFX (X_SF) {
@@ -70,7 +68,7 @@ void emit_xor8c(dynarec_la64_t* dyn, int ninst, int s1, int32_t c, int s3, int s
         SET_DFNONE();
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         IFX (X_ALL) {
             ADDI_D(s3, xZR, c & 0xff);
             X64_XOR_B(s1, s3);
@@ -113,7 +111,7 @@ void emit_xor16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4,
     }
 
 
-    IFXA (X_ALL, la64_lbt) {
+    IFXA (X_ALL, cpuext.lbt) {
         X64_XOR_W(s1, s2);
     }
 
@@ -124,7 +122,7 @@ void emit_xor16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4,
         ST_H(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) return;
+    if (cpuext.lbt) return;
 
     CLEAR_FLAGS(s3);
     IFX (X_ZF | X_SF) {
@@ -153,7 +151,7 @@ void emit_xor32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
         SET_DFNONE();
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         IFX(X_ALL) {
             if (rex.w)
                 X64_XOR_D(s1, s2);
@@ -205,7 +203,7 @@ void emit_xor32c(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int64_t c, i
         SET_DFNONE();
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         IFX (X_ALL) {
             MOV64xw(s3, c);
             if (rex.w)
@@ -272,7 +270,7 @@ void emit_and8(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
         SET_DFNONE();
     }
 
-    IFXA(X_ALL, la64_lbt) {
+    IFXA(X_ALL, cpuext.lbt) {
         X64_AND_B(s1, s2);
     }
 
@@ -282,7 +280,7 @@ void emit_and8(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
         ST_B(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) return;
+    if (cpuext.lbt) return;
 
     CLEAR_FLAGS(s3);
     IFX(X_SF) {
@@ -311,7 +309,7 @@ void emit_and8c(dynarec_la64_t* dyn, int ninst, int s1, int32_t c, int s3, int s
     }
 
 
-    IFXA(X_ALL, la64_lbt) {
+    IFXA(X_ALL, cpuext.lbt) {
         MOV32w(s3, c);
         X64_AND_B(s1, s3);
     }
@@ -322,7 +320,7 @@ void emit_and8c(dynarec_la64_t* dyn, int ninst, int s1, int32_t c, int s3, int s
         ST_D(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) return;
+    if (cpuext.lbt) return;
 
     CLEAR_FLAGS(s3);
     IFX(X_SF) {
@@ -348,8 +346,8 @@ void emit_and16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
         SET_DFNONE();
     }
 
-    IFXA (X_ALL, la64_lbt) {
-        X64_AND_W(s1, s2);
+    IFXA (X_ALL, cpuext.lbt) {
+        X64_AND_H(s1, s2);
     }
 
     AND(s1, s1, s2); // res = s1 & s2
@@ -359,7 +357,7 @@ void emit_and16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
         ST_H(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) return;
+    if (cpuext.lbt) return;
 
     CLEAR_FLAGS(s3);
     IFX (X_SF) {
@@ -387,7 +385,7 @@ void emit_and32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
     }
 
 
-    IFXA(X_ALL, la64_lbt) {
+    IFXA(X_ALL, cpuext.lbt) {
         if (rex.w)
             X64_AND_D(s1, s2);
         else
@@ -401,7 +399,7 @@ void emit_and32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
         SDxw(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) return;
+    if (cpuext.lbt) return;
 
     CLEAR_FLAGS(s3);
     IFX(X_SF) {
@@ -429,7 +427,7 @@ void emit_and32c(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int64_t c, i
         SET_DFNONE();
     }
 
-    IFXA(X_ALL, la64_lbt) {
+    IFXA(X_ALL, cpuext.lbt) {
         MOV64xw(s3, c);
         if (rex.w)
             X64_AND_D(s1, s3);
@@ -440,7 +438,7 @@ void emit_and32c(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int64_t c, i
     if (c >= 0 && c <= 4095) {
         ANDI(s1, s1, c);
     } else {
-        IFXA(X_ALL, la64_lbt) { } else MOV64xw(s3, c);
+        IFXA(X_ALL, cpuext.lbt) { } else MOV64xw(s3, c);
         AND(s1, s1, s3); // res = s1 & s2
     }
 
@@ -448,7 +446,7 @@ void emit_and32c(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int64_t c, i
         SDxw(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) return;
+    if (cpuext.lbt) return;
 
     CLEAR_FLAGS(s3);
     IFX(X_SF) {
@@ -476,7 +474,7 @@ void emit_or16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
         SET_DFNONE();
     }
 
-    IFXA (X_ALL, la64_lbt) {
+    IFXA (X_ALL, cpuext.lbt) {
         X64_OR_H(s1, s2);
     }
 
@@ -486,7 +484,7 @@ void emit_or16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
         ST_D(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) return;
+    if (cpuext.lbt) return;
 
     CLEAR_FLAGS(s3);
     IFX (X_SF) {
@@ -514,7 +512,7 @@ void emit_or32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s3
         SET_DFNONE();
     }
 
-    IFXA (X_ALL, la64_lbt) {
+    IFXA (X_ALL, cpuext.lbt) {
         if (rex.w)
             X64_OR_D(s1, s2);
         else
@@ -527,7 +525,7 @@ void emit_or32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s3
         SDxw(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         if (!rex.w) ZEROUP(s1);
         return;
     }
@@ -560,7 +558,7 @@ void emit_or32c(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int64_t c, in
         SET_DFNONE();
     }
 
-    IFXA(X_ALL, la64_lbt) {
+    IFXA(X_ALL, cpuext.lbt) {
         MOV64xw(s3, c);
         if (rex.w)
             X64_OR_D(s1, s3);
@@ -571,7 +569,7 @@ void emit_or32c(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int64_t c, in
     if (c >= 0 && c <= 4095) {
         ORI(s1, s1, c);
     } else {
-        IFXA(X_ALL, la64_lbt) { } else MOV64xw(s3, c);
+        IFXA(X_ALL, cpuext.lbt) { } else MOV64xw(s3, c);
         OR(s1, s1, s3);
     }
 
@@ -579,7 +577,7 @@ void emit_or32c(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int64_t c, in
         SDxw(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         if (!rex.w) ZEROUP(s1);
         return;
     }
@@ -615,7 +613,7 @@ void emit_or8(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
         SET_DFNONE();
     }
 
-    IFXA (X_ALL, la64_lbt) {
+    IFXA (X_ALL, cpuext.lbt) {
         X64_OR_B(s1, s2);
     }
 
@@ -625,7 +623,7 @@ void emit_or8(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
         ST_B(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) return;
+    if (cpuext.lbt) return;
 
     CLEAR_FLAGS(s3);
     IFX (X_SF) {
