@@ -5,11 +5,9 @@
 
 #include "debug.h"
 #include "box64context.h"
-#include "dynarec.h"
+#include "box64cpu.h"
 #include "emu/x64emu_private.h"
-#include "emu/x64run_private.h"
 #include "la64_emitter.h"
-#include "x64run.h"
 #include "x64emu.h"
 #include "box64stack.h"
 #include "callback.h"
@@ -39,12 +37,12 @@ void emit_add32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
         SET_DFNONE();
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         IFX (X_ALL) {
             if (rex.w)
-                X64_ADD_DU(s1, s2);
+                X64_ADD_D(s1, s2);
             else
-                X64_ADD_WU(s1, s2);
+                X64_ADD_W(s1, s2);
         }
         ADDxw(s1, s1, s2);
         if (!rex.w) ZEROUP(s1);
@@ -137,7 +135,7 @@ void emit_add32c(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int64_t c, i
     }
     IFX (X_PEND | X_AF | X_CF | X_OF) {
         MOV64xw(s2, c);
-    } else if (la64_lbt) {
+    } else if (cpuext.lbt) {
         MOV64xw(s2, c);
     }
     IFX (X_PEND) {
@@ -148,12 +146,12 @@ void emit_add32c(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int64_t c, i
         SET_DFNONE();
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         IFX (X_ALL) {
             if (rex.w)
-                X64_ADD_DU(s1, s2);
+                X64_ADD_D(s1, s2);
             else
-                X64_ADD_WU(s1, s2);
+                X64_ADD_W(s1, s2);
         }
         ADDxw(s1, s1, s2);
         if (!rex.w) ZEROUP(s1);
@@ -248,7 +246,7 @@ void emit_add8(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
         SET_DFNONE();
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         IFX (X_ALL) {
             X64_ADD_B(s1, s2);
         }
@@ -319,7 +317,7 @@ void emit_add8c(dynarec_la64_t* dyn, int ninst, int s1, int c, int s2, int s3, i
         SET_DFNONE();
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         IFX (X_ALL) {
             IFX (X_PEND) {
             } else {
@@ -396,12 +394,12 @@ void emit_add16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4,
     } else IFXORNAT (X_ALL) {
         SET_DFNONE();
     }
-    IFXA (X_AF | X_OF, !la64_lbt) {
+    IFXA (X_AF | X_OF, !cpuext.lbt) {
         OR(s3, s1, s2);  // s3 = op1 | op2
         AND(s4, s1, s2); // s4 = op1 & op2
     }
 
-    IFXA (X_ALL, la64_lbt) {
+    IFXA (X_ALL, cpuext.lbt) {
         X64_ADD_H(s1, s2);
     }
 
@@ -411,7 +409,7 @@ void emit_add16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4,
         ST_W(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         BSTRPICK_D(s1, s1, 15, 0);
         return;
     }
@@ -469,7 +467,7 @@ void emit_sub8(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4, i
         SET_DFNONE();
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         IFX (X_ALL) {
             X64_SUB_B(s1, s2);
         }
@@ -525,12 +523,12 @@ void emit_sub16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4,
         SET_DFNONE();
     }
 
-    IFXA (X_AF | X_CF | X_OF, !la64_lbt) {
+    IFXA (X_AF | X_CF | X_OF, !cpuext.lbt) {
         // for later flag calculation
         NOR(s5, xZR, s1);
     }
 
-    IFXA (X_ALL, la64_lbt) {
+    IFXA (X_ALL, cpuext.lbt) {
         X64_SUB_H(s1, s2);
     }
 
@@ -539,7 +537,7 @@ void emit_sub16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4,
         ST_H(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         BSTRPICK_D(s1, s1, 15, 0);
         return;
     }
@@ -574,12 +572,12 @@ void emit_sub32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
         SET_DFNONE();
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         IFX (X_ALL) {
             if (rex.w)
-                X64_SUB_DU(s1, s2);
+                X64_SUB_D(s1, s2);
             else
-                X64_SUB_WU(s1, s2);
+                X64_SUB_W(s1, s2);
         }
         SUBxw(s1, s1, s2);
         if (!rex.w) ZEROUP(s1);
@@ -641,16 +639,16 @@ void emit_sub32c(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int64_t c, i
         SET_DFNONE();
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         IFX (X_PEND) {
         } else {
             MOV64xw(s2, c);
         }
         IFX (X_ALL) {
             if (rex.w) {
-                X64_SUB_DU(s1, s2);
+                X64_SUB_D(s1, s2);
             } else {
-                X64_SUB_WU(s1, s2);
+                X64_SUB_W(s1, s2);
             }
         }
         SUBxw(s1, s1, s2);
@@ -717,7 +715,7 @@ void emit_sbb8(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4, i
     }
 
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         SBC_B(s3, s1, s2);
         IFX (X_ALL) {
             X64_SBC_B(s1, s2);
@@ -778,7 +776,7 @@ void emit_sbb16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4,
         SET_DFNONE();
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         SBC_H(s3, s1, s2);
 
         IFX (X_ALL) {
@@ -833,7 +831,7 @@ void emit_sbb32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
         SET_DFNONE();
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         if (rex.w) {
             SBC_D(s3, s1, s2);
         } else {
@@ -901,7 +899,7 @@ void emit_neg8(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3)
         MV(s3, s1); // s3 = op1
     }
 
-    IFXA (X_ALL, la64_lbt) {
+    IFXA (X_ALL, cpuext.lbt) {
         X64_SUB_B(xZR, s1);
     }
 
@@ -911,7 +909,7 @@ void emit_neg8(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3)
         ST_B(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) return;
+    if (cpuext.lbt) return;
 
     CLEAR_FLAGS(s2);
     IFX (X_CF) {
@@ -961,17 +959,17 @@ void emit_neg32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
         SET_DFNONE();
     }
 
-    if (!la64_lbt) {
+    if (!cpuext.lbt) {
         IFX (X_AF | X_OF) {
             MV(s3, s1); // s3 = op1
         }
     }
 
-    IFXA (X_ALL, la64_lbt) {
+    IFXA (X_ALL, cpuext.lbt) {
         if (rex.w)
-            X64_SUB_DU(xZR, s1);
+            X64_SUB_D(xZR, s1);
         else
-            X64_SUB_WU(xZR, s1);
+            X64_SUB_W(xZR, s1);
     }
 
     NEGxw(s1, s1);
@@ -979,7 +977,7 @@ void emit_neg32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
         SDxw(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         if (!rex.w) ZEROUP(s1);
         return;
     }
@@ -1036,7 +1034,7 @@ void emit_adc8(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4, i
         SET_DFNONE();
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         ADC_B(s3, s1, s2);
 
         IFX (X_ALL) {
@@ -1120,7 +1118,7 @@ void emit_adc16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4,
         SET_DFNONE();
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         ADC_H(s3, s1, s2);
 
         IFX (X_ALL) {
@@ -1197,7 +1195,7 @@ void emit_adc32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
         SET_DFNONE();
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         if (rex.w)
             ADC_D(s3, s1, s2);
         else
@@ -1302,12 +1300,19 @@ void emit_inc8(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
     } else IFXORNAT (X_ALL) {
         SET_DFNONE();
     }
-    IFXA (X_AF | X_OF, !la64_lbt) {
+
+    IFXA (X_ALL, !cpuext.lbt) {
+        // preserving CF
+        MOV64x(s4, (1UL << F_AF) | (1UL << F_OF) | (1UL << F_ZF) | (1UL << F_SF) | (1UL << F_PF));
+        ANDN(xFlags, xFlags, s4);
+    }
+
+    IFXA (X_AF | X_OF, !cpuext.lbt) {
         ORI(s3, s1, 1);  // s3 = op1 | op2
         ANDI(s4, s1, 1); // s5 = op1 & op2
     }
 
-    IFXA (X_ALL, la64_lbt) {
+    IFXA (X_ALL, cpuext.lbt) {
         X64_INC_B(s1);
     }
 
@@ -1317,16 +1322,11 @@ void emit_inc8(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
         ST_B(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         ANDI(s1, s1, 0xff);
         return;
     }
 
-    IFX (X_ALL) {
-        // preserving CF
-        MOV64x(s4, (1UL << F_AF) | (1UL << F_OF) | (1UL << F_ZF) | (1UL << F_SF) | (1UL << F_PF));
-        ANDN(xFlags, xFlags, s4);
-    }
     IFX (X_AF | X_OF) {
         ANDN(s3, s3, s1); // s3 = ~res & (op1 | op2)
         OR(s3, s3, s4);   // cc = (~res & (op1 | op2)) | (op1 & op2)
@@ -1369,12 +1369,19 @@ void emit_inc16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
     } else IFXORNAT (X_ZF | X_OF | X_AF | X_SF | X_PF) {
         SET_DFNONE();
     }
-    IFXA (X_AF | X_OF, !la64_lbt) {
+
+    IFXA (X_ALL, !cpuext.lbt) {
+        // preserving CF
+        MOV64x(s4, (1UL << F_AF) | (1UL << F_OF) | (1UL << F_ZF) | (1UL << F_SF) | (1UL << F_PF));
+        ANDN(xFlags, xFlags, s4);
+    }
+
+    IFXA (X_AF | X_OF, !cpuext.lbt) {
         ORI(s3, s1, 1);  // s3 = op1 | op2
         ANDI(s4, s1, 1); // s4 = op1 & op2
     }
 
-    IFXA (X_ALL, la64_lbt) {
+    IFXA (X_ALL, cpuext.lbt) {
         X64_INC_H(s1);
     }
 
@@ -1384,17 +1391,11 @@ void emit_inc16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
         ST_H(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         BSTRPICK_D(s1, s1, 15, 0);
         return;
     }
 
-    IFX (X_ALL) {
-        // preserving CF
-        MOV64x(s4, (1UL << F_AF) | (1UL << F_OF) | (1UL << F_ZF) | (1UL << F_SF) | (1UL << F_PF));
-        ANDN(xFlags, xFlags, s4);
-    }
-
     IFX (X_AF | X_OF) {
         ANDN(s3, s3, s1); // s3 = ~res & (op1 | op2)
         OR(s3, s3, s4);   // cc = (~res & (op1 | op2)) | (op1 & op2)
@@ -1440,12 +1441,18 @@ void emit_inc32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
         SET_DFNONE();
     }
 
-    IFXA (X_AF | X_OF, !la64_lbt) {
+    IFXA (X_ALL, !cpuext.lbt) {
+        // preserving CF
+        MOV64x(s4, (1UL << F_AF) | (1UL << F_OF) | (1UL << F_ZF) | (1UL << F_SF) | (1UL << F_PF));
+        ANDN(xFlags, xFlags, s4);
+    }
+
+    IFXA (X_AF | X_OF, !cpuext.lbt) {
         ORI(s3, s1, 1);  // s3 = op1 | op2
         ANDI(s5, s1, 1); // s5 = op1 & op2
     }
 
-    IFXA (X_ALL, la64_lbt) {
+    IFXA (X_ALL, cpuext.lbt) {
         if (rex.w) {
             X64_INC_D(s1);
         } else {
@@ -1459,16 +1466,11 @@ void emit_inc32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
         SDxw(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         if (!rex.w) ZEROUP(s1);
         return;
     }
 
-    IFX (X_ALL) {
-        // preserving CF
-        MOV64x(s4, (1UL << F_AF) | (1UL << F_OF) | (1UL << F_ZF) | (1UL << F_SF) | (1UL << F_PF));
-        ANDN(xFlags, xFlags, s4);
-    }
     IFX (X_AF | X_OF) {
         ANDN(s3, s3, s1); // s3 = ~res & (op1 | op2)
         OR(s3, s3, s5);   // cc = (~res & (op1 | op2)) | (op1 & op2)
@@ -1512,13 +1514,20 @@ void emit_dec8(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
     } else IFXORNAT (X_ALL) {
         SET_DFNONE();
     }
-    IFXA (X_AF | X_OF, !la64_lbt) {
+
+    IFXA (X_ALL, !cpuext.lbt) {
+        // preserving CF
+        MOV64x(s4, (1UL << F_AF) | (1UL << F_OF) | (1UL << F_ZF) | (1UL << F_SF) | (1UL << F_PF));
+        ANDN(xFlags, xFlags, s4);
+    }
+
+    IFXA (X_AF | X_OF, !cpuext.lbt) {
         NOR(s4, xZR, s1); // s4 = ~op1
         ORI(s3, s4, 1);   // s3 = ~op1 | op2
         ANDI(s4, s4, 1);  // s4 = ~op1 & op2
     }
 
-    IFXA (X_ALL, la64_lbt) {
+    IFXA (X_ALL, cpuext.lbt) {
         X64_DEC_B(s1);
     }
 
@@ -1528,16 +1537,11 @@ void emit_dec8(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
         ST_B(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         ANDI(s1, s1, 0xff);
         return;
     }
 
-    IFX (X_ALL) {
-        // preserving CF
-        MOV64x(s4, (1UL << F_AF) | (1UL << F_OF) | (1UL << F_ZF) | (1UL << F_SF) | (1UL << F_PF));
-        ANDN(xFlags, xFlags, s4);
-    }
     IFX (X_AF | X_OF) {
         AND(s3, s1, s3); // s3 = res & (~op1 | op2)
         OR(s3, s3, s4);  // cc = (res & (~op1 | op2)) | (~op1 & op2)
@@ -1586,7 +1590,7 @@ void emit_dec16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4,
         ANDI(s5, s5, 1); // s5 = ~op1 & op2
     }
 
-    IFXA (X_ALL, la64_lbt) {
+    IFXA (X_ALL, cpuext.lbt) {
         X64_DEC_H(s1);
     }
 
@@ -1596,7 +1600,7 @@ void emit_dec16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4,
         ST_H(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         BSTRPICK_D(s1, s1, 15, 0);
         return;
     }
@@ -1649,13 +1653,13 @@ void emit_dec32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
     } else IFXORNAT (X_ALL) {
         SET_DFNONE();
     }
-    IFXA (X_AF | X_OF, !la64_lbt) {
+    IFXA (X_AF | X_OF, !cpuext.lbt) {
         NOR(s5, xZR, s1);
         ORI(s3, s5, 1);  // s3 = ~op1 | op2
         ANDI(s5, s5, 1); // s5 = ~op1 & op2
     }
 
-    IFXA (X_ALL, la64_lbt) {
+    IFXA (X_ALL, cpuext.lbt) {
         if (rex.w) {
             X64_DEC_D(s1);
         } else {
@@ -1669,7 +1673,7 @@ void emit_dec32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
         SDxw(s1, xEmu, offsetof(x64emu_t, res));
     }
 
-    if (la64_lbt) {
+    if (cpuext.lbt) {
         if (!rex.w) ZEROUP(s1);
         return;
     }
@@ -1770,7 +1774,7 @@ void emit_neg16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3)
         ORI(xFlags, xFlags, 1 << F_ZF);
     }
 
-    IFXA (X_ALL, la64_lbt) {
+    IFXA (X_ALL, cpuext.lbt) {
         SPILL_EFLAGS();
     }
 }
