@@ -2,7 +2,11 @@
 
 #include "la64_emitter.h"
 
-#define EMIT(A) *block = (A); ++block;
+#define EMIT(A)       \
+    do {              \
+        *block = (A); \
+        ++block;      \
+    } while (0)
 void CreateJmpNext(void* addr, void* next)
 {
     uint32_t* block = (uint32_t*)addr;
@@ -10,4 +14,4 @@ void CreateJmpNext(void* addr, void* next)
     PCADDU12I(x2, SPLIT20(diff));
     LD_D(x2, x2, SPLIT12(diff));
     BR(x2);
-}
\ No newline at end of file
+}
