@@ -19,9 +19,25 @@
     }
 #define INST_EPILOG dyn->insts[ninst].epilog = dyn->native_size;
 #define INST_NAME(name)
-#define TABLE64(A, V)         \
-    {                         \
+#define TABLE64(A, V)                                \
+    do {                                             \
+        if (dyn->need_reloc && !isTable64(dyn, (V))) \
+            AddRelocTable64Addr(dyn, ninst, (V), 2); \
+        Table64(dyn, (V), 2);                        \
+        EMIT(0);                                     \
+        EMIT(0);                                     \
+    } while (0)
+#define TABLE64_(A, V)        \
+    do {                      \
         Table64(dyn, (V), 2); \
         EMIT(0);              \
         EMIT(0);              \
-    }
+    } while (0)
+#define TABLE64C(A, V)                                       \
+    do {                                                     \
+        if (dyn->need_reloc && !isTable64(dyn, getConst(V))) \
+            AddRelocTable64Const(dyn, ninst, (V), 2);        \
+        Table64(dyn, getConst(V), 2);                        \
+        EMIT(0);                                             \
+        EMIT(0);                                             \
+    } while (0)
