@@ -21,8 +21,8 @@
 #include "elfload_dump.h"
 #include "elfloader_private.h"
 #include "librarian.h"
-#include "x64run.h"
 #include "bridge.h"
+#include "alternate.h"
 #include "wrapper.h"
 #include "box64context.h"
 #include "library.h"
@@ -30,6 +30,7 @@
 #include "box64stack.h"
 #include "callback.h"
 #include "box64stack.h"
+#include "box64cpu_util.h"
 #include "wine_tools.h"
 #include "dictionnary.h"
 #include "symbols.h"
