@@ -24,7 +24,7 @@ static int LoadSH(FILE *f, Elf32_Shdr *s, void** SH, const char* name, uint32_t
     if (type==SHT_SYMTAB && s->sh_size%sizeof(Elf32_Sym)) {
         printf_log(LOG_INFO, "Section Header \"%s\" (off=%d, size=%d) has size (not multiple of %ld)\n", name, s->sh_offset, s->sh_size, sizeof(Elf32_Sym));
     }
-    *SH = box_calloc(1, s->sh_size);
+    *SH = actual_calloc(1, s->sh_size);
     fseeko64(f, s->sh_offset ,SEEK_SET);
     if(fread(*SH, s->sh_size, 1, f)!=1) {
             printf_log(LOG_INFO, "Cannot read Section Header \"%s\" (off=%d, size=%d)\n", name, s->sh_offset, s->sh_size);
@@ -115,7 +115,7 @@ elfheader_t* ParseElfHeader32(FILE* f, const char* name, int exec)
         return NULL;
     }
 
-    elfheader_t *h = box_calloc(1, sizeof(elfheader_t));
+    elfheader_t *h = actual_calloc(1, sizeof(elfheader_t));
     h->name = box_strdup(name);
     h->entrypoint = header.e_entry;
     h->numPHEntries = header.e_phnum;
@@ -130,7 +130,7 @@ elfheader_t* ParseElfHeader32(FILE* f, const char* name, int exec)
             fseeko64(f, header.e_shoff, SEEK_SET);
             Elf32_Shdr section;
             if(fread(&section, sizeof(Elf32_Shdr), 1, f)!=1) {
-                box_free(h);
+                actual_free(h);
                 printf_log(LOG_INFO, "Cannot read Initial Section Header\n");
                 return NULL;
             }
@@ -138,7 +138,7 @@ elfheader_t* ParseElfHeader32(FILE* f, const char* name, int exec)
         }
         // now read all section headers
         printf_log(LOG_DEBUG, "Read %d Section header\n", h->numSHEntries);
-        h->SHEntries._32 = (Elf32_Shdr*)box_calloc(h->numSHEntries, sizeof(Elf32_Shdr));
+        h->SHEntries._32 = (Elf32_Shdr*)actual_calloc(h->numSHEntries, sizeof(Elf32_Shdr));
         fseeko64(f, header.e_shoff ,SEEK_SET);
         if(fread(h->SHEntries._32, sizeof(Elf32_Shdr), h->numSHEntries, f)!=h->numSHEntries) {
                 FreeElfHeader(&h);
@@ -154,7 +154,7 @@ elfheader_t* ParseElfHeader32(FILE* f, const char* name, int exec)
     }
 
     printf_log(LOG_DEBUG, "Read %d Program header\n", h->numPHEntries);
-    h->PHEntries._32 = (Elf32_Phdr*)box_calloc(h->numPHEntries, sizeof(Elf32_Phdr));
+    h->PHEntries._32 = (Elf32_Phdr*)actual_calloc(h->numPHEntries, sizeof(Elf32_Phdr));
     fseeko64(f, header.e_phoff ,SEEK_SET);
     if(fread(h->PHEntries._32, sizeof(Elf32_Phdr), h->numPHEntries, f)!=h->numPHEntries) {
             FreeElfHeader(&h);
@@ -178,14 +178,14 @@ elfheader_t* ParseElfHeader32(FILE* f, const char* name, int exec)
             FreeElfHeader(&h);
             return NULL;
         }
-        if(box64_dump) DumpMainHeader32(&header, h);
+        if (BOX64ENV(dump)) DumpMainHeader32(&header, h);
 
         LoadNamedSection(f, h->SHEntries._32, h->numSHEntries, h->SHStrTab, ".strtab", "SymTab Strings", SHT_STRTAB, (void**)&h->StrTab, NULL);
         LoadNamedSection(f, h->SHEntries._32, h->numSHEntries, h->SHStrTab, ".symtab", "SymTab", SHT_SYMTAB, (void**)&h->SymTab._32, &h->numSymTab);
-        if(box64_dump && h->SymTab._32) DumpSymTab32(h);
+        if (BOX64ENV(dump) && h->SymTab._32) DumpSymTab32(h);
 
         LoadNamedSection(f, h->SHEntries._32, h->numSHEntries, h->SHStrTab, ".dynamic", "Dynamic", SHT_DYNAMIC, (void**)&h->Dynamic._32, &h->numDynamic);
-        if(box64_dump && h->Dynamic._32) DumpDynamicSections32(h);
+        if (BOX64ENV(dump) && h->Dynamic._32) DumpDynamicSections32(h);
         // grab DT_REL & DT_RELA stuffs
         // also grab the DT_STRTAB string table
         {
