@@ -5,14 +5,15 @@
 #include <math.h>
 #include <fenv.h>
 #include <string.h>
-#include <signal.h>
 #include <sys/types.h>
 #include <unistd.h>
 
+#include "x64_signals.h"
+#include "os.h"
 #include "debug.h"
 #include "box64stack.h"
+#include "box64cpu_util.h"
 #include "x64emu.h"
-#include "x64run.h"
 #include "x64emu_private.h"
 #include "x64run_private.h"
 #include "x64primop.h"
@@ -21,8 +22,9 @@
 #include "box64context.h"
 #include "my_cpuid.h"
 #include "bridge.h"
-#include "signals.h"
+#include "emit_signals.h"
 #include "x64shaext.h"
+#include "freq.h"
 #ifdef DYNAREC
 #include "custommem.h"
 #include "../dynarec/native_lock.h"
@@ -94,19 +96,19 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                 case 0xC8:  /* MONITOR */
                     // this is a privilege opcode...
                     #ifndef TEST_INTERPRETER
-                    emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
+                    EmitSignal(emu, X64_SIGSEGV, (void*)R_RIP, 0);
                     #endif
                     break;
                 case 0xC9:  /* MWAIT */
                     // this is a privilege opcode...
                     #ifndef TEST_INTERPRETER
-                    emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
+                    EmitSignal(emu, X64_SIGSEGV, (void*)R_RIP, 0);
                     #endif
                     break;
                 case 0xD0:
                     if(R_RCX) {
                         #ifndef TEST_INTERPRETER
-                        emit_signal(emu, SIGILL, (void*)R_RIP, 0);
+                        EmitSignal(emu, X64_SIGILL, (void*)R_RIP, 0);
                         #endif
                     } else {
                         R_RAX = 0b111;   // x87 & SSE & AVX for now
@@ -160,16 +162,30 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             }
             break;
 
+        case 0x03:                      /* LSL */
+            nextop = F8;
+            GETED(0);
+            GETGD;
+            CHECK_FLAGS(emu);
+            tmp8u = ED->word[0]>>3;
+            if (tmp8u>0x10 || !my_context->segtls[tmp8u].present) {
+                CLEAR_FLAG(F_ZF);
+            } else {
+                GD->dword[0] = my_context->segtls[tmp8u].limit;
+                SET_FLAG(F_ZF);
+            }
+            break;
+
         case 0x05:                      /* SYSCALL */
             #ifndef TEST_INTERPRETER
             R_RIP = addr;
-            x64Syscall(emu);
+            EmuX64Syscall(emu);
             #endif
             break;
         case 0x06:                      /* CLTS */
             // this is a privilege opcode...
             #ifndef TEST_INTERPRETER
-            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
+            EmitSignal(emu, X64_SIGSEGV, (void*)R_RIP, 0);
             #endif
             break;
 
@@ -177,13 +193,13 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
         case 0x09:                      /* WBINVD */
             // this is a privilege opcode...
             #ifndef TEST_INTERPRETER
-            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
+            EmitSignal(emu, X64_SIGSEGV, (void*)R_RIP, 0);
             #endif
             break;
 
         case 0x0B:                      /* UD2 */
             #ifndef TEST_INTERPRETER
-            emit_signal(emu, SIGILL, (void*)R_RIP, 0);
+            EmitSignal(emu, X64_SIGILL, (void*)R_RIP, 0);
             #endif
             break;
 
@@ -206,7 +222,7 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             break;
         case 0x0E:                      /* FEMMS */
             #ifndef TEST_INTERPRETER
-            emit_signal(emu, SIGILL, (void*)R_RIP, 0);
+            EmitSignal(emu, X64_SIGILL, (void*)R_RIP, 0);
             #endif
             break;
 
@@ -226,16 +242,18 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             nextop = F8;
             GETEX(0);
             GETGX;
-            if(MODREG)    /* MOVHLPS Gx,Ex */
+            if(MODREG)    /* MOVHLPS Gx, Ex */
                 GX->q[0] = EX->q[1];
             else
-                GX->q[0] = EX->q[0];    /* MOVLPS Gx,Ex */
+                GX->q[0] = EX->q[0];    /* MOVLPS Gx, Ex */
             break;
-        case 0x13:                      /* MOVLPS Ex,Gx */
+        case 0x13:                      /* MOVLPS Ex, Gx */
             nextop = F8;
-            GETEX(0);
-            GETGX;
-            EX->q[0] = GX->q[0];
+            if(!MODREG) {
+                GETEX(0);
+                GETGX;
+                EX->q[0] = GX->q[0];
+            }
             break;
         case 0x14:                      /* UNPCKLPS Gx, Ex */
             nextop = F8;
@@ -284,6 +302,11 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             break;
 
         case 0x19:
+        case 0x1A:
+        case 0x1B:
+        case 0x1C:
+        case 0x1D:
+        case 0x1E:
         case 0x1F:                      /* NOP (multi-byte) */
             nextop = F8;
             FAKEED(0);
@@ -294,7 +317,7 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
         case 0x23:                      /* MOV drX, REG */
             // this is a privilege opcode...
             #ifndef TEST_INTERPRETER
-            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
+            EmitSignal(emu, X64_SIGSEGV, (void*)R_RIP, 0);
             #endif
             break;
 
@@ -381,7 +404,7 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             nextop = F8;
             GETEX(0);
             GETGX;
-            if(isnan(GX->f[0]) || isnan(EX->f[0])) {
+            if(isnanf(GX->f[0]) || isnanf(EX->f[0])) {
                 SET_FLAG(F_ZF); SET_FLAG(F_PF); SET_FLAG(F_CF);
             } else if(isgreater(GX->f[0], EX->f[0])) {
                 CLEAR_FLAG(F_ZF); CLEAR_FLAG(F_PF); CLEAR_FLAG(F_CF);
@@ -395,7 +418,7 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
         case 0x30:                      /* WRMSR */
             // this is a privilege opcode...
             #ifndef TEST_INTERPRETER
-            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
+            EmitSignal(emu, X64_SIGSEGV, (void*)R_RIP, 0);
             #endif
             break;
         case 0x31:                   /* RDTSC */
@@ -408,20 +431,20 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
         case 0x32:                   /* RDMSR */
             // priviledge instruction
             #ifndef TEST_INTERPRETER
-            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0xbad0);
+            EmitSignal(emu, X64_SIGSEGV, (void*)R_RIP, 0xbad0);
             STEP;
             #endif
             break;
 
         case 0x34:                  /* SYSENTER */
             #ifndef TEST_INTERPRETER
-            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0xbad0);
+            EmitSignal(emu, X64_SIGSEGV, (void*)R_RIP, 0xbad0);
             STEP;
             #endif
             break;
         case 0x35:                  /* SYSEXIT */
             #ifndef TEST_INTERPRETER
-            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0xbad0);
+            EmitSignal(emu, X64_SIGSEGV, (void*)R_RIP, 0xbad0);
             STEP;
             #endif
             break;
@@ -572,7 +595,7 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                     GETEM(0);
                     GETGM;
                     for (int i=0; i<8; ++i) {
-                        GM->sb[i] = abs(EM->sb[i]);
+                        GM->ub[i] = abs(EM->sb[i]);
                     }
                     break;
                 case 0x1D:  /* PABSW Gm, Em */
@@ -580,7 +603,7 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                     GETEM(0);
                     GETGM;
                     for (int i=0; i<4; ++i) {
-                        GM->sw[i] = abs(EM->sw[i]);
+                        GM->uw[i] = abs(EM->sw[i]);
                     }
                     break;
                 case 0x1E:  /* PABSD Gm, Em */
@@ -588,7 +611,7 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                     GETEM(0);
                     GETGM;
                     for (int i=0; i<2; ++i) {
-                        GM->sd[i] = abs(EM->sd[i]);
+                        GM->ud[i] = abs(EM->sd[i]);
                     }
                     break;
 
@@ -695,7 +718,7 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
 
         case 0x3F:
             #ifndef TEST_INTERPRETER
-            emit_signal(emu, SIGILL, (void*)R_RIP, 0);
+            EmitSignal(emu, X64_SIGILL, (void*)R_RIP, 0);
             #endif
             break;
         GOCOND(0x40
@@ -721,7 +744,10 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             GETEX(0);
             GETGX;
             for(int i=0; i<4; ++i)
-                GX->f[i] = sqrtf(EX->f[i]);
+                if (isnan(EX->f[i]))
+                    GX->f[i] = EX->f[i];
+                else
+                    GX->f[i] = (EX->f[i] < 0) ? (-NAN) : sqrtf(EX->f[i]);
             break;
         case 0x52:                      /* RSQRTPS Gx, Ex */
             nextop = F8;
@@ -745,21 +771,22 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             GETEX(0);
             GETGX;
             for(int i=0; i<4; ++i)
-                GX->f[i] = 1.0f/EX->f[i];
+                if (isnan(EX->f[i]))
+                    GX->f[i] = EX->f[i];
+                else
+                    GX->f[i] = 1.0f / EX->f[i];
             break;
         case 0x54:                      /* ANDPS Gx, Ex */
             nextop = F8;
             GETEX(0);
             GETGX;
-            for(int i=0; i<4; ++i)
-                GX->ud[i] &= EX->ud[i];
+            GX->u128 &= EX->u128;
             break;
         case 0x55:                      /* ANDNPS Gx, Ex */
             nextop = F8;
             GETEX(0);
             GETGX;
-            for(int i=0; i<4; ++i)
-                GX->ud[i] = (~GX->ud[i]) & EX->ud[i];
+            GX->u128 = (~GX->u128) & EX->u128;
             break;
         case 0x56:                      /* ORPS Gx, Ex */
             nextop = F8;
@@ -826,7 +853,7 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             GETEX(0);
             GETGX;
             for(int i=0; i<4; ++i) {
-                if (isnan(GX->f[i]) || isnan(EX->f[i]) || isless(EX->f[i], GX->f[i]))
+                if (isnan(GX->f[i]) || isnan(EX->f[i]) || islessequal(EX->f[i], GX->f[i]))
                     GX->f[i] = EX->f[i];
             }
             break;
@@ -845,7 +872,7 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             GETEX(0);
             GETGX;
             for(int i=0; i<4; ++i) {
-                if (isnan(GX->f[i]) || isnan(EX->f[i]) || isgreater(EX->f[i], GX->f[i]))
+                if (isnan(GX->f[i]) || isnan(EX->f[i]) || isgreaterequal(EX->f[i], GX->f[i]))
                     GX->f[i] = EX->f[i];
             }
             break;
@@ -1125,7 +1152,7 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
         GOCOND(0x80
             , tmp32s = F32S; CHECK_FLAGS(emu);
             , addr += tmp32s;
-            ,,
+            ,,STEP3
         )                               /* 0x80 -> 0x8F Jxx */ //STEP3
         GOCOND(0x90
             , nextop = F8; CHECK_FLAGS(emu);
@@ -1391,7 +1418,6 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                         ED->q[0] = GD->dword[0];
                     else
                         ED->dword[0] = GD->dword[0];
-                    R_RAX = R_EAX;   // to erase upper part of RAX
                 } else {
                     R_RAX = ED->dword[0];
                 }
@@ -1588,26 +1614,34 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             nextop = F8;
             GETED(0);
             GETGD;
+            tmp8u = 0;
             if(rex.w) {
                 tmp64u = ED->q[0];
                 if(tmp64u) {
                     CLEAR_FLAG(F_ZF);
-                    tmp8u = 0;
                     while(!(tmp64u&(1LL<<tmp8u))) ++tmp8u;
-                    GD->q[0] = tmp8u;
                 } else {
                     SET_FLAG(F_ZF);
                 }
+                if(tmp64u || !MODREG)
+                    GD->q[0] = tmp8u;
             } else {
                 tmp32u = ED->dword[0];
                 if(tmp32u) {
                     CLEAR_FLAG(F_ZF);
-                    tmp8u = 0;
                     while(!(tmp32u&(1<<tmp8u))) ++tmp8u;
-                    GD->q[0] = tmp8u;
                 } else {
                     SET_FLAG(F_ZF);
                 }
+                if(tmp32u || !MODREG)
+                    GD->q[0] = tmp8u;
+            }
+            if(!BOX64ENV(cputype)) {
+                CONDITIONAL_SET_FLAG(PARITY(tmp8u), F_PF);
+                CLEAR_FLAG(F_CF);
+                CLEAR_FLAG(F_AF);
+                CLEAR_FLAG(F_SF);
+                CLEAR_FLAG(F_OF);
             }
             break;
         case 0xBD:                      /* BSR Ed,Gd */
@@ -1615,16 +1649,18 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             nextop = F8;
             GETED(0);
             GETGD;
+            tmp8u = 0;
             if(rex.w) {
                 tmp64u = ED->q[0];
                 if(tmp64u) {
                     CLEAR_FLAG(F_ZF);
                     tmp8u = 63;
                     while(!(tmp64u&(1LL<<tmp8u))) --tmp8u;
-                    GD->q[0] = tmp8u;
                 } else {
                     SET_FLAG(F_ZF);
                 }
+                if(tmp64u || !MODREG)
+                    GD->q[0] = tmp8u;
             } else {
                 tmp32u = ED->dword[0];
                 if(tmp32u) {
@@ -1634,8 +1670,17 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                     GD->q[0] = tmp8u;
                 } else {
                     SET_FLAG(F_ZF);
+                    if(!MODREG)
+                        GD->q[0] = tmp8u;
                 }
             }
+            if(!BOX64ENV(cputype)) {
+                CONDITIONAL_SET_FLAG(PARITY(tmp8u), F_PF);
+                CLEAR_FLAG(F_CF);
+                CLEAR_FLAG(F_AF);
+                CLEAR_FLAG(F_SF);
+                CLEAR_FLAG(F_OF);
+            }
             break;
         case 0xBE:                      /* MOVSX Gd,Eb */
             nextop = F8;
@@ -1756,8 +1801,8 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                         tmp64u2= ED->q[1];
                         if(R_RAX == tmp64u && R_RDX == tmp64u2) {
                             SET_FLAG(F_ZF);
-                            ED->q[0] = R_EBX;
-                            ED->q[1] = R_ECX;
+                            ED->q[0] = R_RBX;
+                            ED->q[1] = R_RCX;
                         } else {
                             CLEAR_FLAG(F_ZF);
                             R_RAX = tmp64u;
@@ -1954,14 +1999,12 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             nextop = F8;
             GETEM(0);
             GETGM;
-            if(EM->q>31) {
-                for(int i=0; i<2; ++i)
-                    GM->sd[i] = (GM->sd[i]<0)?-1:0;
-            } else {
+            if(EM->q>31)
+                tmp8u = 31;
+            else
                 tmp8u = EM->ub[0];
-                for(int i=0; i<2; ++i)
-                    GM->sd[i] >>= tmp8u;
-            }
+            for(int i=0; i<2; ++i)
+                GM->sd[i] >>= tmp8u;
             break;
         case 0xE3:                   /* PAVGW Gm, Em */
             nextop = F8;
@@ -2068,7 +2111,7 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             else {
                 tmp8u = EM->ub[0];
                 for(int i=0; i<4; ++i)
-                    GM->sw[i] <<= tmp8u;
+                    GM->uw[i] <<= tmp8u;
             }
             break;
         case 0xF2:                   /* PSLLD Gm, Em */
@@ -2080,7 +2123,7 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             else {
                 tmp8u = EM->ub[0];
                 for(int i=0; i<2; ++i)
-                    GM->sd[i] <<= tmp8u;
+                    GM->ud[i] <<= tmp8u;
             }
             break;
         case 0xF3:                   /* PSLLQ Gm, Em */
