@@ -12,7 +12,6 @@
 #include "debug.h"
 #include "box64stack.h"
 #include "x64emu.h"
-#include "x64run.h"
 #include "x64emu_private.h"
 #include "x64run_private.h"
 #include "x64primop.h"
@@ -21,7 +20,6 @@
 #include "box64context.h"
 #include "my_cpuid.h"
 #include "bridge.h"
-#include "signals.h"
 #include "x64shaext.h"
 #ifdef DYNAREC
 #include "custommem.h"
@@ -49,6 +47,7 @@ uintptr_t RunAVX_F20F(x64emu_t *emu, vex_t vex, uintptr_t addr, int *step)
     sse_regs_t *opex, *opgx, *opvx, eax1;
     sse_regs_t *opey, *opgy, *opvy, eay1;
     int is_nan;
+    int mask_nan[4];
 
 
 #ifdef TEST_INTERPRETER
@@ -116,7 +115,7 @@ uintptr_t RunAVX_F20F(x64emu_t *emu, vex_t vex, uintptr_t addr, int *step)
 
         case 0x2C:  /* VCVTTSD2SI Gd, Ex */
             nextop = F8;
-            GETEX(0);
+            _GETEX(0);
             GETGD;
             if(rex.w)
                 if(isnan(EX->d[0]) || isinf(EX->d[0]) || EX->d[0]>=(double)0x7fffffffffffffffLL)
@@ -133,7 +132,7 @@ uintptr_t RunAVX_F20F(x64emu_t *emu, vex_t vex, uintptr_t addr, int *step)
             break;
         case 0x2D:  /* VCVTSD2SI Gd, Ex */
             nextop = F8;
-            GETEX(0);
+            _GETEX(0);
             GETGD;
             if(rex.w) {
                 if(isnan(EX->d[0]) || isinf(EX->d[0]) || EX->d[0]>=(double)0x7fffffffffffffffLL)
@@ -189,6 +188,8 @@ uintptr_t RunAVX_F20F(x64emu_t *emu, vex_t vex, uintptr_t addr, int *step)
             GETGX; GETVX; GETGY;
             if(EX->d[0]<0.0 )
                 GX->d[0] = -NAN;
+            else if(isnan(EX->d[0]))
+                GX->d[0] = EX->d[0];
             else
                 GX->d[0] = sqrt(EX->d[0]);
             GX->q[1] = VX->q[1];
@@ -197,27 +198,25 @@ uintptr_t RunAVX_F20F(x64emu_t *emu, vex_t vex, uintptr_t addr, int *step)
 
         case 0x58:  /* VADDSD Gx, Vx, Ex */
             nextop = F8;
-            GETEX(0);
+            _GETEX(0);
             GETGX;
             GETVX;
             GETGY;
+            MARK_NAN_D_2(VX, EX);
             GX->d[0] = VX->d[0] + EX->d[0];
+            CHECK_NAN_D(GX);
             GX->q[1] = VX->q[1];
             GY->u128 = 0;
             break;
         case 0x59:  /* VMULSD Gx, Vx, Ex */
             nextop = F8;
-            GETEX(0);
+            _GETEX(0);
             GETGX;
             GETVX;
             GETGY;
-            #ifndef NOALIGN
-                // mul generate a -NAN only if doing (+/-)inf * (+/-)0
-                if((isinf(GX->d[0]) && EX->d[0]==0.0) || (isinf(EX->d[0]) && GX->d[0]==0.0))
-                    GX->d[0] = -NAN;
-                else
-            #endif
+            MARK_NAN_D_2(VX, EX);
             GX->d[0] = VX->d[0] * EX->d[0];
+            CHECK_NAN_D(GX);
             if(GX!=VX) {
                 GX->q[1] = VX->q[1];
             }
@@ -225,7 +224,7 @@ uintptr_t RunAVX_F20F(x64emu_t *emu, vex_t vex, uintptr_t addr, int *step)
             break;
         case 0x5A:  /* VCVTSD2SS Gx, Vx, Ex */
             nextop = F8;
-            GETEX(0);
+            _GETEX(0);
             GETGX;
             GETVX;
             GETGY;
@@ -237,7 +236,7 @@ uintptr_t RunAVX_F20F(x64emu_t *emu, vex_t vex, uintptr_t addr, int *step)
 
         case 0x5C:  /* VSUBSD Gx, Vx, Ex */
             nextop = F8;
-            GETEX(0);
+            _GETEX(0);
             GETGX;
             GETVX;
             GETGY;
@@ -251,9 +250,7 @@ uintptr_t RunAVX_F20F(x64emu_t *emu, vex_t vex, uintptr_t addr, int *step)
             GETGX;
             GETVX;
             GETGY;
-            if (VX->d[0] == 0.0 && EX->d[0]  == 0.0)
-                GX->d[0] = EX->d[0];
-            else if (isnan(VX->d[0]) || isnan(EX->d[0]) || isgreater(VX->d[0], EX->d[0]))
+            if (isnan(VX->d[0]) || isnan(EX->d[0]) || islessequal(EX->d[0], VX->d[0]))
                 GX->d[0] = EX->d[0];
             else
                 GX->d[0] = VX->d[0];
@@ -283,9 +280,7 @@ uintptr_t RunAVX_F20F(x64emu_t *emu, vex_t vex, uintptr_t addr, int *step)
             GETGX;
             GETVX;
             GETGY;
-            if (VX->d[0] == 0.0 && EX->d[0]  == 0.0)
-                GX->d[0] = EX->d[0];
-            else if (isnan(VX->d[0]) || isnan(EX->d[0]) || isgreater(EX->d[0], VX->d[0]))
+            if (isnan(VX->d[0]) || isnan(EX->d[0]) || isgreaterequal(EX->d[0], VX->d[0]))
                 GX->d[0] = EX->d[0];
             else
                 GX->d[0] = VX->d[0];
@@ -333,30 +328,44 @@ uintptr_t RunAVX_F20F(x64emu_t *emu, vex_t vex, uintptr_t addr, int *step)
                 eax1 = *EX;
                 EX = &eax1;
             }
-            GX->f[0] = VX->f[0] + VX->f[1];
-            GX->f[1] = VX->f[2] + VX->f[3];
+            mask_nan[0] = isnanf(VX->f[0]) || isnanf(VX->f[1]);
+            mask_nan[1] = isnanf(VX->f[2]) || isnanf(VX->f[3]);
+            GX->f[0] = isnan(VX->f[0]) ? VX->f[0] : (isnan(VX->f[1]) ? VX->f[1] : (VX->f[0] + VX->f[1]));
+            GX->f[1] = isnan(VX->f[2]) ? VX->f[2] : (isnan(VX->f[3]) ? VX->f[3] : (VX->f[2] + VX->f[3]));
             if(EX==VX) {
                 GX->f[2] = GX->f[0];
                 GX->f[3] = GX->f[1];
+                mask_nan[2] = mask_nan[0];
+                mask_nan[3] = mask_nan[1];
             } else {
-                GX->f[2] = EX->f[0] + EX->f[1];
-                GX->f[3] = EX->f[2] + EX->f[3];
+                mask_nan[2] = isnanf(EX->f[0]) || isnanf(EX->f[1]);
+                mask_nan[3] = isnanf(EX->f[2]) || isnanf(EX->f[3]);
+                GX->f[2] = isnan(EX->f[0]) ? EX->f[0] : (isnan(EX->f[1]) ? EX->f[1] : (EX->f[0] + EX->f[1]));
+                GX->f[3] = isnan(EX->f[2]) ? EX->f[2] : (isnan(EX->f[3]) ? EX->f[3] : (EX->f[2] + EX->f[3]));
             }
+            CHECK_NAN_VF(GX);
             if(vex.l) {
                 if(GY==EY) {
                     eay1 = *EY;
                     EY = &eay1;
                 }
                 GETVY;
-                GY->f[0] = VY->f[0] + VY->f[1];
-                GY->f[1] = VY->f[2] + VY->f[3];
+                mask_nan[0] = isnanf(VY->f[0]) || isnanf(VY->f[1]);
+                mask_nan[1] = isnanf(VY->f[2]) || isnanf(VY->f[3]);
+                GY->f[0] = isnan(VY->f[0]) ? VY->f[0] : (isnan(VY->f[1]) ? VY->f[1] : (VY->f[0] + VY->f[1]));
+                GY->f[1] = isnan(VY->f[2]) ? VY->f[2] : (isnan(VY->f[3]) ? VY->f[3] : (VY->f[2] + VY->f[3]));
                 if(EY==VY) {
                     GY->f[2] = GY->f[0];
                     GY->f[3] = GY->f[1];
+                    mask_nan[2] = mask_nan[0];
+                    mask_nan[3] = mask_nan[1];
                 } else {
-                    GY->f[2] = EY->f[0] + EY->f[1];
-                    GY->f[3] = EY->f[2] + EY->f[3];
+                    mask_nan[2] = isnanf(EY->f[0]) || isnanf(EY->f[1]);
+                    mask_nan[3] = isnanf(EY->f[2]) || isnanf(EY->f[3]);
+                    GY->f[2] = isnan(EY->f[0]) ? EY->f[0] : (isnan(EY->f[1]) ? EY->f[1] : (EY->f[0] + EY->f[1]));
+                    GY->f[3] = isnan(EY->f[2]) ? EY->f[2] : (isnan(EY->f[3]) ? EY->f[3] : (EY->f[2] + EY->f[3]));
                 }
+                CHECK_NAN_VF(GY);
             } else
                 GY->u128 = 0;
             break;
@@ -367,38 +376,51 @@ uintptr_t RunAVX_F20F(x64emu_t *emu, vex_t vex, uintptr_t addr, int *step)
             GETVX;
             GETGY;
             GETEY;
-            if(GX==EX) {
+            if (GX == EX) {
                 eax1 = *EX;
                 EX = &eax1;
             }
-            GX->f[0] = VX->f[0] - VX->f[1];
-            GX->f[1] = VX->f[2] - VX->f[3];
-            if(EX==VX) {
+            mask_nan[0] = isnanf(VX->f[0]) || isnanf(VX->f[1]);
+            mask_nan[1] = isnanf(VX->f[2]) || isnanf(VX->f[3]);
+            GX->f[0] = isnan(VX->f[0]) ? VX->f[0] : (isnan(VX->f[1]) ? VX->f[1] : (VX->f[0] - VX->f[1]));
+            GX->f[1] = isnan(VX->f[2]) ? VX->f[2] : (isnan(VX->f[3]) ? VX->f[3] : (VX->f[2] - VX->f[3]));
+            if (EX == VX) {
                 GX->f[2] = GX->f[0];
                 GX->f[3] = GX->f[1];
+                mask_nan[2] = mask_nan[0];
+                mask_nan[3] = mask_nan[1];
             } else {
-                GX->f[2] = EX->f[0] - EX->f[1];
-                GX->f[3] = EX->f[2] - EX->f[3];
+                mask_nan[2] = isnanf(EX->f[0]) || isnanf(EX->f[1]);
+                mask_nan[3] = isnanf(EX->f[2]) || isnanf(EX->f[3]);
+                GX->f[2] = isnan(EX->f[0]) ? EX->f[0] : (isnan(EX->f[1]) ? EX->f[1] : (EX->f[0] - EX->f[1]));
+                GX->f[3] = isnan(EX->f[2]) ? EX->f[2] : (isnan(EX->f[3]) ? EX->f[3] : (EX->f[2] - EX->f[3]));
             }
-            if(vex.l) {
-                if(GY==EY) {
+            CHECK_NAN_VF(GX);
+            if (vex.l) {
+                if (GY == EY) {
                     eay1 = *EY;
                     EY = &eay1;
                 }
                 GETVY;
-                GY->f[0] = VY->f[0] - VY->f[1];
-                GY->f[1] = VY->f[2] - VY->f[3];
-                if(EY==VY) {
+                mask_nan[0] = isnanf(VY->f[0]) || isnanf(VY->f[1]);
+                mask_nan[1] = isnanf(VY->f[2]) || isnanf(VY->f[3]);
+                GY->f[0] = isnan(VY->f[0]) ? VY->f[0] : (isnan(VY->f[1]) ? VY->f[1] : (VY->f[0] - VY->f[1]));
+                GY->f[1] = isnan(VY->f[2]) ? VY->f[2] : (isnan(VY->f[3]) ? VY->f[3] : (VY->f[2] - VY->f[3]));
+                if (EY == VY) {
                     GY->f[2] = GY->f[0];
                     GY->f[3] = GY->f[1];
+                    mask_nan[2] = mask_nan[0];
+                    mask_nan[3] = mask_nan[1];
                 } else {
-                    GY->f[2] = EY->f[0] - EY->f[1];
-                    GY->f[3] = EY->f[2] - EY->f[3];
+                    mask_nan[2] = isnanf(EY->f[0]) || isnanf(EY->f[1]);
+                    mask_nan[3] = isnanf(EY->f[2]) || isnanf(EY->f[3]);
+                    GY->f[2] = isnan(EY->f[0]) ? EY->f[0] : (isnan(EY->f[1]) ? EY->f[1] : (EY->f[0] - EY->f[1]));
+                    GY->f[3] = isnan(EY->f[2]) ? EY->f[2] : (isnan(EY->f[3]) ? EY->f[3] : (EY->f[2] - EY->f[3]));
                 }
+                CHECK_NAN_VF(GY);
             } else
                 GY->u128 = 0;
             break;
-
         case 0xC2:  /* VCMPSD Gx, Vx, Ex, Ib */
             nextop = F8;
             GETEX(1);
@@ -438,17 +460,21 @@ uintptr_t RunAVX_F20F(x64emu_t *emu, vex_t vex, uintptr_t addr, int *step)
             GETGX;
             GETVX;
             GETGY;
-            GX->f[0] = VX->f[0] - EX->f[0];
-            GX->f[1] = VX->f[1] + EX->f[1];
-            GX->f[2] = VX->f[2] - EX->f[2];
-            GX->f[3] = VX->f[3] + EX->f[3];
+            MARK_NAN_VF_2(VX, EX);
+            GX->f[0] = isnan(VX->f[0]) ? VX->f[0] : (isnan(EX->f[0]) ? EX->f[0] : (VX->f[0] - EX->f[0]));
+            GX->f[1] = isnan(VX->f[1]) ? VX->f[1] : (isnan(EX->f[1]) ? EX->f[1] : (VX->f[1] + EX->f[1]));
+            GX->f[2] = isnan(VX->f[2]) ? VX->f[2] : (isnan(EX->f[2]) ? EX->f[2] : (VX->f[2] - EX->f[2]));
+            GX->f[3] = isnan(VX->f[3]) ? VX->f[3] : (isnan(EX->f[3]) ? EX->f[3] : (VX->f[3] + EX->f[3]));
+            CHECK_NAN_VF(GX);
             if(vex.l) {
                 GETEY;
                 GETVY;
-                GY->f[0] = VY->f[0] - EY->f[0];
-                GY->f[1] = VY->f[1] + EY->f[1];
-                GY->f[2] = VY->f[2] - EY->f[2];
-                GY->f[3] = VY->f[3] + EY->f[3];
+                MARK_NAN_VF_2(VY, EY);
+                GY->f[0] = isnan(VY->f[0]) ? VY->f[0] : (isnan(EY->f[0]) ? EY->f[0] : (VY->f[0] - EY->f[0]));
+                GY->f[1] = isnan(VY->f[1]) ? VY->f[1] : (isnan(EY->f[1]) ? EY->f[1] : (VY->f[1] + EY->f[1]));
+                GY->f[2] = isnan(VY->f[2]) ? VY->f[2] : (isnan(EY->f[2]) ? EY->f[2] : (VY->f[2] - EY->f[2]));
+                GY->f[3] = isnan(VY->f[3]) ? VY->f[3] : (isnan(EY->f[3]) ? EY->f[3] : (VY->f[3] + EY->f[3]));
+                CHECK_NAN_VF(GY);
             } else
                 GY->u128 = 0;
             break;
