@@ -12,7 +12,6 @@
 #include "debug.h"
 #include "box64stack.h"
 #include "x64emu.h"
-#include "x64run.h"
 #include "x64emu_private.h"
 #include "x64run_private.h"
 #include "x64primop.h"
@@ -25,9 +24,9 @@
 #include "modrm.h"
 
 #ifdef TEST_INTERPRETER
-uintptr_t TestD9(x64test_t *test, rex_t rex, uintptr_t addr, uintptr_t offs)
+uintptr_t TestD9(x64test_t *test, rex_t rex, uintptr_t addr)
 #else
-uintptr_t RunD9(x64emu_t *emu, rex_t rex, uintptr_t addr, uintptr_t offs)
+uintptr_t RunD9(x64emu_t *emu, rex_t rex, uintptr_t addr)
 #endif
 {
     uint8_t nextop;
@@ -169,48 +168,14 @@ uintptr_t RunD9(x64emu_t *emu, rex_t rex, uintptr_t addr, uintptr_t offs)
             }
             break;
 
-        case 0xF8:  /* FPREM */
-            {
-                int e0, e1;
-                frexp(ST0.d, &e0);
-                frexp(ST1.d, &e1);
-                tmp32s = e0 - e1;
-            }
-            if(tmp32s<64)
-            {
-                ll = (int64_t)floor(ST0.d/ST1.d);
-                ST0.d = ST0.d - (ST1.d*ll);
-                emu->sw.f.F87_C2 = 0;
-                emu->sw.f.F87_C1 = (ll&1)?1:0;
-                emu->sw.f.F87_C3 = (ll&2)?1:0;
-                emu->sw.f.F87_C0 = (ll&4)?1:0;
-            } else {
-                ll = (int64_t)(floor((ST0.d/ST1.d))/exp2(tmp32s - 32));
-                ST0.d = ST0.d - ST1.d*ll*exp2(tmp32s - 32);
-                emu->sw.f.F87_C2 = 1;
-            }
-            break;
         case 0xF5:  /* FPREM1 */
             // get exponant(ST(0))-exponant(ST(1)) in temp32s
-            {
-                int e0, e1;
-                frexp(ST0.d, &e0);
-                frexp(ST1.d, &e1);
-                tmp32s = e0 - e1;
-            }
-            if(tmp32s<64)
-            {
-                ll = (int64_t)round(ST0.d/ST1.d);
-                ST0.d = ST0.d - (ST1.d*ll);
-                emu->sw.f.F87_C2 = 0;
-                emu->sw.f.F87_C1 = (ll&1)?1:0;
-                emu->sw.f.F87_C3 = (ll&2)?1:0;
-                emu->sw.f.F87_C0 = (ll&4)?1:0;
-            } else {
-                ll = (int64_t)(trunc((ST0.d/ST1.d))/exp2(tmp32s - 32));
-                ST0.d = ST0.d - ST1.d*ll*exp2(tmp32s - 32);
-                emu->sw.f.F87_C2 = 1;
-            }
+            ll = (int64_t)round(ST0.d/ST1.d);
+            ST0.d = ST0.d - (ST1.d*ll);
+            emu->sw.f.F87_C2 = 0;
+            emu->sw.f.F87_C1 = (ll&1)?1:0;
+            emu->sw.f.F87_C3 = (ll&2)?1:0;
+            emu->sw.f.F87_C0 = (ll&4)?1:0;
             break;
         case 0xF6:  /* FDECSTP */
             emu->top=(emu->top-1)&7;    // this will probably break a few things
@@ -221,6 +186,14 @@ uintptr_t RunD9(x64emu_t *emu, rex_t rex, uintptr_t addr, uintptr_t offs)
             else
                 emu->top=(emu->top+1)&7;    // this will probably break a few things
             break;
+        case 0xF8:  /* FPREM */
+            ll = (int64_t)trunc(ST0.d/ST1.d);
+            ST0.d = ST0.d - (ST1.d*ll);
+            emu->sw.f.F87_C2 = 0;
+            emu->sw.f.F87_C1 = (ll&1)?1:0;
+            emu->sw.f.F87_C3 = (ll&2)?1:0;
+            emu->sw.f.F87_C0 = (ll&4)?1:0;
+            break;
         case 0xF9:  /* FYL2XP1 */
             // Using the log1p instead of log2(ST0+1) can avoid losing precision much,
             // expecially when ST0 is close to zero (which loses the precise when +1).
@@ -231,6 +204,7 @@ uintptr_t RunD9(x64emu_t *emu, rex_t rex, uintptr_t addr, uintptr_t offs)
         case 0xFA:  /* FSQRT */
             oldround = fpu_setround(emu);
             ST0.d = sqrt(ST0.d);
+            if(!emu->cw.f.C87_PC) ST0.d = (float)ST0.d;
             fesetround(oldround);
             break;
         case 0xFB:  /* FSINCOS */
@@ -276,30 +250,18 @@ uintptr_t RunD9(x64emu_t *emu, rex_t rex, uintptr_t addr, uintptr_t offs)
     } else
         switch((nextop>>3)&7) {
             case 0:     /* FLD ST0, Ed float */
-                if(offs) {
-                    GETE4_OFFS(0, offs);
-                } else {
-                    GETE4(0);
-                }
+                GETE4(0);
                 fpu_do_push(emu);
                 ST0.d = *(float*)ED;
                 break;
             case 2:     /* FST Ed, ST0 */
-                if(offs) {
-                    GETE4_OFFS(0, offs);
-                } else {
-                    GETE4(0);
-                }
+                GETE4(0);
                 oldround = fpu_setround(emu);
                 *(float*)ED = ST0.d;
                 fesetround(oldround);
                 break;
             case 3:     /* FSTP Ed, ST0 */
-                if(offs) {
-                    GETE4_OFFS(0, offs);
-                } else {
-                    GETE4(0);
-                }
+                GETE4(0);
                 oldround = fpu_setround(emu);
                 *(float*)ED = ST0.d;
                 fesetround(oldround);
@@ -307,44 +269,28 @@ uintptr_t RunD9(x64emu_t *emu, rex_t rex, uintptr_t addr, uintptr_t offs)
                 break;
             case 4:     /* FLDENV m */
                 // warning, incomplete
-                if(offs) {
-                    _GETED_OFFS(0, offs);
-                } else {
-                    _GETED(0);
-                }
+                _GETED(0);
                 fpu_loadenv(emu, (char*)ED, 0);
                 break;
             case 5:     /* FLDCW Ew */
-                if(offs) {
-                    GETEW_OFFS(0, offs);
-                } else {
-                    GETEW(0);
-                }
+                GETEW(0);
                 emu->cw.x16 = EW->word[0];
                 // do something with cw?
                 break;
             case 6:     /* FNSTENV m */
                 // warning, incomplete
-                if(offs) {
-                    GETE8_OFFS(0, offs);
-                } else {
-                    GETE8(0);
-                }
+                GETE8(0);
                 fpu_savenv(emu, (char*)ED, 0);
                 // intruction pointer: 48bits
                 // data (operand) pointer: 48bits
                 // last opcode: 11bits save: 16bits restaured (1st and 2nd opcode only)
                 break;
             case 7: /* FNSTCW Ew */
-                if(offs) {
-                    GETEW_OFFS(0, offs);
-                } else {
-                    GETEW(0);
-                }
+                GETEW(0);
                 EW->word[0] = emu->cw.x16;
                 break;
             default:
                 return 0;
         }
     return addr;
-}
\ No newline at end of file
+}
