@@ -1,7 +1,8 @@
 #ifndef __BOX64CONTEXT_H_
 #define __BOX64CONTEXT_H_
 #include <stdint.h>
-#include <pthread.h>
+
+#include "mypthread.h"
 #include "pathcoll.h"
 #include "dictionnary.h"
 #ifdef DYNAREC
@@ -52,7 +53,11 @@ typedef struct kh_dynablocks_s  kh_dynablocks_t;
 typedef void* (*procaddress_t)(const char* name);
 typedef void* (*vkprocaddess_t)(void* instance, const char* name);
 
+#ifdef LA64_ABI_1
+#define MAX_SIGNAL 128
+#else
 #define MAX_SIGNAL 64
+#endif
 
 typedef struct tlsdatasize_s {
     int         tlssize;
@@ -134,7 +139,7 @@ typedef struct box64context_s {
     uintptr_t           ep;             // entry point
 
     lib_t               *maplib;        // lib and symbols handling
-    lib_t               *local_maplib;  // libs and symbols openned has local (only collection of libs, no symbols)
+    lib_t               *local_maplib;  // libs and symbols opened has local (only collection of libs, no symbols)
     dic_t               *versym;        // dictionnary of versioned symbols
     kh_mapsymbols_t     *globdata;      // GLOBAL_DAT relocation for COPY mapping in main elf
     kh_mapsymbols_t     *uniques;       // symbols with STB_GNU_UNIQUE bindings
@@ -230,21 +235,25 @@ typedef struct box64context_s {
     int                 stack_clone_used;
 
     // rolling logs
-    char*               *log_call;
-    char*               *log_ret;
+    char*               log_call;   // is a large string composed of slices of 256 chars
+    char*               log_ret;    // is a large string composed of sloces of 128 chars
     int                 current_line;
 
 } box64context_t;
 
 #ifndef USE_CUSTOM_MUTEX
-#define mutex_lock(A)       pthread_mutex_lock(A)
-#define mutex_trylock(A)    pthread_mutex_trylock(A)
-#define mutex_unlock(A)     pthread_mutex_unlock(A)
+#define mutex_lock(A)    pthread_mutex_lock(A)
+#define mutex_trylock(A) pthread_mutex_trylock(A)
+#define mutex_unlock(A)  pthread_mutex_unlock(A)
 #else
-int GetTID(void);
-#define mutex_lock(A)       {uint32_t tid = (uint32_t)GetTID(); while(native_lock_storeifnull_d(A, tid)) sched_yield();}
-#define mutex_trylock(A)    native_lock_storeifnull_d(A, (uint32_t)GetTID())
-#define mutex_unlock(A)     native_lock_storeifref_d(A, 0, (uint32_t)GetTID())
+#define mutex_lock(A)                             \
+    do {                                          \
+        uint32_t tid = (uint32_t)GetTID();        \
+        while (native_lock_storeifnull_d(A, tid)) \
+            sched_yield();                        \
+    } while (0)
+#define mutex_trylock(A) native_lock_storeifnull_d(A, (uint32_t)GetTID())
+#define mutex_unlock(A)  native_lock_storeifref_d(A, 0, (uint32_t)GetTID())
 #endif
 
 extern box64context_t *my_context; // global context
@@ -255,7 +264,7 @@ void FreeBox64Context(box64context_t** context);
 // Cycle log handling
 void freeCycleLog(box64context_t* ctx);
 void initCycleLog(box64context_t* context);
-void print_cycle_log(int loglevel);
+void print_rolling_log(int loglevel);
 
 // return the index of the added header
 int AddElfHeader(box64context_t* ctx, elfheader_t* head);
@@ -270,8 +279,6 @@ void thread_set_emu(x64emu_t* emu);
 void thread_forget_emu();
 x64emu_t* thread_get_emu(void);
 
-// unlock mutex that are locked by current thread (for signal handling). Return a mask of unlock mutex
-int unlockMutex(void);
 // relock the muxtex that were unlocked
 void relockMutex(int locks);
 
