@@ -286,13 +286,13 @@ static void initWrappedLib(library_t *lib, box64context_t* context) {
             linkmap_t *lm = addLinkMapLib(lib);
             if(!lm) {
                 // Crashed already
-                printf_dump(LOG_DEBUG, "Failure to add lib %s linkmap\n", lib->name);
+                printf_dlsym_dump(LOG_DEBUG, "Failure to add lib %s linkmap\n", lib->name);
                 break;
             }
             struct link_map real_lm;
             #ifndef ANDROID
             if(dlinfo(lib->w.lib, RTLD_DI_LINKMAP, &real_lm)) {
-                printf_dump(LOG_DEBUG, "Failed to dlinfo lib %s\n", lib->name);
+                printf_dlsym_dump(LOG_DEBUG, "Failed to dlinfo lib %s\n", lib->name);
             }
             #endif
             lm->l_addr = real_lm.l_addr;
@@ -314,7 +314,7 @@ static int loadEmulatedLib(const char* libname, library_t *lib, box64context_t*
         }
         elfheader_t *elf_header = LoadAndCheckElfHeader(f, libname, 0);
         if(!elf_header) {
-            printf_dump(LOG_DEBUG, "Error: reading elf header of %s\n", libname);    // this one can be too alarming...
+            printf_dlsym_dump(LOG_DEBUG, "Error: reading elf header of %s\n", libname);    // this one can be too alarming...
             fclose(f);
             return 0;
         }
@@ -356,15 +356,18 @@ static int loadEmulatedLib(const char* libname, library_t *lib, box64context_t*
         }
 
         printf_dump(LOG_INFO, "Using emulated %s\n", libname);
+        int env_changed = 0;
         #ifdef DYNAREC
         if(libname && BOX64ENV(dynarec_bleeding_edge) && strstr(libname, "libmonobdwgc-2.0.so")) {
             printf_dump(LOG_INFO, "MonoBleedingEdge detected, disable Dynarec BigBlock and enable Dynarec StrongMem\n");
             SET_BOX64ENV(dynarec_bigblock, 0);
             SET_BOX64ENV(dynarec_strongmem, 1);
+            env_changed = 1;
         }
         if(libname && BOX64ENV(dynarec_tbb) && strstr(libname, "libtbb.so")) {
             printf_dump(LOG_INFO, "libtbb detected, enable Dynarec StrongMem\n");
             SET_BOX64ENV(dynarec_strongmem, 1);
+            env_changed = 1;
         }
         #endif
         if(libname && BOX64ENV(jvm) && strstr(libname, "libjvm.so")) {
@@ -376,11 +379,14 @@ static int loadEmulatedLib(const char* libname, library_t *lib, box64context_t*
             printf_dump(LOG_INFO, "libjvm detected, hide SSE 4.2\n");
             #endif
             SET_BOX64ENV(sse42, 0);
+            env_changed = 1;
         }
         if(libname && BOX64ENV(libcef) && strstr(libname, "libcef.so")) {
-            printf_dump(LOG_INFO, "libcef detected, using malloc_hack_2\n");
+            printf_dump(LOG_INFO, "libcef detected, using malloc_hack=2\n");
             SET_BOX64ENV(malloc_hack, 2);
+            env_changed = 1;
         }
+        if (env_changed) PrintEnvVariables(&box64env, LOG_INFO);
         return 1;
     }
     return 0;
@@ -434,7 +440,7 @@ static const char* essential_libs[] = {
     "libXtst.so.6", "libXt.so.6", "libXcomposite.so.1", "libXdamage.so.1", "libXmu.so.6", "libxkbcommon.so.0", 
     "libxkbcommon-x11.so.0", "libpulse-simple.so.0", "libpulse.so.0", "libvulkan.so.1", "libvulkan.so",
     "ld-linux-x86-64.so.2", "crashhandler.so", "libtcmalloc_minimal.so.0", "libtcmalloc_minimal.so.4", "libanl.so.1",
-    "ld-linux.so.2", "ld-linux.so.3"
+    "ld-linux.so.2", "ld-linux.so.3", "libthread_db.so.1"
 };
 static const char* essential_libs_egl[] = {
     "libEGL.so", "libGLESv2.so"
@@ -463,7 +469,7 @@ static size_t lib_cap = 0;
 
 library_t *NewLibrary(const char* path, box64context_t* context, elfheader_t* verneeded)
 {
-    printf_dump(LOG_DEBUG, "Trying to load \"%s\"\n", path);
+    printf_dlsym_dump(LOG_DEBUG, "Trying to load \"%s\"\n", path);
     //library_t *lib = (library_t*)box_calloc(1, sizeof(library_t));
     if(cur_lib==lib_cap) {
         lib_brick_t *new_brick = box_calloc(1, sizeof(lib_brick_t));
@@ -488,7 +494,7 @@ library_t *NewLibrary(const char* path, box64context_t* context, elfheader_t* ve
         lib->name = Path2Name(path);
     lib->nbdot = NbDot(lib->name);
     lib->type = LIB_UNNKNOW;
-    printf_dump(LOG_DEBUG, "Simplified name is \"%s\"\n", lib->name);
+    printf_dlsym_dump(LOG_DEBUG, "Simplified name is \"%s\"\n", lib->name);
     if(BOX64ENV(nopulse)) {
         if(strstr(lib->name, "libpulse.so")==lib->name || strstr(lib->name, "libpulse-simple.so")==lib->name) {
             box_free(lib->name);
@@ -618,7 +624,7 @@ void Free1Library(library_t **the_lib, x64emu_t* emu)
 
     FiniLibrary(lib, emu);
 
-    printf_dump(LOG_DEBUG, "Free1Library %s\n", lib->name?:"???");
+    printf_dlsym_dump(LOG_DEBUG, "Free1Library %s\n", lib->name?:"???");
     // remove lib from maplib/local_maplib...
     if(my_context) {
         MapLibRemoveLib(my_context->maplib, lib);
