@@ -15,6 +15,12 @@ GO("libdl.so.2", libdl)
 #endif
 GO("libresolv.so.2", libresolv)
 GO("libresolv.so", libresolv)
+GO("libselinux.so.1", selinux)
+GO("libselinux.so", selinux)
+GO("libblkid.so.1", blkid)
+GO("libblkid.so", blkid)
+GO("libmount.so.1", mount)
+GO("libmount.so", mount)
 
 GO("ld-linux.so.3", ldlinux)
 GO("ld-linux.so.2", ldlinux)
@@ -31,6 +37,10 @@ GO("libasound.so.2", libasound)
 GO("libasound.so", libasound)
 GO("libfreetype.so.6", freetype)
 GO("libfontconfig.so.1", fontconfig)
+GO("libEGL.so", libegl)
+GO("libEGL.so.1", libegl)
+GO("libGLX.so", libglx)
+GO("libGLX.so.0", libglx)
 
 GO("libX11.so", libx11)
 GO("libXdamage.so", libxdamage)
@@ -72,6 +82,7 @@ GO("libgnutls.so.30", gnutls)
 GO("libopenal.so.1", openal)
 GO("libopenal.so.0", openal)
 GO("libopenal.so", openal)
+GO("libopenal-soft.so.1", openal)
 GO("openal.so", openal)
 
 GO("libexpat.so.1", expat)
@@ -91,6 +102,12 @@ GO("libnsl.so.1", nsl)
 GO("libnsl.so", nsl)
 GO("libcups.so.2", libcups)
 GO("libcups.so", libcups)
+GO("libgbm.so.1", gbm)
+GO("libgbm.so", gbm)
+GO("libdbus-1.so.3", dbus)
+GO("libcurl.so.4", curl)
+GO("libcurl-gnutls.so.4", curl)
+GO("libcurl.so.3", curl)
 
 //GO("crashhandler.so", crashhandler)
 GO("libtcmalloc_minimal.so.0", tcmallocminimal)
