@@ -1407,3 +1407,88 @@ void* inplace_XDeviceInfo_enlarge(void* a)
     }
     return a;
 }
+
+
+void* inplace_XFilters_shrink(void* a)
+{
+    if(a) {
+        my_XFilters_t* src = a;
+        my_XFilters_32_t* dst = a;
+        dst->nfilter = src->nfilter;
+        char** filter = src->filter;
+        dst->filter = to_ptrv(filter);
+        dst->nalias = src->nalias;
+        dst->alias = to_ptrv(src->alias);
+        for(int i=0; i<dst->nfilter; ++i)
+            ((ptr_t*)(filter))[i] = to_ptrv(filter[i]);
+    }
+    return a;
+}
+void* inplace_XFilters_enlarge(void* a)
+{
+    if(a) {
+        my_XFilters_t* dst = a;
+        my_XFilters_32_t* src = a;
+        dst->alias = from_ptrv(src->alias);
+        dst->nalias = src->nalias;
+        dst->filter = from_ptrv(src->filter);
+        dst->nfilter = src->nfilter;
+        char** filter = dst->filter;
+        for(int i=dst->nfilter-1; i>=0; --i)
+            filter[i] = from_ptrv(((ptr_t*)(filter))[i]);
+    }
+    return a;
+}
+
+void* inplace_XRRMonitorInfo_shrink(void* a, int n)
+{
+    if(a) {
+        my_XRRMonitorInfo_t* src = a;
+        my_XRRMonitorInfo_32_t* dst = a;
+        for(int i=0; i<n; ++i) {
+            for(int j=0; j<src->noutput; ++j)
+                ((ulong_t*)src->outputs)[j] = to_ulong(src->outputs[j]);
+            dst->name = to_ulong(src->name);
+            dst->primary = src->primary;
+            dst->automatic = src->automatic;
+            dst->noutput = src->noutput;
+            dst->x = src->x;
+            dst->y = src->y;
+            dst->width = src->width;
+            dst->height = src->height;
+            dst->mwidth = src->mwidth;
+            dst->mheight = src->mheight;
+            dst->outputs = to_ptrv(src->outputs);
+            src++;
+            dst++;
+        }
+        src->name = 0;  // mark the last record...
+    }
+    return a;
+}
+
+void* inplace_XRRMonitorInfo_enlarge(void* a, int n)
+{
+    if(a) {
+        my_XRRMonitorInfo_32_t* src = a;
+        my_XRRMonitorInfo_t* dst = a;
+        for(int i=0; i<n; ++i) {
+            dst->name = from_ulong(src->name);
+            dst->primary = src->primary;
+            dst->automatic = src->automatic;
+            dst->noutput = src->noutput;
+            dst->x = src->x;
+            dst->y = src->y;
+            dst->width = src->width;
+            dst->height = src->height;
+            dst->mwidth = src->mwidth;
+            dst->mheight = src->mheight;
+            dst->outputs = from_ptrv(src->outputs);
+            for(int j=dst->noutput-1; j>=0; --j)
+                ((unsigned long*)dst->outputs)[j] = from_ulong(dst->outputs[j]);
+            src++;
+            dst++;
+        }
+    }
+    return a;
+}
