@@ -113,4 +113,10 @@ void* inplace_XAnyClassInfo_enlarge(void* a);
 void* inplace_XDeviceInfo_shrink(void* a, int n);
 void* inplace_XDeviceInfo_enlarge(void* a);
 
+void* inplace_XFilters_shrink(void* a);
+void* inplace_XFilters_enlarge(void* a);
+
+void* inplace_XRRMonitorInfo_shrink(void* a, int n);
+void* inplace_XRRMonitorInfo_enlarge(void* a, int n);
+
 #endif//MY_X11_CONV
\ No newline at end of file
