@@ -1465,4 +1465,26 @@ typedef struct my_XFixesCursorImage_s {
     void*           name;                  /* Version >= 2 only */
 } my_XFixesCursorImage_t;
 
+typedef struct my_XFilters_s {
+    int nfilter;
+    char **filter;
+    int nalias;
+    short *alias;
+} my_XFilters_t;
+
+typedef struct my__XRRMonitorInfo_s {
+    XID     name;
+    int     primary;
+    int     automatic;
+    int     noutput;
+    int     x;
+    int     y;
+    int     width;
+    int     height;
+    int     mwidth;
+    int     mheight;
+    XID*    outputs;
+} my_XRRMonitorInfo_t;
+
+
 #endif//MY_X11_DEFS
\ No newline at end of file
