@@ -1407,4 +1407,26 @@ typedef struct my_XFixesCursorImage_32_s {
     ptr_t           name;                  /* Version >= 2 only */
 } my_XFixesCursorImage_32_t;
 
+typedef struct my_XFilters_32_s {
+    int nfilter;
+    ptr_t filter;   //char **
+    int nalias;
+    ptr_t alias;    //short *
+} my_XFilters_32_t;
+
+typedef struct my__XRRMonitorInfo_32_s {
+    XID_32  name;
+    int     primary;
+    int     automatic;
+    int     noutput;
+    int     x;
+    int     y;
+    int     width;
+    int     height;
+    int     mwidth;
+    int     mheight;
+    ptr_t   outputs;   //XID*
+} my_XRRMonitorInfo_32_t;
+
+
 #endif//MY_X11_DEFS_32
\ No newline at end of file
