@@ -4,25 +4,24 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <pthread.h>
-#include <signal.h>
 #include <errno.h>
 #include <setjmp.h>
-#include <sys/mman.h>
-#include <dlfcn.h>
 
+#include "x64_signals.h"
+#include "os.h"
 #include "debug.h"
 #include "box64context.h"
 #include "threads.h"
 #include "emu/x64emu_private.h"
-#include "x64run.h"
 #include "x64emu.h"
 #include "box64stack.h"
+#include "box64cpu.h"
+#include "box64cpu_util.h"
 #include "callback.h"
 #include "custommem.h"
 #include "khash.h"
 #include "emu/x64run_private.h"
 #include "x64trace.h"
-#include "dynarec.h"
 #include "bridge.h"
 #include "myalign.h"
 #ifdef DYNAREC
@@ -31,6 +30,7 @@
 #endif
 #ifdef BOX32
 #include "box32.h"
+#include "threads32.h"
 #endif
 
 //void _pthread_cleanup_push_defer(void* buffer, void* routine, void* arg);	// declare hidden functions
@@ -65,9 +65,6 @@ typedef struct x64_unwind_buff_s {
 typedef void(*vFv_t)();
 
 KHASH_MAP_INIT_INT64(threadstack, threadstack_t*)
-#ifndef ANDROID
-KHASH_MAP_INIT_INT64(cancelthread, __pthread_unwind_buf_t*)
-#endif
 
 void CleanStackSize(box64context_t* context)
 {
@@ -129,6 +126,9 @@ int GetStackSize(uintptr_t attr, void** stack, size_t* stacksize)
 }
 
 void my_longjmp(x64emu_t* emu, /*struct __jmp_buf_tag __env[1]*/void *p, int32_t __val);
+#ifdef BOX32
+void my32_longjmp(x64emu_t* emu, /*struct __jmp_buf_tag __env[1]*/void *p, int32_t __val);
+#endif
 
 static pthread_key_t thread_key;
 
@@ -153,7 +153,13 @@ static void emuthread_cancel(void* p)
 	// check cancels threads
 	for(int i=et->cancel_size-1; i>=0; --i) {
 		et->emu->flags.quitonlongjmp = 0;
-		my_longjmp(et->emu, ((x64_unwind_buff_t*)et->cancels[i])->__cancel_jmp_buf, 1);
+		et->emu->quit = 0;
+		#ifdef BOX32
+		if(et->is32bits)
+			my32_longjmp(et->emu, ((i386_unwind_buff_t*)et->cancels[i])->__cancel_jmp_buf, 1);
+		else
+		#endif
+			my_longjmp(et->emu, ((x64_unwind_buff_t*)et->cancels[i])->__cancel_jmp_buf, 1);
 		DynaRun(et->emu);	// will return after a __pthread_unwind_next()
 	}
 	#ifdef BOX32
@@ -204,10 +210,10 @@ x64emu_t* thread_get_emu()
 		if(box64_is32bits)
 			stack = mmap64(NULL, stacksize, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS|MAP_32BIT, -1, 0);
 		else
-			stack = internal_mmap(NULL, stacksize, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS|MAP_GROWSDOWN, -1, 0);
+            stack = InternalMmap(NULL, stacksize, PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_ANONYMOUS | MAP_GROWSDOWN, -1, 0);
 		if(stack!=MAP_FAILED)
-			setProtection((uintptr_t)stack, stacksize, PROT_READ|PROT_WRITE);
-		x64emu_t *emu = NewX64Emu(my_context, 0, (uintptr_t)stack, stacksize, 1);
+			setProtection_stack((uintptr_t)stack, stacksize, PROT_READ|PROT_WRITE);
+		x64emu_t *emu = NewX64Emu(my_context, my_context->exit_bridge, (uintptr_t)stack, stacksize, 1);
 		SetupX64Emu(emu, NULL);
 		thread_set_emu(emu);
 		return emu;
@@ -325,14 +331,17 @@ EXPORT int my_pthread_attr_getguardsize(x64emu_t* emu, pthread_attr_t* attr, siz
 	PTHREAD_ATTR_ALIGN(attr);
 	return pthread_attr_getguardsize(PTHREAD_ATTR(attr), size);
 }
-#ifndef TERMUX
 EXPORT int my_pthread_attr_getinheritsched(x64emu_t* emu, pthread_attr_t* attr, int* sched)
 {
+#ifndef TERMUX
 	(void)emu;
 	PTHREAD_ATTR_ALIGN(attr);
 	return pthread_attr_getinheritsched(PTHREAD_ATTR(attr), sched);
-}
+#else
+	(void)emu; (void)attr; (void)sched;
+	return 0;
 #endif
+}
 EXPORT int my_pthread_attr_getschedparam(x64emu_t* emu, pthread_attr_t* attr, void* param)
 {
 	(void)emu;
@@ -413,16 +422,19 @@ EXPORT int my_pthread_attr_setguardsize(x64emu_t* emu, pthread_attr_t* attr, siz
 	PTHREAD_ATTR_UNALIGN(attr);
 	return ret;
 }
-#ifndef TERMUX
 EXPORT int my_pthread_attr_setinheritsched(x64emu_t* emu, pthread_attr_t* attr, int sched)
 {
+#ifndef TERMUX
 	(void)emu;
 	PTHREAD_ATTR_ALIGN(attr);
 	int ret = pthread_attr_setinheritsched(PTHREAD_ATTR(attr), sched);
 	PTHREAD_ATTR_UNALIGN(attr);
 	return ret;
-}
+#else
+	(void)emu; (void)attr; (void)sched;
+	return 0;
 #endif
+}
 EXPORT int my_pthread_attr_setschedparam(x64emu_t* emu, pthread_attr_t* attr, void* param)
 {
 	(void)emu;
@@ -521,9 +533,9 @@ EXPORT int my_pthread_create(x64emu_t *emu, void* t, void* attr, void* start_rou
 		stacksize = attr_stacksize;
 		own = 0;
 	} else {
-		stack = internal_mmap(NULL, stacksize, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS|MAP_GROWSDOWN, -1, 0);
-		if(stack!=MAP_FAILED)
-	        setProtection((uintptr_t)stack, stacksize, PROT_READ|PROT_WRITE);
+        stack = InternalMmap(NULL, stacksize, PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_ANONYMOUS | MAP_GROWSDOWN, -1, 0);
+        if(stack!=MAP_FAILED)
+	        setProtection_stack((uintptr_t)stack, stacksize, PROT_READ|PROT_WRITE);
 		own = 1;
 	}
 
@@ -550,9 +562,9 @@ EXPORT int my_pthread_create(x64emu_t *emu, void* t, void* attr, void* start_rou
 void* my_prepare_thread(x64emu_t *emu, void* f, void* arg, int ssize, void** pet)
 {
 	int stacksize = (ssize)?ssize:(2*1024*1024);	//default stack size is 2Mo
-	void* stack = internal_mmap(NULL, stacksize, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS|MAP_GROWSDOWN, -1, 0);
-	if(stack!=MAP_FAILED)
-		setProtection((uintptr_t)stack, stacksize, PROT_READ|PROT_WRITE);
+    void* stack = InternalMmap(NULL, stacksize, PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_ANONYMOUS | MAP_GROWSDOWN, -1, 0);
+    if(stack!=MAP_FAILED)
+		setProtection_stack((uintptr_t)stack, stacksize, PROT_READ|PROT_WRITE);
 	emuthread_t *et = (emuthread_t*)box_calloc(1, sizeof(emuthread_t));
 	x64emu_t *emuthread = NewX64Emu(emu->context, (uintptr_t)f, (uintptr_t)stack, stacksize, 1);
 	SetupX64Emu(emuthread, emu					);
@@ -832,6 +844,7 @@ EXPORT int my_pthread_setaffinity_np_old(x64emu_t* emu, pthread_t thread, void*
 
 EXPORT int my_pthread_kill(x64emu_t* emu, void* thread, int sig)
 {
+	sig = signal_from_x64(sig);
 	// should ESCHR result be filtered, as this is expected to be the 2.34 behaviour?
 	(void)emu;
 	// check for old "is everything ok?"
@@ -842,6 +855,7 @@ EXPORT int my_pthread_kill(x64emu_t* emu, void* thread, int sig)
 
 EXPORT int my_pthread_kill_old(x64emu_t* emu, void* thread, int sig)
 {
+	sig = signal_from_x64(sig);
     // check for old "is everything ok?"
     if((thread==NULL) && (sig==0))
         return real_phtread_kill_old(pthread_self(), 0);
@@ -878,16 +892,20 @@ EXPORT int my_pthread_mutexattr_getkind_np(x64emu_t* emu, my_mutexattr_t *attr,
 	attr->x86 = mattr.x86;
 	return ret;
 }
-#ifndef TERMUX
 EXPORT int my_pthread_mutexattr_getprotocol(x64emu_t* emu, my_mutexattr_t *attr, void* p)
 {
+#ifndef TERMUX
+	(void)emu;
 	my_mutexattr_t mattr = {0};
 	mattr.x86 = attr->x86;
 	int ret = pthread_mutexattr_getprotocol(&mattr.nat, p);
 	attr->x86 = mattr.x86;
 	return ret;
-}
+#else
+	(void)emu; (void)attr; (void)p;
+	return 0;
 #endif
+}
 EXPORT int my_pthread_mutexattr_gettype(x64emu_t* emu, my_mutexattr_t *attr, void* p)
 {
 	my_mutexattr_t mattr = {0};
@@ -924,16 +942,34 @@ EXPORT int my_pthread_mutexattr_setkind_np(x64emu_t* emu, my_mutexattr_t *attr,
 	attr->x86 = mattr.x86;
 	return ret;
 }
-#ifndef TERMUX
+EXPORT int my_pthread_mutexattr_setprioceiling(x64emu_t* emu, my_mutexattr_t *attr, int p)
+{
+#if !defined(TERMUX) && !defined(ANDROID)
+	(void)emu;
+	my_mutexattr_t mattr = {0};
+	mattr.x86 = attr->x86;
+	int ret = pthread_mutexattr_setprioceiling(&mattr.nat, p);
+	attr->x86 = mattr.x86;
+	return ret;
+#else
+	(void)emu; (void)attr; (void)p;
+	return 0;
+#endif
+}
 EXPORT int my_pthread_mutexattr_setprotocol(x64emu_t* emu, my_mutexattr_t *attr, int p)
 {
+#ifndef TERMUX
+	(void)emu;
 	my_mutexattr_t mattr = {0};
 	mattr.x86 = attr->x86;
 	int ret = pthread_mutexattr_setprotocol(&mattr.nat, p);
 	attr->x86 = mattr.x86;
 	return ret;
-}
+#else
+	(void)emu; (void)attr; (void)p;
+	return 0;
 #endif
+}
 EXPORT int my_pthread_mutexattr_setpshared(x64emu_t* emu, my_mutexattr_t *attr, int p)
 {
 	my_mutexattr_t mattr = {0};
