@@ -10,21 +10,23 @@
 #include <sys/mman.h>
 #include <dlfcn.h>
 
+#include "os.h"
 #include "debug.h"
 #include "box32context.h"
 #include "threads.h"
 #include "emu/x64emu_private.h"
 #include "tools/bridge_private.h"
-#include "x64run.h"
 #include "x64emu.h"
 #include "box64stack.h"
+#include "box64cpu.h"
+#include "box64cpu_util.h"
 #include "callback.h"
 #include "custommem.h"
 #include "khash.h"
 #include "emu/x64run_private.h"
 #include "x64trace.h"
-#include "dynarec.h"
 #include "bridge.h"
+#include "threads32.h"
 #ifdef DYNAREC
 #include "dynablock.h"
 #endif
@@ -52,32 +54,6 @@ typedef struct threadstack_s {
 	size_t 	stacksize;
 } threadstack_t;
 
-// longjmp / setjmp
-typedef struct jump_buff_i386_s {
- uint32_t save_ebx;
- uint32_t save_esi;
- uint32_t save_edi;
- uint32_t save_ebp;
- uint32_t save_esp;
- uint32_t save_eip;
-} jump_buff_i386_t;
-
-// sigset_t should have the same size on 32bits and 64bits machine (64bits)
-typedef struct __attribute__((packed, aligned(4))) __jmp_buf_tag_s {
-    jump_buff_i386_t __jmpbuf;
-    int              __mask_was_saved;
-    sigset_t         __saved_mask;
-} __jmp_buf_tag_t;
-
-typedef struct i386_unwind_buff_s {
-	struct {
-		jump_buff_i386_t	__cancel_jmp_buf;	
-		int					__mask_was_saved;
-	} __cancel_jmp_buf[1];
-	ptr_t __pad[2];
-	ptr_t __pad3;
-} __attribute__((packed, aligned(4))) i386_unwind_buff_t;
-
 // those are define in thread.c
 emuthread_t* thread_get_et();
 void thread_set_et(emuthread_t* et);
@@ -89,9 +65,6 @@ static void del_attr(void* attr);
 typedef void(*vFv_t)();
 
 KHASH_MAP_INIT_INT(threadstack, threadstack_t)
-#ifndef ANDROID
-KHASH_MAP_INIT_INT(cancelthread, __pthread_unwind_buf_t*)
-#endif
 
 void CleanStackSize(box64context_t* context);
 void FreeStackSize(uintptr_t attr);
@@ -106,11 +79,14 @@ static void emuthread_cancel(void* p)
 	if(!et)
 		return;
 	// check cancels threads
+	uintptr_t rax = et->emu->regs[_AX].q[0];
 	for(int i=et->cancel_size-1; i>=0; --i) {
 		et->emu->flags.quitonlongjmp = 0;
+		et->emu->quit = 0;
 		my32_longjmp(et->emu, ((i386_unwind_buff_t*)et->cancels[i])->__cancel_jmp_buf, 1);
 		DynaRun(et->emu);	// will return after a __pthread_unwind_next()
 	}
+	et->emu->regs[_AX].q[0] = rax;
 	box_free(et->cancels);
 	et->cancels=NULL;
 	et->cancel_size = et->cancel_cap = 0;
@@ -130,6 +106,7 @@ static void* pthread_routine(void* p)
 	// call the function
 	emuthread_t *et = (emuthread_t*)p;
 	thread_set_et(et);
+	et->is32bits = 1;
 	et->emu->type = EMUTYPE_MAIN;
 	et->self = (uintptr_t)pthread_self();
 	et->hself = to_hash(et->self);
@@ -183,13 +160,26 @@ EXPORT int my32_pthread_create(x64emu_t *emu, void* t, void* attr, void* start_r
 	size_t attr_stacksize;
 	int own;
 	void* stack = NULL;
+	pthread_attr_t* my_attr = NULL;
+	pthread_attr_t actual_attr;
 
 	if(attr) {
 		size_t stsize;
+		static size_t minsize = 0;
+		if(!minsize) {
+			minsize = PTHREAD_STACK_MIN;
+			if(minsize<512*1024) minsize = 512*1024;
+		}
 		if(pthread_attr_getstacksize(get_attr(attr), &stsize)==0)
 			stacksize = stsize;
-		if(stacksize<512*1024)	// emu and all needs some stack space, don't go too low
-			pthread_attr_setstacksize(get_attr(attr), 512*1024);
+		if(stacksize<minsize)	// emu and all needs some stack space, don't go too low
+			pthread_attr_setstacksize(get_attr(attr), minsize);
+		if(stacksize>1*1024*1024)
+			pthread_attr_setstacksize(get_attr(attr), 1*1024*1024);
+	} else {
+		my_attr = &actual_attr;
+		pthread_attr_init(my_attr);
+		pthread_attr_setstacksize(my_attr, 1*1024*1024);
 	}
 	if(GetStackSize((uintptr_t)attr, &attr_stack, &attr_stacksize))
 	{
@@ -203,12 +193,16 @@ EXPORT int my32_pthread_create(x64emu_t *emu, void* t, void* attr, void* start_r
 	}
 	if(!stack) {
 		//stack = malloc(stacksize);
-		stack = mmap64(NULL, stacksize, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS|MAP_32BIT, -1, 0);
+		stack = box_mmap(NULL, stacksize, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS|MAP_32BIT, -1, 0);
+		setProtection_stack((uintptr_t)stack, stacksize, PROT_READ|PROT_WRITE);
 		own = 1;
 	}
 
 	if((uintptr_t)stack>=0x100000000LL) {
-		if(own) munmap(stack, stacksize);
+		if(own) {
+			box_munmap(stack, stacksize);
+			freeProtection((uintptr_t)stack, stacksize);
+		}
 		return EAGAIN;
 	}
 
@@ -235,8 +229,10 @@ EXPORT int my32_pthread_create(x64emu_t *emu, void* t, void* attr, void* start_r
 	}
 	#endif
 	// create thread
-	return pthread_create((pthread_t*)t, get_attr(attr), 
+	int ret = pthread_create((pthread_t*)t, my_attr?my_attr:get_attr(attr), 
 		pthread_routine, et);
+	if(my_attr) pthread_attr_destroy(my_attr);
+	return ret;
 }
 
 EXPORT int my32_pthread_detach(x64emu_t* emu, pthread_t p)
@@ -292,6 +288,7 @@ EXPORT void my32___pthread_unregister_cancel(x64emu_t* emu, i386_unwind_buff_t*
 			if(i!=et->cancel_size-1)
 				memmove(et->cancels+i, et->cancels+i+1, sizeof(i386_unwind_buff_t*)*(et->cancel_size-i-1));
 			et->cancel_size--;
+			return;
 		}
 	}
 }
@@ -366,7 +363,7 @@ GO(29)
 static uintptr_t my32_cleanup_routine_fct_##A = 0;  				\
 static void my32_cleanup_routine_##A(void* a)    					\
 {                                       							\
-    RunFunctionFmt(my32_cleanup_routine_fct_##A, "p", to_ptrv(a));	\
+    RunFunctionFmt(my32_cleanup_routine_fct_##A, "p", a);			\
 }
 SUPER()
 #undef GO
@@ -389,7 +386,7 @@ static void* findcleanup_routineFct(void* fct)
 static uintptr_t my32_key_destructor_fct_##A = 0;  					\
 static void my32_key_destructor_##A(void* a)    					\
 {                                       							\
-    RunFunctionFmt(my32_key_destructor_fct_##A, "p", to_ptrv(a));	\
+    RunFunctionFmt(my32_key_destructor_fct_##A, "p", a);			\
 }
 SUPER()
 #undef GO
@@ -587,7 +584,12 @@ EXPORT int my32_pthread_cond_timedwait_old(x64emu_t* emu, pthread_cond_2_0_t* co
 {
 	pthread_mutex_t* m = getAlignedMutex((pthread_mutex_t*)mutex);
 	pthread_cond_t * c = get_cond_old(cond);
-	return pthread_cond_timedwait(c, m, (const struct timespec*)abstime);
+	struct timespec* atime = abstime;
+	while(atime->tv_nsec>1000000000LL) {
+		atime->tv_nsec-=1000000000LL;
+		++atime->tv_sec;
+	}
+	return pthread_cond_timedwait(c, m, atime);
 }
 EXPORT int my32_pthread_cond_wait_old(x64emu_t* emu, pthread_cond_2_0_t* cond, void* mutex)
 {
@@ -600,7 +602,12 @@ EXPORT int my32_pthread_cond_timedwait(x64emu_t* emu, void* cond, void* mutex, v
 {
 	pthread_mutex_t* m = getAlignedMutex((pthread_mutex_t*)mutex);
 	pthread_cond_t * c = get_cond(cond);
-	return pthread_cond_timedwait(c, m, (const struct timespec*)abstime);
+	struct timespec* atime = abstime;
+	while(atime->tv_nsec>1000000000LL) {
+		atime->tv_nsec-=1000000000LL;
+		++atime->tv_sec;
+	}
+	return pthread_cond_timedwait(c, m, atime);
 }
 EXPORT int my32_pthread_cond_wait(x64emu_t* emu, void* cond, void* mutex)
 {
