@@ -88,7 +88,7 @@ EXPORT void* my_atspi_event_listener_new(x64emu_t* emu, void* f, void* data, voi
 }
 
 #define PRE_INIT    \
-    if(box64_nogtk) \
+    if(BOX64ENV(nogtk)) \
         return -1;
 
 #include "wrappedlib_init.h"
