@@ -166,9 +166,9 @@ static void* find_OnProcessingPassStartFunc_Fct(void* fct)
 }
 typedef struct my_FAudioEngineCallback_s
 {
-	void* OnCriticalError;
-	void* OnProcessingPassEnd;
-	void* OnProcessingPassStart;
+    void* OnCriticalError;
+    void* OnProcessingPassEnd;
+    void* OnProcessingPassStart;
 } my_FAudioEngineCallback_t;
 // OnBufferEndFunc ...
 #define GO(A)   \
@@ -326,13 +326,13 @@ static void* find_OnVoiceProcessingPassStartFunc_Fct(void* fct)
 }
 typedef struct my_FAudioVoiceCallback_s
 {
-	void* OnBufferEnd;
-	void* OnBufferStart;
-	void* OnLoopEnd;
-	void* OnStreamEnd;
-	void* OnVoiceError;
-	void* OnVoiceProcessingPassEnd;
-	void* OnVoiceProcessingPassStart;
+    void* OnBufferEnd;
+    void* OnBufferStart;
+    void* OnLoopEnd;
+    void* OnStreamEnd;
+    void* OnVoiceError;
+    void* OnVoiceProcessingPassEnd;
+    void* OnVoiceProcessingPassStart;
 } my_FAudioVoiceCallback_t;
 // FAudioEngineCallEXT ...
 #define GO(A)   \
@@ -444,12 +444,4 @@ EXPORT uint32_t my_FAPOFX_CreateFXWithCustomAllocatorEXT(x64emu_t* emu, void* cl
     return my->FAPOFX_CreateFXWithCustomAllocatorEXT(clsid, p, data, size, find_FAudioMalloc_Fct(customMalloc), find_FAudioFree_Fct(customFree), find_FAudioRealloc_Fct(customRealloc));
 }
 
-
-#define CUSTOM_INIT \
-    getMy(lib);
-
-#define CUSTOM_FINI \
-    freeMy();
-
 #include "wrappedlib_init.h"
-
