@@ -24,6 +24,7 @@ GO(FcCharSetCopy, pFp)
 GO(FcCharSetCount, uFp)
 GO(FcCharSetCoverage, uFpup)
 GO(FcCharSetCreate, pFv)
+GO(FcCharSetDelChar, iFpu)
 GO(FcCharSetDestroy, vFp)
 GO(FcCharSetEqual, iFpp)
 GO(FcCharSetFirstPage, uFppp)   // second p is a fixed sized array
@@ -90,7 +91,7 @@ GO(FcFontSetSort, pFppipipp)
 GO(FcFontSetSortDestroy, vFp)
 GO(FcFontSort, pFppipp)
 GO(FcFreeTypeCharIndex, uFpu)
-//GO(FcFreeTypeCharSet, 
+GO(FcFreeTypeCharSet, pFpp)
 //GO(FcFreeTypeCharSetAndSpacing, 
 GO(FcFreeTypeQuery, pFpupp)
 GO(FcFreeTypeQueryFace, pFppip)
