@@ -3,15 +3,15 @@
 #endif
 
 //GOM(gdk_add_client_message_filter, vFpBp)
-//GO(gdk_add_option_entries_libgtk_only, 
+GO(gdk_add_option_entries_libgtk_only, vFp)
 //GO(gdk_app_launch_context_get_type, 
-//GO(gdk_app_launch_context_new, 
-//GO(gdk_app_launch_context_set_desktop, 
+GO(gdk_app_launch_context_new, pFv)
+GO(gdk_app_launch_context_set_desktop, vFpi)
 //GO(gdk_app_launch_context_set_display, 
-//GO(gdk_app_launch_context_set_icon, 
-//GO(gdk_app_launch_context_set_icon_name, 
-//GO(gdk_app_launch_context_set_screen, 
-//GO(gdk_app_launch_context_set_timestamp, 
+GO(gdk_app_launch_context_set_icon, vFpp)
+GO(gdk_app_launch_context_set_icon_name, vFpp)
+GO(gdk_app_launch_context_set_screen, vFpp)
+GO(gdk_app_launch_context_set_timestamp, vFpu)
 GO(gdk_atom_intern, pFpi)
 GO(gdk_atom_intern_static_string, pFp)
 GO(gdk_atom_name, pFp)
@@ -28,10 +28,10 @@ GO(gdk_cairo_set_source_pixbuf, vFppdd)
 GO(gdk_cairo_set_source_pixmap, vFppdd)
 GO(gdk_cairo_set_source_window, vFppdd)
 //GO(gdk_cap_style_get_type, 
-//GO(gdk_char_height, 
+GO(gdk_char_height, iFpC)
 //GO(gdk_char_measure, 
 //GO(gdk_char_width, 
-//GO(gdk_char_width_wc, 
+GO(gdk_char_width_wc, iFpu)
 GO(gdk_color_alloc, iFpp)
 GO(gdk_color_black, iFpp)
 GO(gdk_color_change, iFpp)
@@ -73,23 +73,23 @@ GO(gdk_cursor_ref, pFp)
 GO(gdk_cursor_type_get_type, LFv)
 GO(gdk_cursor_unref, vFp)
 GO(gdk_device_free_history, vFpi)
-GO(gdk_device_get_axis, iFppip)
-GO(gdk_device_get_axis_use, iFpu)
+GO(gdk_device_get_axis, iFppup)
+GO(gdk_device_get_axis_use, uFpu)
 GO(gdk_device_get_core_pointer, pFv)
 GO(gdk_device_get_has_cursor, iFp)
 GO(gdk_device_get_history, iFppuupp)
 GO(gdk_device_get_key, vFpupp)
-GO(gdk_device_get_mode, iFp)
+GO(gdk_device_get_mode, uFp)
 GO(gdk_device_get_name, pFp)
 GO(gdk_device_get_n_axes, iFp)
 GO(gdk_device_get_n_keys, iFp)
-GO(gdk_device_get_source, iFp)
+GO(gdk_device_get_source, uFp)
 GO(gdk_device_get_state, vFpppp)
 GO(gdk_device_get_type, LFv)
-GO(gdk_device_set_axis_use, vFpui)
-GO(gdk_device_set_key, vFpuui)
-GO(gdk_device_set_mode, iFpi)
-GO(gdk_device_set_source, vFpi)
+GO(gdk_device_set_axis_use, vFpuu)
+GO(gdk_device_set_key, vFpuuu)
+GO(gdk_device_set_mode, iFpu)
+GO(gdk_device_set_source, vFpu)
 GO(gdk_devices_list, pFv)
 //GOM(gdk_display_add_client_message_filter, vFEpppp)
 GO(gdk_display_beep, vFp)
@@ -141,8 +141,8 @@ GO(gdk_drag_action_get_type, LFv)
 GO(gdk_drag_begin, pFpp)
 GO(gdk_drag_context_get_actions, iFp)
 GO(gdk_drag_context_get_dest_window, pFp)
-GO(gdk_drag_context_get_protocol, iFp)
-GO(gdk_drag_context_get_selected_action, iFp)
+GO(gdk_drag_context_get_protocol, uFp)
+GO(gdk_drag_context_get_selected_action, uFp)
 GO(gdk_drag_context_get_source_window, pFp)
 GO(gdk_drag_context_get_suggested_action, iFp)
 GO(gdk_drag_context_get_type, LFv)
@@ -154,12 +154,12 @@ GO(gdk_drag_drop, vFpu)
 GO(gdk_drag_drop_succeeded, iFp)
 GO(gdk_drag_find_window, vFppiipp)
 GO(gdk_drag_find_window_for_screen, vFpppiipp)
-GO(gdk_drag_get_protocol, pFpp)
+GO(gdk_drag_get_protocol, uFup)
 GO(gdk_drag_get_protocol_for_display, pFppp)
 GO(gdk_drag_get_selection, pFp)
-GO(gdk_drag_motion, iFppiiiiiu)
+GO(gdk_drag_motion, iFppuiiuuu)
 GO(gdk_drag_protocol_get_type, LFv)
-GO(gdk_drag_status, vFpiu)
+GO(gdk_drag_status, vFpuu)
 GO(gdk_drawable_copy_to_image, pFppiiiiii)
 GO(gdk_drawable_get_clip_region, pFp)
 GO(gdk_drawable_get_colormap, pFp)
@@ -180,22 +180,22 @@ GO(gdk_draw_arc, vFppiiiiiii)
 GO(gdk_draw_drawable, vFpppiiiiii)
 GO(gdk_draw_glyphs, vFpppiip)
 GO(gdk_draw_glyphs_transformed, vFppppiip)
-GO(gdk_draw_gray_image, vFppiiiiipi)
+GO(gdk_draw_gray_image, vFppiiiiupi)
 GO(gdk_draw_image, vFpppiiiiii)
-GO(gdk_draw_indexed_image, vFppiiiiipip)
+GO(gdk_draw_indexed_image, vFppiiiiupip)
 GO(gdk_draw_layout, vFppiip)
 GO(gdk_draw_layout_line, vFppiip)
 GO(gdk_draw_layout_line_with_colors, vFppiippp)
 GO(gdk_draw_layout_with_colors, vFppiippp)
 GO(gdk_draw_line, vFppiiii)
 GO(gdk_draw_lines, vFpppi)
-GO(gdk_draw_pixbuf, vFpppiiiiiiiii)
+GO(gdk_draw_pixbuf, vFpppiiiiiiuii)
 GO(gdk_draw_point, vFppii)
 GO(gdk_draw_points, vFpppi)
 GO(gdk_draw_polygon, vFppipi)
 GO(gdk_draw_rectangle, vFppiiiii)
 GO(gdk_draw_rgb_32_image, vFppiiiiipi)
-GO(gdk_draw_rgb_32_image_dithalign, vFppiiiiipiii)
+GO(gdk_draw_rgb_32_image_dithalign, vFppiiiiupiii)
 GO(gdk_draw_rgb_image, vFppiiiiipi)
 GO(gdk_draw_rgb_image_dithalign, vFppiiiiipiii)
 GO(gdk_draw_segments, vFpppi)
@@ -210,7 +210,7 @@ GO(gdk_error_trap_push, vFv)
 GO(gdk_event_copy, pFp)
 GO(gdk_event_free, vFp)
 GO(gdk_event_get, pFv)
-GO(gdk_event_get_axis, iFpip)
+GO(gdk_event_get_axis, iFpup)
 GO(gdk_event_get_coords, iFppp)
 GO(gdk_event_get_graphics_expose, pFp)
 GO(gdk_event_get_root_coords, iFppp)
@@ -224,7 +224,7 @@ GO(gdk_event_new, pFi)
 GO(gdk_event_peek, pFv)
 GO(gdk_event_put, vFp)
 GO(gdk_event_request_motions, vFp)
-GO(gdk_event_send_client_message, iFpp)
+GO(gdk_event_send_client_message, iFpu)
 GO(gdk_event_send_client_message_for_display, iFppp)
 GO(gdk_event_send_clientmessage_toall, vFp)
 GO(gdk_event_set_screen, vFpp)
@@ -236,19 +236,19 @@ GO(gdk_exit, vFi)
 //GO(gdk_fill_rule_get_type, 
 //GO(gdk_filter_return_get_type, 
 GO(gdk_flush, vFv)
-//GO(gdk_font_equal, 
-//GO(gdk_font_from_description, 
+GO(gdk_font_equal, iFpp)
+GO(gdk_font_from_description, pFp)
 //GO(gdk_font_from_description_for_display, 
 //GO(gdk_font_get_display, 
 GO(gdk_font_get_type, LFv)
-//GO(gdk_font_id, 
+GO(gdk_font_id, iFp)
 //GO(gdk_font_load, 
 //GO(gdk_font_load_for_display, 
-//GO(gdk_font_ref, 
-//GO(gdk_fontset_load, 
+GO(gdk_font_ref, pFp)
+GO(gdk_fontset_load, pFp)
 //GO(gdk_fontset_load_for_display, 
 //GO(gdk_font_type_get_type, 
-//GO(gdk_font_unref, 
+GO(gdk_font_unref, vFp)
 GO(gdk_free_compound_text, vFp)
 GO(gdk_free_text_list, vFp)
 GO(gdk_function_get_type, LFv)
@@ -269,15 +269,15 @@ GO(gdk_gc_set_clip_region, vFpp)
 GO(gdk_gc_set_colormap, vFpp)
 GO(gdk_gc_set_dashes, vFpipi)
 GO(gdk_gc_set_exposures, vFpi)
-GO(gdk_gc_set_fill, vFpi)
+GO(gdk_gc_set_fill, vFpu)
 GO(gdk_gc_set_font, vFpp)
 GO(gdk_gc_set_foreground, vFpp)
-GO(gdk_gc_set_function,vFpi) 
-GO(gdk_gc_set_line_attributes, vFpiiii)
+GO(gdk_gc_set_function, vFpu)
+GO(gdk_gc_set_line_attributes, vFpiuuu)
 GO(gdk_gc_set_rgb_bg_color, vFpp)
 GO(gdk_gc_set_rgb_fg_color, vFpp)
 GO(gdk_gc_set_stipple, vFpp)
-GO(gdk_gc_set_subwindow, vFpi)
+GO(gdk_gc_set_subwindow, vFpu)
 GO(gdk_gc_set_tile, vFpp)
 GO(gdk_gc_set_ts_origin, vFpii)
 GO(gdk_gc_set_values, vFppu)
@@ -293,19 +293,19 @@ GO(gdk_grab_status_get_type, LFv)
 GO(gdk_gravity_get_type, LFv)
 GO(gdk_image_get, pFpiiii)
 GO(gdk_image_get_bits_per_pixel, WFp)
-GO(gdk_image_get_byte_order, iFp)
+GO(gdk_image_get_byte_order, uFp)
 GO(gdk_image_get_bytes_per_line, WFp)
 GO(gdk_image_get_bytes_per_pixel, WFp)
 GO(gdk_image_get_colormap, pFp)
 GO(gdk_image_get_depth, WFp)
 GO(gdk_image_get_height, iFp)
-GO(gdk_image_get_image_type, iFp)
+GO(gdk_image_get_image_type, uFp)
 GO(gdk_image_get_pixel, uFpii)
 GO(gdk_image_get_pixels, pFp)
 GO(gdk_image_get_type, LFv)
 GO(gdk_image_get_visual, pFp)
 GO(gdk_image_get_width, iFp)
-GO(gdk_image_new, pFipii)
+GO(gdk_image_new, pFupii)
 GO(gdk_image_new_bitmap, pFppii)
 GO(gdk_image_put_pixel, vFpiiu)
 GO(gdk_image_ref, pFp)
@@ -319,10 +319,10 @@ GOM(gdk_input_add_full, iFEiippp)
 GO(gdk_input_condition_get_type, LFv)
 GO(gdk_input_mode_get_type, LFv)
 GO(gdk_input_remove, vFi)
-GO(gdk_input_set_extension_events, vFpii)
+GO(gdk_input_set_extension_events, vFpiu)
 GO(gdk_input_source_get_type, LFv)
 GO(gdk_join_style_get_type, LFv)
-GO(gdk_keyboard_grab, iFpiu)
+GO(gdk_keyboard_grab, uFpiu)
 //GO(gdk_keyboard_grab_info_libgtk_only, 
 GO(gdk_keyboard_ungrab, vFu)
 GO(gdk_keymap_add_virtual_modifiers, vFpp)
@@ -347,7 +347,7 @@ GO(gdk_keyval_to_unicode, uFu)
 GO(gdk_keyval_to_upper, uFu)
 GO(gdk_line_style_get_type, LFv)
 GO(gdk_list_visuals, pFv)
-//GO(gdk_mbstowcs, 
+GO(gdk_mbstowcs, iFppi)
 GO(gdk_modifier_type_get_type, LFv)
 GO(gdk_net_wm_supports, iFp)
 GO(gdk_notify_startup_complete, vFv)
@@ -372,16 +372,16 @@ GO(gdk_pango_renderer_get_type, LFv)
 GO(gdk_pango_renderer_new, pFp)
 GO(gdk_pango_renderer_set_drawable, vFpp)
 GO(gdk_pango_renderer_set_gc, vFpp)
-GO(gdk_pango_renderer_set_override_color, vFpip)
-GO(gdk_pango_renderer_set_stipple, vFpip)
+GO(gdk_pango_renderer_set_override_color, vFpup)
+GO(gdk_pango_renderer_set_stipple, vFpup)
 GO(gdk_parse_args, vFpp)
 GO(gdk_pixbuf_get_from_drawable, pFpppiiiiii)
 GO(gdk_pixbuf_get_from_image, pFpppiiiiii)
 GO(gdk_pixbuf_render_pixmap_and_mask, vFpppi)
 GO(gdk_pixbuf_render_pixmap_and_mask_for_colormap, vFppppi)
 GO(gdk_pixbuf_render_threshold_alpha, vFppiiiiiii)
-GO(gdk_pixbuf_render_to_drawable, vFpppiiiiiiiii)
-GO(gdk_pixbuf_render_to_drawable_alpha, vFppiiiiiiiiiii)
+GO(gdk_pixbuf_render_to_drawable, vFpppiiiiiiuii)
+GO(gdk_pixbuf_render_to_drawable_alpha, vFppiiiiiiuiuii)
 GO(gdk_pixmap_colormap_create_from_xpm, pFppppp)
 GO(gdk_pixmap_colormap_create_from_xpm_d, pFppppp)
 GO(gdk_pixmap_create_from_data, pFppiiipp)
@@ -389,19 +389,19 @@ GO(gdk_pixmap_create_from_xpm, pFpppp)
 GO(gdk_pixmap_create_from_xpm_d, pFpppp)
 GO(gdk_pixmap_foreign_new, pFp)
 GO(gdk_pixmap_foreign_new_for_display, pFpp)
-GO(gdk_pixmap_foreign_new_for_screen, pFppiii)
+GO(gdk_pixmap_foreign_new_for_screen, pFpuiii)
 GO(gdk_pixmap_get_size, vFppp)
 GO(gdk_pixmap_get_type, LFv)
 GO(gdk_pixmap_impl_x11_get_type, LFv)
-GO(gdk_pixmap_lookup, pFp)
+GO(gdk_pixmap_lookup, pFu)
 GO(gdk_pixmap_lookup_for_display, pFpp)
 GO(gdk_pixmap_new, pFpiii)
-GO(gdk_pointer_grab, iFpiippu)
+GO(gdk_pointer_grab, uFpiuppu)
 //GO(gdk_pointer_grab_info_libgtk_only, 
 GO(gdk_pointer_is_grabbed, iFv)
 GO(gdk_pointer_ungrab, vFu)
 //GO(gdk_pre_parse_libgtk_only, 
-GO(gdk_property_change, vFpppiipi)
+GO(gdk_property_change, vFpppiupi)
 GO(gdk_property_delete, vFpp)
 GO(gdk_property_get, iFpppLLipppp)
 GO(gdk_property_state_get_type, LFv)
@@ -412,7 +412,7 @@ GO(gdk_rectangle_get_type, LFv)
 GO(gdk_rectangle_intersect, iFppp)
 GO(gdk_rectangle_union, vFppp)
 GO(gdk_region_copy, pFp)
-GO(gdk_region_destroy, vFpp)
+GO(gdk_region_destroy, vFp)
 GO(gdk_region_empty, iFp)
 GO(gdk_region_equal, iFpp)
 GO(gdk_region_get_clipbox, vFpp)
@@ -421,10 +421,10 @@ GO(gdk_region_intersect, vFpp)
 GO(gdk_region_new, pFv)
 GO(gdk_region_offset, vFpii)
 GO(gdk_region_point_in, iFpii)
-GO(gdk_region_polygon, pFpii)
+GO(gdk_region_polygon, pFpiu)
 GO(gdk_region_rectangle, pFp)
 GO(gdk_region_rect_equal, iFpp)
-GO(gdk_region_rect_in, iFpp)
+GO(gdk_region_rect_in, uFpp)
 GO(gdk_region_shrink, vFpii)
 //GOM(gdk_region_spans_intersect_foreach, vFppiiBp)
 GO(gdk_region_subtract, vFpp)
@@ -494,7 +494,7 @@ GO(gdk_selection_owner_get_for_display, pFpp)
 GO(gdk_selection_owner_set, iFppui)
 GO(gdk_selection_owner_set_for_display, iFpppui)
 GO(gdk_selection_property_get, iFpppp)
-GO(gdk_selection_send_notify, vFppppu)
+GO(gdk_selection_send_notify, vFupppu)
 GO(gdk_selection_send_notify_for_display, vFpppppu)
 GO(gdk_set_double_click_time, vFu)
 GO(gdk_set_locale, pFv)
@@ -509,31 +509,31 @@ GO(gdk_spawn_command_line_on_screen, iFppp)
 //GOM(gdk_spawn_on_screen, iFEppppipppp)
 //GOM(gdk_spawn_on_screen_with_pipes, iFEppppippppppp)
 GO(gdk_status_get_type, LFv)
-//GO(gdk_string_extents, 
-//GO(gdk_string_height, 
+GO(gdk_string_extents, vFppppppp)
+GO(gdk_string_height, iFpp)
 //GO(gdk_string_measure, 
 GO(gdk_string_to_compound_text, iFppppp)
 GO(gdk_string_to_compound_text_for_display, iFpppppp)
 //GO(gdk_string_width, 
 //GO(gdk_subwindow_mode_get_type, 
 //GO(gdk_synthesize_window_state, 
-//GO(gdk_test_render_sync, 
-//GO(gdk_test_simulate_button, 
+GO(gdk_test_render_sync, vFp)
+GO(gdk_test_simulate_button, iFpiiuui)
 //GO(gdk_test_simulate_key, 
-//GO(gdk_text_extents, 
-//GO(gdk_text_extents_wc, 
-//GO(gdk_text_height, 
+GO(gdk_text_extents, vFppippppp)
+GO(gdk_text_extents_wc, vFppippppp)
+GO(gdk_text_height, iFppi)
 //GO(gdk_text_measure, 
 GO(gdk_text_property_to_text_list, iFpipip)
 GO(gdk_text_property_to_text_list_for_display, iFppipip)
 GO(gdk_text_property_to_utf8_list, iFpipip)
 //GO(gdk_text_property_to_utf8_list_for_display, 
 //GO(gdk_text_width, 
-//GO(gdk_text_width_wc, 
+GO(gdk_text_width_wc, iFppi)
 //GOM(gdk_threads_add_idle, uFEpp)
 //GOM(gdk_threads_add_idle_full, uFEippp)
 //GOM(gdk_threads_add_timeout, uFEupp)
-//GOM(gdk_threads_add_timeout_full, uFEiuppp)
+GOM(gdk_threads_add_timeout_full, uFEiuppp)
 //GOM(gdk_threads_add_timeout_seconds, uFEupp)
 //GOM(gdk_threads_add_timeout_seconds_full, uFEiuppp)
 GO(gdk_threads_enter, vFv)
@@ -547,13 +547,13 @@ GO(gdk_utf8_to_string_target, pFp)
 GO(gdk_visibility_state_get_type, LFv)
 GO(gdk_visual_get_best, pFv)
 GO(gdk_visual_get_best_depth, iFv)
-GO(gdk_visual_get_best_type, iFv)
-GO(gdk_visual_get_best_with_both, pFii)
+GO(gdk_visual_get_best_type, uFv)
+GO(gdk_visual_get_best_with_both, pFiu)
 GO(gdk_visual_get_best_with_depth, pFi)
-GO(gdk_visual_get_best_with_type, pFi)
+GO(gdk_visual_get_best_with_type, pFu)
 GO(gdk_visual_get_bits_per_rgb, iFp)
 GO(gdk_visual_get_blue_pixel_details, vFpppp)
-GO(gdk_visual_get_byte_order, iFp)
+GO(gdk_visual_get_byte_order, uFp)
 GO(gdk_visual_get_colormap_size, iFp)
 GO(gdk_visual_get_depth, iFp)
 GO(gdk_visual_get_green_pixel_details, vFpppp)
@@ -561,9 +561,9 @@ GO(gdk_visual_get_red_pixel_details, vFpppp)
 GO(gdk_visual_get_screen, pFp)
 GO(gdk_visual_get_system, pFv)
 GO(gdk_visual_get_type, LFv)
-GO(gdk_visual_get_visual_type, iFp)
+GO(gdk_visual_get_visual_type, uFp)
 GO(gdk_visual_type_get_type, LFv)
-//GO(gdk_wcstombs, 
+GO(gdk_wcstombs, pFp)
 GOM(gdk_window_add_filter, vFEppp)
 GO(gdk_window_at_pointer, pFpp)
 //GO(gdk_window_attributes_type_get_type, 
@@ -580,7 +580,7 @@ GO(gdk_window_configure_finished, vFp)
 GO(gdk_window_constrain_size, vFpuiipp)
 GO(gdk_window_coords_from_parent, vFpddpp)
 GO(gdk_window_coords_to_parent, vFpddpp)
-GO(gdk_window_create_similar_surface, pFpiii)
+GO(gdk_window_create_similar_surface, pFpuii)
 GO(gdk_window_deiconify, vFp)
 GO(gdk_window_destroy, vFp)
 //GO(gdk_window_destroy_notify, 
@@ -606,7 +606,7 @@ GO(gdk_window_get_deskrelative_origin, iFppp)
 GO(gdk_window_get_display, pFp)
 GO(gdk_window_get_effective_parent, pFp)
 GO(gdk_window_get_effective_toplevel, pFp)
-GO(gdk_window_get_events, vFpi)
+GO(gdk_window_get_events, uFp)
 GO(gdk_window_get_focus_on_map, iFp)
 GO(gdk_window_get_frame_extents, vFpp)
 GO(gdk_window_get_geometry, vFpppppp)
@@ -621,15 +621,15 @@ GO(gdk_window_get_position, vFppp)
 GO(gdk_window_get_root_coords, vFpiipp)
 GO(gdk_window_get_root_origin, vFppp)
 GO(gdk_window_get_screen, pFp)
-GO(gdk_window_get_state, iFp)
+GO(gdk_window_get_state, uFp)
 GO(gdk_window_get_toplevel, pFp)
 GO(gdk_window_get_toplevels, pFp)
-GO(gdk_window_get_type_hint, iFp)
+GO(gdk_window_get_type_hint, uFp)
 GO(gdk_window_get_update_area, pFp)
 GO(gdk_window_get_user_data, vFpp)
 GO(gdk_window_get_visual, pFp)
 GO(gdk_window_get_width, iFp)
-GO(gdk_window_get_window_type, iFp)
+GO(gdk_window_get_window_type, uFp)
 GO(gdk_window_has_native, iFp)
 GO(gdk_window_hide, vFp)
 GO(gdk_window_hints_get_type, LFv)
@@ -646,7 +646,7 @@ GO(gdk_window_is_input_only, iFp)
 GO(gdk_window_is_shaped, iFp)
 GO(gdk_window_is_viewable, iFp)
 GO(gdk_window_is_visible, iFp)
-GO(gdk_window_lookup, pFp)
+GO(gdk_window_lookup, pFu)
 GO(gdk_window_lookup_for_display, pFpp)
 GO(gdk_window_lower, vFp)
 GO(gdk_window_maximize, vFp)
@@ -677,11 +677,11 @@ GO(gdk_window_set_child_shapes, vFp)
 GO(gdk_window_set_composited, vFpi)
 GO(gdk_window_set_cursor, vFpp)
 GO(gdk_window_set_debug_updates, vFi)
-GO(gdk_window_set_decorations, vFpp)
-GO(gdk_window_set_events, vFpp)
+GO(gdk_window_set_decorations, vFpu)
+GO(gdk_window_set_events, vFpu)
 GO(gdk_window_set_focus_on_map, vFpi)
 GO(gdk_window_set_functions, vFpu)
-GO(gdk_window_set_geometry_hints, vFppi)
+GO(gdk_window_set_geometry_hints, vFppu)
 GO(gdk_window_set_group, vFpp)
 GO(gdk_window_set_hints, vFpiiiiiii)
 GO(gdk_window_set_icon, vFpppp)
@@ -690,7 +690,7 @@ GO(gdk_window_set_icon_name, vFpp)
 GO(gdk_window_set_keep_above, vFpi)
 GO(gdk_window_set_keep_below, vFpi)
 GO(gdk_window_set_modal_hint, iFp)
-GO(gdk_window_set_opacity, vFpi)
+GO(gdk_window_set_opacity, vFpd)
 GO(gdk_window_set_override_redirect, vFpi)
 GO(gdk_window_set_role, vFpp)
 GO(gdk_window_set_skip_pager_hint, vFpi)
@@ -699,7 +699,7 @@ GO(gdk_window_set_startup_id, vFpp)
 GO(gdk_window_set_static_gravities, iFpi)
 GO(gdk_window_set_title, vFpp)
 GO(gdk_window_set_transient_for, vFpp)
-GO(gdk_window_set_type_hint, vFpi)
+GO(gdk_window_set_type_hint, vFpu)
 GO(gdk_window_set_urgency_hint, vFpi)
 GO(gdk_window_set_user_data, vFpp)
 GO(gdk_window_shape_combine_mask, vFppii)
@@ -789,4 +789,4 @@ GO(dummy_set_child_property, vFppupp)       // to get vFppupp for gtkclass.c
 GO2(gdk_pixmap_ref, pFp, g_object_ref)
 GO2(gdk_pixmap_unref, vFp, g_object_unref)
 GO2(gdk_bitmap_ref, pFp, g_object_ref)
-GO2(gdk_bitmap_unref, vFp, g_object_unref)
\ No newline at end of file
+GO2(gdk_bitmap_unref, vFp, g_object_unref)
