@@ -458,9 +458,9 @@ GO(g_dbus_method_invocation_get_sender, pFp)
 GO(g_dbus_method_invocation_get_type, LFv)
 GO(g_dbus_method_invocation_get_user_data, pFp)
 GO(g_dbus_method_invocation_return_dbus_error, vFppp)
-//GOM(g_dbus_method_invocation_return_error, vFpuipV)
+GOM(g_dbus_method_invocation_return_error, vFpuipV)
 GO(g_dbus_method_invocation_return_error_literal, vFpuip)
-//GOM(g_dbus_method_invocation_return_error_valist, vFpuipA)
+GOM(g_dbus_method_invocation_return_error_valist, vFpuipA)
 GO(g_dbus_method_invocation_return_gerror, vFpp)
 GO(g_dbus_method_invocation_return_value, vFpp)
 GO(g_dbus_method_invocation_return_value_with_unix_fd_list, vFppp)
@@ -1146,6 +1146,7 @@ GO(g_network_monitor_can_reach_finish, iFppp)
 GO(g_network_monitor_get_connectivity, iFp)
 GO(g_network_monitor_get_default, pFv)
 GO(g_network_monitor_get_network_available, iFp)
+GO(g_network_monitor_get_network_metered, iFp)
 GO(g_network_monitor_get_type, LFv)
 GO(g_network_service_get_domain, pFp)
 GO(g_network_service_get_protocol, pFp)
