@@ -1115,7 +1115,7 @@ EXPORT void* my_g_variant_new_va(x64emu_t* emu, char* fmt, void* endptr, x64_va_
     #else
       #if defined(__loongarch64) || defined(__riscv)
         va_list sysv_varargs;
-        myStackAlignGVariantNew(emu, fmt, emu->scratch, b);
+        myStackAlignGVariantNewVa(emu, fmt, emu->scratch, b);
         sysv_varargs = (va_list)emu->scratch;
       #else
         CREATE_VALIST_FROM_VALIST(*b, emu->scratch);
@@ -1126,7 +1126,12 @@ EXPORT void* my_g_variant_new_va(x64emu_t* emu, char* fmt, void* endptr, x64_va_
 
 EXPORT void* my_g_variant_new(x64emu_t* emu, char* fmt, uint64_t* V)
 {
+#if defined(__loongarch64) || defined(__riscv)
+    myStackAlignGVariantNew(emu, fmt, V, emu->scratch, R_EAX);
+    PREPARE_VALIST;
+#else
     CREATE_VALIST_FROM_VAARG(V, emu->scratch, 1);
+#endif
     return my->g_variant_new_va(fmt, NULL, &VARARGS);
 }
 
@@ -1536,6 +1541,26 @@ EXPORT void my_g_queue_insert_sorted(x64emu_t* emu, void* queue, void* data, voi
     my->g_queue_insert_sorted(queue, data, findGCompareDataFuncFct(f), user_data);
 }
 
+EXPORT void* my_g_async_queue_new_full(x64emu_t* emu, void* item_free_func)
+{
+    return my->g_async_queue_new_full(findGDestroyNotifyFct(item_free_func));
+}
+
+EXPORT void* my_g_thread_pool_new(x64emu_t* emu, void* func, void* user_data, int32_t max_threads, int32_t exclusive, void* error)
+{
+    return my->g_thread_pool_new(findGFuncFct(func), user_data, max_threads, exclusive, error);
+}
+
+EXPORT void my_g_async_queue_push_sorted(x64emu_t* emu, void* queue, void* data, void* func, void* user_data)
+{
+    my->g_async_queue_push_sorted(queue, data, findGCompareDataFuncFct(func), user_data);
+}
+
+EXPORT void my_g_thread_pool_set_sort_function(x64emu_t* emu, void* pool, void* func, void* user_data)
+{
+    my->g_thread_pool_set_sort_function(pool, findGCompareDataFuncFct(func), user_data);
+}
+
 #define PRE_INIT    \
     if(BOX64ENV(nogtk)) \
         return -1;
