@@ -47,11 +47,11 @@ GO(g_async_queue_length, iFp)
 GO(g_async_queue_length_unlocked, iFp)
 GO(g_async_queue_lock, vFp)
 GO(g_async_queue_new, pFv)
-//GOM(g_async_queue_new_full, pFp)
+GOM(g_async_queue_new_full, pFEp)
 GO(g_async_queue_pop, pFp)
 GO(g_async_queue_pop_unlocked, pFp)
 GO(g_async_queue_push, vFpp)
-//GOM(g_async_queue_push_sorted, vFpppp)
+GOM(g_async_queue_push_sorted, vFEpppp)
 //GOM(g_async_queue_push_sorted_unlocked, vFpppp)
 GO(g_async_queue_push_unlocked, vFpp)
 GO(g_async_queue_ref, pFp)
@@ -1191,6 +1191,7 @@ GO(g_string_prepend_c, pFpc)
 GO(g_string_prepend_len, pFppl)
 GO(g_string_prepend_unichar, pFpu)
 GOM(g_string_printf, vFEppV)
+GO(g_string_replace, uFpppu)
 GO(g_string_set_size, pFpL)
 GO(g_string_sized_new, pFL)
 GO(g_string_truncate, pFpL)
@@ -1277,20 +1278,20 @@ GO(g_thread_get_initialized, iFv)
 //GO(g_thread_init_glib, 
 GO(g_thread_join, pFp)
 GOM(g_thread_new, pFEppp)
-//GOM(g_thread_pool_free, vFpii)
+GO(g_thread_pool_free, vFpii)
 GO(g_thread_pool_get_max_idle_time, uFv)
-//GOM(g_thread_pool_get_max_threads, iFp)
+GO(g_thread_pool_get_max_threads, iFp)
 GO(g_thread_pool_get_max_unused_threads, iFv)
-//GOM(g_thread_pool_get_num_threads, uFp)
+GO(g_thread_pool_get_num_threads, uFp)
 GO(g_thread_pool_get_num_unused_threads, uFv)
-//GOM(g_thread_pool_new, pFppiip)
-//GOM(g_thread_pool_push, iFppp)
+GOM(g_thread_pool_new, pFEppiip)
+GO(g_thread_pool_push, iFppp)
 GO(g_thread_pool_set_max_idle_time, vFu)
-//GOM(g_thread_pool_set_max_threads, iFpip)
+GO(g_thread_pool_set_max_threads, iFpip)
 GO(g_thread_pool_set_max_unused_threads, vFi)
-//GOM(g_thread_pool_set_sort_function, vFppp)
+GOM(g_thread_pool_set_sort_function, vFEppp)
 GO(g_thread_pool_stop_unused_threads, vFv)
-//GOM(g_thread_pool_unprocessed, uFp)
+GO(g_thread_pool_unprocessed, uFp)
 GO(g_thread_ref, pFp)
 GO(g_thread_self, pFv)
 GO(g_thread_set_priority, vFpu)
@@ -1437,6 +1438,8 @@ GO(g_utf8_to_ucs4_fast, pFplp)
 GO(g_utf8_to_utf16, pFplppp)
 GO(g_utf8_validate, iFplp)
 GO(g_utime, iFpp) // Warning: failed to confirm
+GO(g_uuid_string_is_valid, iFp)
+GO(g_uuid_string_random, pFv)
 GOM(g_variant_builder_add, vFEppV)
 //GOM(g_variant_builder_add_parsed, vFppV)
 GO(g_variant_builder_add_value, vFpp)
