@@ -133,7 +133,7 @@ GO(__gmpn_add_n, LFpppL)
 //GO(__gmpn_cmp, 
 GO(__gmpn_cnd_add_n, LFLpppL)
 GO(__gmpn_cnd_sub_n, LFLpppL)
-//GO(__gmpn_cnd_swap, 
+GO(__gmpn_cnd_swap, vFippL)
 //GO(__gmpn_com, 
 //GO(__gmpn_compute_powtab, 
 GO(__gmpn_copyd, vFppL)
@@ -279,25 +279,25 @@ GO(__gmpn_rshift, LFppLu)
 //GO(__gmpn_sbpi1_div_qr, 
 //GO(__gmpn_scan0, 
 //GO(__gmpn_scan1, 
-//GO(__gmpn_sec_add_1, 
-//GO(__gmpn_sec_add_1_itch, 
+GO(__gmpn_sec_add_1, LFppLL)
+GO(__gmpn_sec_add_1_itch, LFL)
 //GO(__gmpn_sec_div_qr, 
 //GO(__gmpn_sec_div_qr_itch, 
-//GO(__gmpn_sec_div_r, 
-//GO(__gmpn_sec_div_r_itch, 
-//GO(__gmpn_sec_invert, 
-//GO(__gmpn_sec_invert_itch, 
-//GO(__gmpn_sec_mul, 
-//GO(__gmpn_sec_mul_itch, 
+GO(__gmpn_sec_div_r, LFppLpLL)
+GO(__gmpn_sec_div_r_itch, LFL)
+GO(__gmpn_sec_invert, iFpppLL)
+GO(__gmpn_sec_invert_itch, LFL)
+GO(__gmpn_sec_mul, vFpppLp)
+GO(__gmpn_sec_mul_itch, LFL)
 //GO(__gmpn_sec_pi1_div_qr, 
 //GO(__gmpn_sec_pi1_div_r, 
-//GO(__gmpn_sec_powm, 
-//GO(__gmpn_sec_powm_itch, 
+GO(__gmpn_sec_powm, vFppLpLpLp)
+GO(__gmpn_sec_powm_itch, LFLL)
 //GO(__gmpn_sec_sqr, 
 //GO(__gmpn_sec_sqr_itch, 
-//GO(__gmpn_sec_sub_1, 
+GO(__gmpn_sec_sub_1, LFppLL)
 //GO(__gmpn_sec_sub_1_itch, 
-//GO(__gmpn_sec_tabselect, 
+GO(__gmpn_sec_tabselect, vFppLLL)
 //GO(__gmpn_set_str, 
 //GO(__gmpn_sizeinbase, 
 GO(__gmpn_sqr, vFppL)
@@ -441,7 +441,7 @@ GO(__gmpz_add, vFppp)
 //GO(__gmpz_addmul, 
 //GO(__gmpz_addmul_ui, 
 GO(__gmpz_add_ui, vFppL)
-//GO(__gmpz_and, 
+GO(__gmpz_and, vFppp)
 //GO(__gmpz_aorsmul_1, 
 //GO(__gmpz_array_init, 
 //GO(__gmpz_bin_ui, 
@@ -513,7 +513,7 @@ GO(__gmpz_init, vFp)
 GO(__gmpz_init_set, vFpp)
 //GO(__gmpz_init_set_d, 
 //GO(__gmpz_init_set_si, 
-//GO(__gmpz_init_set_str, 
+GO(__gmpz_init_set_str, iFppi)
 GO(__gmpz_init_set_ui, vFpL)
 //GO(__gmpz_inp_raw, 
 //GO(__gmpz_inp_str, 
@@ -523,7 +523,7 @@ GO(__gmpz_invert, iFppp)
 //GO(__gmpz_jacobi, 
 //GO(__gmpz_kronecker_si, 
 //GO(__gmpz_kronecker_ui, 
-//GO(__gmpz_lcm, 
+GO(__gmpz_lcm, vFppp)
 //GO(__gmpz_lcm_ui, 
 //GO(__gmpz_legendre, 
 GO(__gmpz_limbs_finish, vFpL)
@@ -578,11 +578,11 @@ GO(__gmpz_set_ui, vFpL)
 //GO(__gmpz_si_kronecker, 
 //GO(__gmpz_size, 
 GO(__gmpz_sizeinbase, LFpi)
-//GO(__gmpz_sqrt, 
+GO(__gmpz_sqrt, vFpp)
 //GO(__gmpz_sqrtrem, 
 //GO(__gmpz_stronglucas, 
 GO(__gmpz_sub, vFppp)
-//GO(__gmpz_submul, 
+GO(__gmpz_submul, vFppp)
 GO(__gmpz_submul_ui, vFppL)
 GO(__gmpz_sub_ui, vFppu)
 //GO(__gmpz_swap, 
@@ -592,7 +592,7 @@ GO(__gmpz_tdiv_qr, vFpppp)
 //GO(__gmpz_tdiv_qr_ui, 
 //GO(__gmpz_tdiv_q_ui, 
 //GO(__gmpz_tdiv_r, 
-//GO(__gmpz_tdiv_r_2exp, 
+GO(__gmpz_tdiv_r_2exp, vFppL)
 //GO(__gmpz_tdiv_r_ui, 
 //GO(__gmpz_tdiv_ui, 
 GO(__gmpz_tstbit, iFpL)
