@@ -293,6 +293,29 @@ static void* findWeakNotifyFct(void* fct)
     return NULL;
 }
 
+// GToggleNotify
+#define GO(A)   \
+static uintptr_t my_togglenotifyfunc_fct_##A = 0;                       \
+static int my_togglenotifyfunc_##A(void* a, void* b, int c)             \
+{                                                                       \
+    return RunFunctionFmt(my_togglenotifyfunc_fct_##A, "ppi", a, b, c); \
+}
+SUPER()
+#undef GO
+static void* findToggleNotifyFct(void* fct)
+{
+    if(!fct) return fct;
+    if(GetNativeFnc((uintptr_t)fct))  return GetNativeFnc((uintptr_t)fct);
+    #define GO(A) if(my_togglenotifyfunc_fct_##A == (uintptr_t)fct) return my_togglenotifyfunc_##A;
+    SUPER()
+    #undef GO
+    #define GO(A) if(my_togglenotifyfunc_fct_##A == 0) {my_togglenotifyfunc_fct_##A = (uintptr_t)fct; return my_togglenotifyfunc_##A; }
+    SUPER()
+    #undef GO
+    printf_log(LOG_NONE, "Warning, no more slot for gobject GToggleNotify callback\n");
+    return NULL;
+}
+
 // GParamSpecTypeInfo....
 // First the structure GParamSpecTypeInfo statics, with paired x64 source pointer
 typedef struct my_GParamSpecTypeInfo_s {
@@ -432,6 +455,50 @@ static void* findcompareFct(void* fct)
     printf_log(LOG_NONE, "Warning, no more slot for gobject compare callback\n");
     return NULL;
 }
+
+// GTypeModuleClass
+#define GO(A)   \
+static uintptr_t my_load_fct_##A = 0;                         \
+static int my_load_##A(my_GTypeModule_t* module)              \
+{                                                             \
+    return (int)RunFunctionFmt(my_load_fct_##A, "p", module); \
+}
+SUPER()
+#undef GO
+static void* findLoadFct(void* fct)
+{
+    if(!fct) return fct;
+    #define GO(A) if(my_load_fct_##A == (uintptr_t)fct) return my_load_##A;
+    SUPER()
+    #undef GO
+    #define GO(A) if(my_load_fct_##A == 0) {my_load_fct_##A = (uintptr_t)fct; return my_load_##A; }
+    SUPER()
+    #undef GO
+    printf_log(LOG_NONE, "Warning, no more slot for gobject GTypeModuleClass load callback\n");
+    return NULL;
+}
+
+#define GO(A)   \
+static uintptr_t my_unload_fct_##A = 0;                         \
+static int my_unload_##A(my_GTypeModule_t* module)              \
+{                                                             \
+    return (int)RunFunctionFmt(my_unload_fct_##A, "p", module); \
+}
+SUPER()
+#undef GO
+static void* findUnloadFct(void* fct)
+{
+    if(!fct) return fct;
+    #define GO(A) if(my_unload_fct_##A == (uintptr_t)fct) return my_unload_##A;
+    SUPER()
+    #undef GO
+    #define GO(A) if(my_unload_fct_##A == 0) {my_unload_fct_##A = (uintptr_t)fct; return my_unload_##A; }
+    SUPER()
+    #undef GO
+    printf_log(LOG_NONE, "Warning, no more slot for gobject GTypeModuleClass unload callback\n");
+    return NULL;
+}
+
 #undef SUPER
 
 #include "super100.h"
@@ -803,6 +870,21 @@ EXPORT void my_g_object_weak_ref(x64emu_t* emu, void* object, void* notify, void
     my->g_object_weak_ref(object, findWeakNotifyFct(notify), data);
 }
 
+EXPORT void my_g_object_weak_unref(x64emu_t* emu, void* object, void* notify, void* data)
+{
+    my->g_object_weak_unref(object, findWeakNotifyFct(notify), data);
+}
+
+EXPORT void my_g_object_add_toggle_ref(x64emu_t* emu, void* object, void* notify, void* data)
+{
+    my->g_object_add_toggle_ref(object, findToggleNotifyFct(notify), data);
+}
+
+EXPORT void my_g_object_remove_toggle_ref(x64emu_t* emu, void* object, void* notify, void* data)
+{
+    my->g_object_remove_toggle_ref(object, findToggleNotifyFct(notify), data);
+}
+
 EXPORT void my_g_signal_override_class_handler(x64emu_t* emu, char* name, size_t gtype, void* callback)
 {
     my->g_signal_override_class_handler(name, gtype, findGCallbackFct(callback));
@@ -838,6 +920,33 @@ EXPORT void* my_g_type_value_table_peek(x64emu_t* emu, size_t type)
     return findFreeGTypeValueTable(my->g_type_value_table_peek(type));
 }
 
+EXPORT int my_g_type_module_use(x64emu_t* emu, my_GTypeModule_t* module)
+{
+    my_GTypeModuleClass_t* module_class = (my_GTypeModuleClass_t*)(((my_GTypeInstance_t*)module)->g_class);
+    if (module_class) {
+        module_class->load = findLoadFct(module_class->load);
+        module_class->unload = findUnloadFct(module_class->unload);
+    }
+    return my->g_type_module_use(module);
+}
+
+EXPORT void my_g_type_module_add_interface(x64emu_t* emu, my_GTypeModule_t* module, size_t instance_type, size_t interface_type, my_GInterfaceInfo_t* interface_info)
+{
+    if (interface_info) {
+        interface_info->interface_init = findGInterfaceInitFuncFct(interface_info->interface_init, interface_type);
+        interface_info->interface_finalize = findGInterfaceFinalizeFuncFct(interface_info->interface_finalize);
+    }
+    return my->g_type_module_add_interface(module, instance_type, interface_type, interface_info);
+}
+
+EXPORT size_t my_g_type_module_register_type(x64emu_t* emu, my_GTypeModule_t* module, size_t parent_type, char* type_name, my_GTypeInfo_t* type_info, uint32_t flags)
+{
+    if (type_info) {
+        type_info->class_init = find_class_init_Fct(type_info->class_init, parent_type);
+    }
+    return my->g_type_module_register_type(module, parent_type, type_name, type_info, flags);
+}
+
 #define PRE_INIT    \
     if(BOX64ENV(nogtk)) \
         return -1;
