@@ -77,15 +77,15 @@ GOM(g_closure_add_finalize_notifier, vFEppp)
 //GOM(g_closure_add_marshal_guards, vFppppp)
 GO(g_closure_get_type, LFv)
 //GOM(g_closure_invalidate, vFp)
-//GOM(g_closure_invoke, vFppupp)
+GO(g_closure_invoke, vFppupp)
 //GOM(g_closure_new_object, pFup)
 GO(g_closure_new_simple, pFup)
-//GOM(g_closure_ref, pFp)
+GO(g_closure_ref, pFp)
 GOM(g_closure_remove_finalize_notifier, vFEppp)
 //GOM(g_closure_remove_invalidate_notifier, vFppp)
 GOM(g_closure_set_marshal, vFEpp)
 //GOM(g_closure_set_meta_marshal, vFppp)
-//GOM(g_closure_sink, vFp)
+GO(g_closure_sink, vFp)
 GO(g_closure_unref, vFp)
 GO(g_date_get_type, LFv)
 GO(g_date_time_get_type, LFv)
@@ -112,7 +112,7 @@ GO(g_main_loop_get_type, LFv)
 GO(g_mapped_file_get_type, LFv)
 GO(g_markup_parse_context_get_type, LFv)
 GO(g_match_info_get_type, LFv)
-//GOM(g_object_add_toggle_ref, vFEppp)
+GOM(g_object_add_toggle_ref, vFEppp)
 GO(g_object_add_weak_pointer, vFpp)
 GO(g_object_bind_property, pFppppu)
 //GOM(g_object_bind_property_full, pFppppupppp)
@@ -146,7 +146,7 @@ GO(g_object_notify, vFpp)
 GO(g_object_notify_by_pspec, vFpp)
 GO(g_object_ref, pFp)
 GO(g_object_ref_sink, pFp)
-//GOM(g_object_remove_toggle_ref, vFEppp)
+GOM(g_object_remove_toggle_ref, vFEppp)
 GO(g_object_remove_weak_pointer, vFpp)
 //GOM(g_object_replace_data, iFEpppppp)
 //GOM(g_object_replace_qdata, iFEpupppp)
@@ -164,7 +164,7 @@ GO(g_object_thaw_notify, vFp)
 GO(g_object_unref, vFp)
 GO(g_object_watch_closure, vFpp)    // needs wrapping?
 GOM(g_object_weak_ref, vFEppp)
-GO(g_object_weak_unref, vFppp)  // needs wrapping?
+GOM(g_object_weak_unref, vFEppp)
 GO(g_param_spec_boolean, pFpppii)
 GO(g_param_spec_boxed, pFpppLi)
 GO(g_param_spec_char, pFpppccci)
@@ -315,14 +315,14 @@ GO(g_type_interface_peek_parent, pFp)
 GO(g_type_interface_prerequisites, pFLp)
 GO(g_type_interfaces, pFLp)
 GO(g_type_is_a, iFLL)
-//GOM(g_type_module_add_interface, vFpLLp)
+GOM(g_type_module_add_interface, vFEpLLp)
 GO(g_type_module_get_type, LFv)
 GO(g_type_module_register_enum, LFppp)
 GO(g_type_module_register_flags, LFppp)
-//GOM(g_type_module_register_type, LFpLppu)
+GOM(g_type_module_register_type, LFEpLppu)
 GO(g_type_module_set_name, vFpp)
 GO(g_type_module_unuse, vFp)
-GO(g_type_module_use, iFp)
+GOM(g_type_module_use, iFEp)
 GO(g_type_name, pFL)
 GO(g_type_name_from_class, pFp)
 GO(g_type_name_from_instance, pFp)
