@@ -36,7 +36,7 @@ typedef size_t  (*LFv_t)();
 #include "wrappercallback.h"
 
 #define PRE_INIT    \
-    if(box64_nogtk) \
+    if(BOX64ENV(nogtk)) \
         return -1;
 
 #define CUSTOM_INIT \
