@@ -446,7 +446,7 @@ GO(gst_debug_message_get, pFp)
 DATAB(_gst_debug_min, 4)
 //GOM(_gst_debug_nameof_funcptr, pFEp)
 GO(gst_debug_print_stack_trace, vFv)
-GOM(_gst_debug_register_funcptr, vFpp)  //no need to wrap it seems, it's the name that get registered
+GO(_gst_debug_register_funcptr, vFpp)  //no need to wrap it seems, it's the name that get registered
 //GOM(gst_debug_remove_log_function, uFEp)
 GO(gst_debug_remove_log_function_by_data, uFp)
 GO(gst_debug_remove_ring_buffer_logger, vFv)
@@ -743,7 +743,7 @@ GO(gst_is_caps_features, iFp)
 GO(gst_is_initialized, iFv)
 GO(gst_iterator_copy, pFp)
 //GOM(gst_iterator_filter, pFEppp)
-//GOM(gst_iterator_find_custom, iFEpppp)
+GOM(gst_iterator_find_custom, iFEpppp)
 GOM(gst_iterator_fold, uFEpppp)
 //GOM(gst_iterator_foreach, uFEppp)
 GO(gst_iterator_free, vFp)
