@@ -1277,7 +1277,7 @@ static void addGtk2Alternate(library_t* lib)
 }
 
 #define PRE_INIT    \
-    if(box64_nogtk) \
+    if(BOX64ENV(nogtk)) \
         return -1;
 
 #define CUSTOM_INIT \
