@@ -4,7 +4,7 @@
 
 DATAB(alsa_lisp_nil, 4)
 DATAB(alsa_lisp_t, 4)
-DATAB(snd_config, 4)
+DATAB(snd_config, 8)
 DATAB(_snd_config_hook_load_dlsym_config_hook_001, 4)
 DATAB(_snd_config_hook_load_for_all_cards_dlsym_config_hook_001, 4)
 DATAB(__snd_ctl_hw_open_dlsym_control_001, 4)
@@ -72,20 +72,20 @@ GO(snd_shm_area_share, pFp)
 GO(snd_shm_area_destroy, iFp)
 GO(snd_user_file, iFpp)
 GO(snd_input_stdio_open, iFppp)
-GO(snd_input_stdio_attach, iFppi)
+GO(snd_input_stdio_attach, iFpSi)
 GO(snd_input_buffer_open, iFppl)
 GO(snd_input_close, iFp)
-//GOM(snd_input_scanf, iFppV)
+//GOM(snd_input_scanf, iFEppV)
 GO(snd_input_gets, pFppL)
 GO(snd_input_getc, iFp)
 GO(snd_input_ungetc, iFpi)
 GO(snd_output_stdio_open, iFppp)
-GO(snd_output_stdio_attach, iFppi)
+GO(snd_output_stdio_attach, iFpSi)
 GO(snd_output_buffer_open, iFp)
 GO(snd_output_buffer_string, LFpp)
 GO(snd_output_close, iFp)
-//GOM(snd_output_printf, iFppV)
-//GOM(snd_output_vprintf, iFppA)
+//GOM(snd_output_printf, iFEppV)
+//GOM(snd_output_vprintf, iFEppA)
 GO(snd_output_puts, iFpp)
 GO(snd_output_putc, iFpi)
 GO(snd_output_flush, iFp)
@@ -100,7 +100,7 @@ GO(snd_config_update_r, iFppp)
 GO(snd_config_update_free, iFp)
 GO(snd_config_update_free_global, iFv)
 GO(snd_config_search, iFppp)
-//GOM(snd_config_searchv, iFppV)
+//GOM(snd_config_searchv, iFEppV)
 GO(snd_config_search_definition, iFpppp)
 GO(snd_config_expand, iFppppp)
 GO(snd_config_evaluate, iFpppp)
@@ -145,12 +145,14 @@ GO(snd_config_get_bool_ascii, iFp)
 GO(snd_config_get_bool, iFp)
 GO(snd_config_get_ctl_iface_ascii, iFp)
 GO(snd_config_get_ctl_iface, iFp)
+GO(snd_ctl_ascii_value_parse, iFpppp)
 GO(snd_device_name_free_hint, iFp)
 GO(snd_device_name_get_hint, pFpp)
 GO(snd_device_name_hint, iFipp)
 GO(snd_names_list, iFpp)
 GO(snd_names_list_free, vFp)
 GO(snd_pcm_format_mask_sizeof, LFv)
+GO(snd_pcm_free_chmaps, vFp)
 GO(snd_pcm_subformat_mask_sizeof, LFv)
 GO(snd_pcm_status_sizeof, LFv)
 GOM(snd_async_add_pcm_handler, iFEpppp)
@@ -407,6 +409,7 @@ GO(snd_pcm_poll_descriptors, iFppu)
 GO(snd_pcm_poll_descriptors_count, iFp)
 GO(snd_pcm_poll_descriptors_revents, iFppup)
 GO(snd_pcm_prepare, iFp)
+GO(snd_pcm_query_chmaps, pFp)
 GO(snd_pcm_readi, lFppL)
 GO(snd_pcm_readn, lFppL)
 GO(snd_pcm_recover, iFpii)
@@ -560,7 +563,7 @@ GO(snd_timer_query_status, iFpp)
 GO(snd_timer_open, iFppi)
 GO(snd_timer_open_lconf, iFppip)
 GO(snd_timer_close, iFp)
-//GOM(snd_async_add_timer_handler, iFpppp)
+//GOM(snd_async_add_timer_handler, iFEpppp)
 GO(snd_async_handler_get_timer, pFp)
 GO(snd_timer_poll_descriptors_count, iFp)
 GO(snd_timer_poll_descriptors, iFppu)
@@ -677,7 +680,7 @@ GO(snd_ctl_open, iFppi)
 GO(snd_ctl_open_lconf, iFppip)
 GO(snd_ctl_close, iFp)
 GO(snd_ctl_nonblock, iFpi)
-//GOM(snd_async_add_ctl_handler, iFpppp)
+//GOM(snd_async_add_ctl_handler, iFEpppp)
 GO(snd_async_handler_get_ctl, pFp)
 GO(snd_ctl_poll_descriptors_count, iFp)
 GO(snd_ctl_poll_descriptors, iFppu)
@@ -860,11 +863,11 @@ GO(snd_hctl_poll_descriptors_count, iFp)
 GO(snd_hctl_poll_descriptors, iFppu)
 GO(snd_hctl_poll_descriptors_revents, iFppup)
 GO(snd_hctl_get_count, uFp)
-//GOM(snd_hctl_set_compare, iFpp)
+//GOM(snd_hctl_set_compare, iFEpp)
 GO(snd_hctl_first_elem, pFp)
 GO(snd_hctl_last_elem, pFp)
 GO(snd_hctl_find_elem, pFpp)
-//GOM(snd_hctl_set_callback, vFpp)
+//GOM(snd_hctl_set_callback, vFEpp)
 GO(snd_hctl_set_callback_private, vFpp)
 GO(snd_hctl_get_callback_private, pFp)
 GO(snd_hctl_load, iFp)
@@ -889,7 +892,7 @@ GO(snd_hctl_elem_get_device, uFp)
 GO(snd_hctl_elem_get_subdevice, uFp)
 GO(snd_hctl_elem_get_name, pFp)
 GO(snd_hctl_elem_get_index, uFp)
-//GOM(snd_hctl_elem_set_callback, vFpp)
+//GOM(snd_hctl_elem_set_callback, vFEpp)
 GO(snd_hctl_elem_get_callback_private, pFp)
 GO(snd_hctl_elem_set_callback_private, vFpp)
 GO(snd_sctl_build, iFppppi)
@@ -913,7 +916,7 @@ GO(snd_mixer_load, iFp)
 GO(snd_mixer_free, vFp)
 GO(snd_mixer_wait, iFpi)
 GOM(snd_mixer_set_compare, iFEpp)
-//GOM(snd_mixer_set_callback, vFpp)
+//GOM(snd_mixer_set_callback, vFEpp)
 GO(snd_mixer_get_callback_private, pFp)
 GO(snd_mixer_set_callback_private, vFpp)
 GO(snd_mixer_get_count, uFp)
@@ -1276,4 +1279,4 @@ GO(snd_seq_parse_address, iFppp)
 
 GO(snd_hwdep_info_get_iface, uFp)
 DATAM(snd_lib_error, 8)
-GO(superdummy_error_handler, vFpipipV)    // dummy to have the bridge definition
+GO(superdummy_error_handler, vFpipipV)
