@@ -17,18 +17,18 @@
 #include "x64tls.h"
 
 
-#ifdef ANDROID
-const char* libbsdName = "libbsd.so";
-#else
 const char* libbsdName = "libbsd.so.0";
-#endif
+#define ALTNAME "libbsd.so"
 #define LIBNAME libbsd
 
 #ifdef STATICBUILD
-void arc4random_addrandom(unsigned char *dat, int datlen);
-void arc4random_stir(void);
-const char *getprogname(void);
-void setprogname(const char *);
+extern uint32_t arc4random(void);
+extern void arc4random_addrandom(unsigned char *dat, int datlen);
+extern void arc4random_buf(void *buf, size_t nbytes);
+extern void arc4random_stir(void);
+extern uint32_t arc4random_uniform(uint32_t upper_bound);
+extern const char *getprogname(void);
+extern void setprogname(const char *);
 #endif
 
 #ifndef STATICBUILD
