@@ -14,7 +14,6 @@
 const char* libformwName = "libformw.so.5";
 #define LIBNAME libformw
 
-#define CUSTOM_INIT \
-    SETALT(myw_);   \
+#define ALTMY myw_
 
 #include "wrappedlib_init.h"
