@@ -32,7 +32,7 @@ GOM(lio_listio, iFEipip)
 GO(mq_close, iFi)
 GO(mq_getattr, iFip)
 GO(mq_notify, iFip)
-GO(mq_open, iFpOM)
+GO(mq_open, iFpONN)
 GO(__mq_open_2, iFpO)
 GO(mq_receive, lFipLp)
 GO(mq_send, iFipLu)
