@@ -27,29 +27,29 @@ const char* libsmName = "libSM.so.6";
 
 typedef struct my_SmcCallbacks_s {
     struct {
-	void*	 callback;
-	void*	 client_data;
+        void*    callback;
+        void*    client_data;
     } save_yourself;
 
     struct {
-	void*	 callback;
-	void*	 client_data;
+        void*    callback;
+        void*    client_data;
     } die;
 
     struct {
-	void*	 callback;
-	void*		 client_data;
+        void*    callback;
+        void*    client_data;
     } save_complete;
 
     struct {
-	void* callback;
-	void*		 client_data;
+        void*   callback;
+        void*    client_data;
     } shutdown_cancelled;
 } my_SmcCallbacks_t;
-#define SmcSaveYourselfProcMask		    (1L << 0)
-#define SmcDieProcMask			        (1L << 1)
-#define SmcSaveCompleteProcMask		    (1L << 2)
-#define SmcShutdownCancelledProcMask	(1L << 3)
+#define SmcSaveYourselfProcMask         (1L << 0)
+#define SmcDieProcMask                  (1L << 1)
+#define SmcSaveCompleteProcMask         (1L << 2)
+#define SmcShutdownCancelledProcMask    (1L << 3)
 
 static uintptr_t my_save_yourself_fct = 0;
 static void my_save_yourself(void* smcConn, void* clientData, int saveType, int shutdown, int interactStyle, int fast)
@@ -120,6 +120,39 @@ static void* findRequestFct(void* fct)
     printf_log(LOG_NONE, "Warning, no more slot for libSM Request callback\n");
     return NULL;
 }
+// SmcErrorHandler
+#define GO(A)   \
+static uintptr_t my_SmcErrorHandler_fct_##A = 0;        \
+static void my_SmcErrorHandler_##A(void* a, int b, int c, unsigned long d, int e, int f, void* g)     \
+{                                               \
+    RunFunctionFmt(my_SmcErrorHandler_fct_##A, "piiLiip", a, b, c, d, e, f, g);\
+}
+SUPER()
+#undef GO
+static void* findSmcErrorHandlerFct(void* fct)
+{
+    if(!fct) return NULL;
+    void* p;
+    if((p = GetNativeFnc((uintptr_t)fct))) return p;
+    #define GO(A) if(my_SmcErrorHandler_fct_##A == (uintptr_t)fct) return my_SmcErrorHandler_##A;
+    SUPER()
+    #undef GO
+    #define GO(A) if(my_SmcErrorHandler_fct_##A == 0) {my_SmcErrorHandler_fct_##A = (uintptr_t)fct; return my_SmcErrorHandler_##A; }
+    SUPER()
+    #undef GO
+    printf_log(LOG_NONE, "Warning, no more slot for libSM SmcErrorHandler callback\n");
+    return NULL;
+}
+static void* reverse_SmcErrorHandler_Fct(void* fct)
+{
+    if(!fct) return fct;
+    if(CheckBridged(my_lib->w.bridge, fct))
+        return (void*)CheckBridged(my_lib->w.bridge, fct);
+    #define GO(A) if(my_SmcErrorHandler_##A == fct) return (void*)my_SmcErrorHandler_fct_##A;
+    SUPER()
+    #undef GO
+    return (void*)AddBridge(my_lib->w.bridge, vFpiiLiip, fct, 0, NULL);
+}
 
 #undef SUPER
 
@@ -135,10 +168,9 @@ EXPORT int my_SmcRequestSaveYourselfPhase2(x64emu_t* emu, void* smcConn, void* c
     return my->SmcRequestSaveYourselfPhase2(smcConn, findRequestFct(cb), data);
 }
 
-#define CUSTOM_INIT \
-    getMy(lib);
-
-#define CUSTOM_FINI \
-    freeMy();
+EXPORT void* my_SmcSetErrorHandler(x64emu_t* emu, void* f)
+{
+    return reverse_SmcErrorHandler_Fct(my->SmcSetErrorHandler(findSmcErrorHandlerFct(f)));
+}
 
 #include "wrappedlib_init.h"
