@@ -28,7 +28,7 @@ GO(libusb_free_pollfds, vFp)
 GO(libusb_free_ss_endpoint_companion_descriptor, vFp)
 GO(libusb_free_ss_usb_device_capability_descriptor, vFp)
 GO(libusb_free_streams, iFppi)
-GOM(libusb_free_transfer, vFp)
+GOM(libusb_free_transfer, vFEp)
 GO(libusb_free_usb_2_0_extension_descriptor, vFp)
 GO(libusb_get_active_config_descriptor, iFpp)
 GO(libusb_get_active_interface_association_descriptors, iFpp)
