@@ -17,11 +17,8 @@
 #include "librarian.h"
 #include "myalign.h"
 
-#ifdef ANDROID
-	const char* libvorbisName = "libvorbis.so";
-#else
-	const char* libvorbisName = "libvorbis.so.0";
-#endif
+const char* libvorbisName = "libvorbis.so.0";
+#define ALTNAME "libvorbis.so"
 
 #define LIBNAME libvorbis
 
