@@ -17,11 +17,8 @@
 #include "box64context.h"
 #include "emu/x64emu_private.h"
 
-#ifdef ANDROID
-	const char* libxcbicccmName = "libxcb-icccm.so";
-#else
-	const char* libxcbicccmName = "libxcb-icccm.so.4";
-#endif
+const char* libxcbicccmName = "libxcb-icccm.so.4";
+#define ALTNAME "libxcb-icccm.so"
 
 #define LIBNAME libxcbicccm
 
