@@ -11,7 +11,7 @@ GO(xcb_is_pf_key, iFu)
 GO(xcb_is_private_keypad_key, iFu)
 GO(xcb_key_press_lookup_keysym, uFppi)
 GO(xcb_key_release_lookup_keysym, uFppi)
-GO(xcb_key_symbols_alloc, pFp)
+GO(xcb_key_symbols_alloc, pFb)
 GO(xcb_key_symbols_free, vFp)
 GO(xcb_key_symbols_get_keycode, pFpu)
 GO(xcb_key_symbols_get_keysym, uFpCi)
