@@ -11,11 +11,8 @@
 #include "librarian/library_private.h"
 #include "x64emu.h"
 
-#ifdef ANDROID
-	const char* libxdmcpName = "libXdmcp.so";
-#else
-	const char* libxdmcpName = "libXdmcp.so.6";
-#endif
+const char* libxdmcpName = "libXdmcp.so.6";
+#define ALTNAME "libXdmcp.so"
 
 #define LIBNAME libxdmcp
 
