@@ -22,10 +22,6 @@ const char* libxxf86vmName = "libXxf86vm.so.1";
 
 #define LIBNAME libxxf86vm
 
-#ifdef ANDROID
-#define NEEDED_LIBS "libX11.so", "libXext.so"
-#else
 #define NEEDED_LIBS "libX11.so.6", "libXext.so.6"
-#endif
 
 #include "wrappedlib_init.h"
