@@ -1,29 +1,29 @@
 #if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
-#error meh!
+#error Meh...
 #endif
 
 GO(pango_fc_decoder_get_charset, pFpp)
 GO(pango_fc_decoder_get_glyph, uFppu)
-GO(pango_fc_decoder_get_type, iFv)
+GO(pango_fc_decoder_get_type, LFv)
 GO(pango_fc_font_create_base_metrics_for_context, pFpp)
 GO(pango_fc_font_description_from_pattern, pFpi)
 GO(pango_fc_font_get_glyph, uFpu)
 GO(pango_fc_font_get_raw_extents, vFpupp)
-GO(pango_fc_font_get_type, iFv)
+GO(pango_fc_font_get_type, LFv)
 GO(pango_fc_font_get_unknown_glyph, uFpu)
 GO(pango_fc_font_has_char, iFpu)
 GO(pango_fc_font_kern_glyphs, vFpp)
 GO(pango_fc_font_key_get_context_key, pFp)
 GO(pango_fc_font_key_get_matrix, pFp)
 GO(pango_fc_font_key_get_pattern, pFp)
-GO(pango_fc_font_lock_face, pFp)
-//GOM(pango_fc_font_map_add_decoder_find_func, vFEpBpp)
+GOM(pango_fc_font_lock_face, pFp)
+//GOM(pango_fc_font_map_add_decoder_find_func, vFEpppp)
 GO(pango_fc_font_map_cache_clear, vFp)
 GO(pango_fc_font_map_config_changed, vFp)
 GO(pango_fc_font_map_create_context, pFp)
 GO(pango_fc_font_map_find_decoder, pFpp)
 GO(pango_fc_font_map_get_config, pFp)
-GO(pango_fc_font_map_get_type, iFv)
+GO(pango_fc_font_map_get_type, LFv)
 GO(pango_fc_font_map_shutdown, vFp)
 GO(pango_fc_font_map_set_config, vFpp)
 GO(pango_fc_fontset_key_get_absolute_size, dFp)
@@ -41,7 +41,7 @@ GO(pango_ft2_font_map_create_context, pFp)
 GO(pango_ft2_font_map_for_display, pFv)
 GO(pango_ft2_font_map_get_type, LFv)
 GO(pango_ft2_font_map_new, pFv)
-//GOM(pango_ft2_font_map_set_default_substitute, vFEpBpB)
+//GOM(pango_ft2_font_map_set_default_substitute, vFEpppp)
 GO(pango_ft2_font_map_set_resolution, vFpdd)
 GO(pango_ft2_font_map_substitute_changed, vFp)
 GO(pango_ft2_get_context, pFdd)
