@@ -13,6 +13,7 @@ GO(png_free, vFpp)
 GO(png_get_bit_depth, CFpp)
 GO(png_get_color_type, CFpp)
 GO(png_get_error_ptr, pFp)
+GO(png_get_header_version, CFp)
 GO(png_get_iCCP, uFpppppp)
 GO(png_get_IHDR, uFppppppppp)
 GO(png_get_image_height, uFpp)
@@ -50,6 +51,7 @@ GO(png_set_pHYs, vFppuui)
 GOM(png_set_read_fn, vFEppp)
 GOM(png_set_read_user_transform_fn, vFEpp)
 GO(png_set_rows, vFppp)
+GO(png_set_shift, vFpC)
 GO(png_set_sig_bytes, vFpi)
 GO(png_set_strip_16, vFp)
 GO(png_set_tIME, vFppp)
