@@ -80,6 +80,7 @@ GO(pa_context_ref, pFp)
 //GOM(pa_context_remove_autoload_by_name, pFppupp)
 //GOM(pa_context_remove_sample, pFpppp)
 GOM(pa_context_rttime_new, pFEpUpp)
+GO(pa_context_rttime_restart, vFppU)
 GOM(pa_context_set_card_profile_by_index, pFEpuppp)
 //GOM(pa_context_set_card_profile_by_name, pFppppp)
 GOM(pa_context_set_default_sink, pFEpppp)
@@ -163,6 +164,7 @@ GO(pa_msleep, iFL)
 GO(pa_operation_cancel, vFp)
 GO(pa_operation_get_state, uFp)
 GO(pa_operation_ref, pFp)
+GOM(pa_operation_set_state_callback, vFEppp)
 GO(pa_operation_unref, vFp)
 GO(pa_parse_sample_format, iFp)
 GO(pa_path_get_filename, pFp)
