@@ -66,12 +66,6 @@ void EXPORT *my_IMG_Load_RW(x64emu_t* emu, void* a, int32_t b)
     return r;
 }
 
-#define CUSTOM_INIT \
-    getMy(lib);   \
-    setNeededLibs(lib, 2, "libSDL-1.2.so.0", "libz.so.1");
-
-#define CUSTOM_FINI \
-    freeMy();
+#define NEEDED_LIBS "libSDL-1.2.so.0", "libz.so.1"
 
 #include "wrappedlib_init.h"
-
