@@ -189,7 +189,7 @@ GO(SDL_GetClipRect, vFpp)
 GO(SDL_GetClosestDisplayMode, pFipp)
 GO(SDL_GetColorKey, iFpp)
 GO(SDL_GetCPUCacheLineSize, iFv)
-GO(SDL_GetCPUCount, iFv)
+GOM(SDL_GetCPUCount, iFEv)
 GO(SDL_GetCurrentAudioDriver, pFv)
 GO(SDL_GetCurrentDisplayMode, iFip)
 GO(SDL_GetCurrentVideoDriver, pFv)
@@ -533,6 +533,7 @@ GO(SDL_RenderFillRect, iFpp)
 GO(SDL_RenderFillRects, iFppi)
 // SDL_RenderFillRectsF
 // SDL_RenderFlush
+GO(SDL_RenderGeometryRaw, iFpppipipiipii)
 GO(SDL_RenderGetClipRect, vFpp)
 GO(SDL_RenderGetIntegerScale, iFp)
 GO(SDL_RenderGetLogicalSize, vFppp)
