@@ -47,6 +47,7 @@ GO(Mix_Linked_Version,pFv)
 GO(Mix_LoadMUS,pFp)
 GOM(Mix_LoadMUSType_RW,pFEpii)
 GOM(Mix_LoadMUS_RW,pFEpi)
+GO(Mix_LoadWAV, pFp)
 GOM(Mix_LoadWAV_RW,pFEpi)
 GO(Mix_OpenAudio, iFiWii)
 GO(Mix_OpenAudioDevice, iFiWiipi)
@@ -54,6 +55,7 @@ GO(Mix_Pause,vFi)
 GO(Mix_PauseMusic,vFv)
 GO(Mix_Paused,iFi)
 GO(Mix_PausedMusic,iFv)
+GO(Mix_PlayChannel, iFipi)
 GO(Mix_PlayChannelTimed,iFipii)
 GO(Mix_PlayMusic,iFpi)
 GO(Mix_Playing,iFi)
