@@ -74,11 +74,6 @@ EXPORT void* my2_SMPEG_new_rwops(x64emu_t* emu, void* src, void* info, int32_t f
     return ret;
 }
 
-#define CUSTOM_INIT \
-    getMy(lib); \
-    SETALT(my2_);
-
-#define CUSTOM_FINI \
-    freeMy();
+#define ALTMY my2_
 
 #include "wrappedlib_init.h"
