@@ -29,8 +29,9 @@ typedef void(*vFpUp_t)      (void*, uint64_t, void*);
 
 #include "generated/wrappedvulkantypes.h"
 
+// track current instance and device.
 #define ADDED_STRUCT()                              \
-    void* currentInstance;  // track current instance. If using multiple instance, that will be a mess!
+    void* currentInstance;
 
 #define ADDED_SUPER 1
 #include "wrappercallback.h"
@@ -60,7 +61,7 @@ static symbol1_t* getWrappedSymbol(x64emu_t* emu, const char* rname, int warning
     }
     if(k==kh_end(emu->context->vkwrappers)) {
         if(warning) {
-            printf_dlsym(LOG_DEBUG, "%p\n", NULL);
+            printf_dlsym_prefix(0, LOG_DEBUG, "%p\n", NULL);
             printf_dlsym(LOG_INFO, "Warning, no wrapper for %s\n", rname);
         }
         return NULL;
@@ -79,7 +80,7 @@ static void* resolveSymbol(x64emu_t* emu, void* symbol, const char* rname)
         s->resolved = 1;
     }
     void* ret = (void*)s->addr;
-    printf_dlsym(LOG_DEBUG, "%p (%p)\n", ret, symbol);
+    printf_dlsym_prefix(0, LOG_DEBUG, "%p (%p)\n", ret, symbol);
     return ret;
 }
 
@@ -91,12 +92,15 @@ EXPORT void* my_vkGetDeviceProcAddr(x64emu_t* emu, void* device, void* name)
     printf_dlsym(LOG_DEBUG, "Calling my_vkGetDeviceProcAddr(%p, \"%s\") => ", device, rname);
     if(!emu->context->vkwrappers)
         fillVulkanProcWrapper(emu->context);
+    // not caching Device functions
+    /*
     symbol1_t* s = getWrappedSymbol(emu, rname, 0);
     if(s && s->resolved) {
         void* ret = (void*)s->addr;
-        printf_dlsym(LOG_DEBUG, "%p (cached)\n", ret);
+        printf_dlsym_prefix(0, LOG_DEBUG, "%p (cached)\n", ret);
         return ret;
     }
+    */
     k = kh_get(symbolmap, emu->context->vkmymap, rname);
     int is_my = (k==kh_end(emu->context->vkmymap))?0:1;
     void* symbol = my->vkGetDeviceProcAddr(device, name);
@@ -112,7 +116,7 @@ EXPORT void* my_vkGetDeviceProcAddr(x64emu_t* emu, void* device, void* name)
         #undef GO
     }
     if(!symbol) {
-        printf_dlsym(LOG_DEBUG, "%p\n", NULL);
+        printf_dlsym_prefix(0, LOG_DEBUG, "%p\n", NULL);
         return NULL;    // easy
     }
     return resolveSymbol(emu, symbol, rname);
@@ -127,13 +131,14 @@ EXPORT void* my_vkGetInstanceProcAddr(x64emu_t* emu, void* instance, void* name)
     if(!emu->context->vkwrappers)
         fillVulkanProcWrapper(emu->context);
     if(instance!=my->currentInstance) {
+        printf_dlsym(LOG_DEBUG, "(switch instance from %p to %p) ", my->currentInstance, instance);
         my->currentInstance = instance;
         updateInstance(emu, my);
     }
     symbol1_t* s = getWrappedSymbol(emu, rname, 0);
     if(s && s->resolved) {
         void* ret = (void*)s->addr;
-        printf_dlsym(LOG_DEBUG, "%p (cached)\n", ret);
+        printf_dlsym_prefix(0, LOG_DEBUG, "%p (cached)\n", ret);
         return ret;
     }
     // check if vkprocaddress is filled, and search for lib and fill it if needed
@@ -142,7 +147,7 @@ EXPORT void* my_vkGetInstanceProcAddr(x64emu_t* emu, void* instance, void* name)
     int is_my = (k==kh_end(emu->context->vkmymap))?0:1;
     void* symbol = my_context->vkprocaddress(instance, rname);
     if(!symbol) {
-        printf_dlsym(LOG_DEBUG, "%p\n", NULL);
+        printf_dlsym_prefix(0, LOG_DEBUG, "%p\n", NULL);
         return NULL;    // easy
     }
     if(is_my) {
@@ -170,7 +175,7 @@ void* my_GetVkProcAddr(x64emu_t* emu, void* name, void*(*getaddr)(const char*))
     symbol1_t* s = getWrappedSymbol(emu, rname, 0);
     if(s && s->resolved) {
         void* ret = (void*)s->addr;
-        printf_dlsym(LOG_DEBUG, "%p (cached)\n", ret);
+        printf_dlsym_prefix(0, LOG_DEBUG, "%p (cached)\n", ret);
         return ret;
     }
     // check if vkprocaddress is filled, and search for lib and fill it if needed
@@ -827,4 +832,28 @@ EXPORT void my_vkGetPhysicalDeviceProperties2(x64emu_t* emu, void* device, void*
         }
         p = p->pNext;
     }
-}
\ No newline at end of file
+}
+
+CREATE(vkCreateIndirectCommandsLayoutEXT)
+CREATE(vkCreateIndirectExecutionSetEXT)
+DESTROY64(vkDestroyIndirectCommandsLayoutEXT)
+DESTROY64(vkDestroyIndirectExecutionSetEXT)
+
+CREATE(vkCreatePipelineBinariesKHR)
+DESTROY64(vkDestroyPipelineBinaryKHR)
+DESTROY(vkReleaseCapturedPipelineDataKHR)
+
+CREATE(vkCreateTensorARM)
+CREATE(vkCreateTensorViewARM)
+DESTROY64(vkDestroyTensorARM)
+DESTROY64(vkDestroyTensorViewARM)
+
+CREATE(vkCreateDataGraphPipelineSessionARM)
+EXPORT int my_vkCreateDataGraphPipelinesARM(x64emu_t* emu, void* device, uint64_t deferredOperation, uint64_t pipelineCache,
+                                             uint32_t createInfoCount, void* pCreateInfos,
+                                             my_VkAllocationCallbacks_t* alloc, void* pPipelines)
+{
+    my_VkAllocationCallbacks_t my_alloc;
+    return my->vkCreateDataGraphPipelinesARM(device, deferredOperation, pipelineCache, createInfoCount, pCreateInfos, find_VkAllocationCallbacks(&my_alloc, alloc), pPipelines);
+}
+DESTROY64(vkDestroyDataGraphPipelineSessionARM)
\ No newline at end of file
