@@ -262,6 +262,27 @@ GO(vkCmdSetStencilOp, vFpuiiii)
 GO(vkCmdSetStencilTestEnable, vFpu)
 GO(vkCmdSetViewportWithCount, vFpup)
 
+// VK_VERSION_1_4
+GO(vkCmdBindDescriptorSets2, vFpp)
+GO(vkCmdBindIndexBuffer2, vFpUUUi)
+GO(vkCmdPushConstants2, vFpp)
+GO(vkCmdPushDescriptorSet, vFpiUuup)
+GO(vkCmdPushDescriptorSet2,vFpp)
+GO(vkCmdPushDescriptorSetWithTemplate, vFpUUup)
+GO(vkCmdPushDescriptorSetWithTemplate2, vFpp)
+GO(vkCmdSetLineStipple, vFpuW)
+GO(vkCmdSetRenderingAttachmentLocations, vFpp)
+GO(vkCmdSetRenderingInputAttachmentIndices, vFpp)
+GO(vkCopyImageToImage, iFpp)
+GO(vkCopyImageToMemory, iFpp)
+GO(vkCopyMemoryToImage, iFpp)
+GO(vkGetDeviceImageSubresourceLayout, vFppp)
+GO(vkGetImageSubresourceLayout2, vFpUpp)
+GO(vkGetRenderingAreaGranularity, vFppp)
+GO(vkMapMemory2, iFppp)
+GO(vkTransitionImageLayout, iFpup)
+GO(vkUnmapMemory2, iFpp)
+
 // VK_EXT_debug_report
 GOM(vkCreateDebugReportCallbackEXT, iFEpppp)
 GO(vkDebugReportMessageEXT, vFpuiULipp)
@@ -458,6 +479,11 @@ GO(vkReleaseProfilingLockKHR, vFp)
 // VK_NV_cooperative_matrix
 GO(vkGetPhysicalDeviceCooperativeMatrixPropertiesNV, iFppp)
 
+// VK_NV_cooperative_vector
+GO(vkCmdConvertCooperativeVectorMatrixNV, vFpup)
+GO(vkConvertCooperativeVectorMatrixNV, iFpp)
+GO(vkGetPhysicalDeviceCooperativeVectorPropertiesNV, iFppp)
+
 // VK_KHR_fragment_shading_rate
 GO(vkCmdSetFragmentShadingRateKHR, vFppp)
 GO(vkGetPhysicalDeviceFragmentShadingRatesKHR, iFppp)
@@ -994,4 +1020,56 @@ GO(wine_vkAcquireKeyedMutex, iFpUUu)
 GO(wine_vkReleaseKeyedMutex, iFpUU)
 
 // VK_NV_cooperative_matrix2
-GO(vkGetPhysicalDeviceCooperativeMatrixFlexibleDimensionsPropertiesNV, iFppp)
\ No newline at end of file
+GO(vkGetPhysicalDeviceCooperativeMatrixFlexibleDimensionsPropertiesNV, iFppp)
+
+// VK_AMD_anti_lag
+GO(vkAntiLagUpdateAMD, vFpp)
+
+// VK_EXT_device_generated_commands
+GO(vkCmdExecuteGeneratedCommandsEXT, vFpup)
+GO(vkCmdPreprocessGeneratedCommandsEXT, vFppp)
+GOM(vkCreateIndirectCommandsLayoutEXT, iFEpppp)
+GOM(vkCreateIndirectExecutionSetEXT, iFEpppp)
+GOM(vkDestroyIndirectCommandsLayoutEXT, vFEpUp)
+GOM(vkDestroyIndirectExecutionSetEXT, vFEpUp)
+GO(vkGetGeneratedCommandsMemoryRequirementsEXT, vFppp)
+GO(vkUpdateIndirectExecutionSetPipelineEXT, vFpUup)
+GO(vkUpdateIndirectExecutionSetShaderEXT, vFpUup)
+
+// VK_KHR_pipeline_binary
+GOM(vkCreatePipelineBinariesKHR, iFEpppp)
+GOM(vkDestroyPipelineBinaryKHR, vFEpUp)
+GO(vkGetPipelineBinaryDataKHR, iFppppp)
+GO(vkGetPipelineKeyKHR, iFppp)
+GOM(vkReleaseCapturedPipelineDataKHR, iFEppp)
+
+// VK_EXT_depth_clamp_control
+GO(vkCmdSetDepthClampRangeEXT, vFpup)
+
+// VK_ARM_tensors
+GO(vkBindTensorMemoryARM, iFpup)
+GO(vkCmdCopyTensorARM, vFpp)
+GOM(vkCreateTensorARM, iFEpppp)
+GOM(vkCreateTensorViewARM, iFEpppp)
+GOM(vkDestroyTensorARM, vFEpUp)
+GOM(vkDestroyTensorViewARM, vFEpUp)
+GO(vkGetDeviceTensorMemoryRequirementsARM, vFppp)
+GO(vkGetPhysicalDeviceExternalTensorPropertiesARM, vFppp)
+GO(vkGetTensorMemoryRequirementsARM, vFppp)
+
+// VK_EXT_descriptor_buffer + VK_ARM_tensors
+GO(vkGetTensorOpaqueCaptureDescriptorDataARM, iFppp)
+GO(vkGetTensorViewOpaqueCaptureDescriptorDataARM, iFppp)
+
+// VK_ARM_data_graph
+GO(vkBindDataGraphPipelineSessionMemoryARM, iFpup)
+GO(vkCmdDispatchDataGraphARM, vFpUp)
+GOM(vkCreateDataGraphPipelineSessionARM, iFEpppp)
+GOM(vkCreateDataGraphPipelinesARM, iFEpUUuppp)
+GOM(vkDestroyDataGraphPipelineSessionARM, vFEpUp)
+GO(vkGetDataGraphPipelineAvailablePropertiesARM, iFpppp)
+GO(vkGetDataGraphPipelinePropertiesARM, iFpppp)
+GO(vkGetDataGraphPipelineSessionBindPointRequirementsARM, iFpppp)
+GO(vkGetDataGraphPipelineSessionMemoryRequirementsARM, vFppp)
+GO(vkGetPhysicalDeviceQueueFamilyDataGraphProcessingEnginePropertiesARM, vFppp)
+GO(vkGetPhysicalDeviceQueueFamilyDataGraphPropertiesARM, iFpupp)
\ No newline at end of file
