@@ -7,12 +7,12 @@ GO(wl_array_copy, iFpp)
 GO(wl_array_init, vFp)
 GO(wl_array_release, vFp)
 DATA(wl_buffer_interface, 40)
-//DATA(wl_callback_interface, 0)
+DATA(wl_callback_interface, 40)
 DATA(wl_compositor_interface, 40)
-//DATA(wl_data_device_interface, 0)
-//DATA(wl_data_device_manager_interface, 0)
-//DATA(wl_data_offer_interface, 0)
-//DATA(wl_data_source_interface, 0)
+DATA(wl_data_device_interface, 40)
+DATA(wl_data_device_manager_interface, 40)
+DATA(wl_data_offer_interface, 40)
+DATA(wl_data_source_interface, 40)
 GO(wl_display_cancel_read, vFp)
 GO(wl_display_connect, pFp)
 GO(wl_display_connect_to_fd, pFi)
