@@ -1,5 +1,5 @@
 #if !(defined(GO) && defined(GOM) && defined(GO2) && defined(DATA))
-#error meh!
+#error Meh...
 #endif
 
 GO(XineramaIsActive, iFp)
@@ -7,8 +7,8 @@ GO(XineramaQueryExtension, iFppp)
 GO(XineramaQueryScreens, pFpp)
 GO(XineramaQueryVersion, iFppp)
 GO(XPanoramiXAllocInfo, pFv)
-GO(XPanoramiXGetScreenCount, iFppp)
-GO(XPanoramiXGetScreenSize, iFppip)
-GO(XPanoramiXGetState, iFppp)
+GO(XPanoramiXGetScreenCount, iFpLp)
+GO(XPanoramiXGetScreenSize, iFpLip)
+GO(XPanoramiXGetState, iFpLp)
 GO(XPanoramiXQueryExtension, iFppp)
 GO(XPanoramiXQueryVersion, iFppp)
