@@ -120,7 +120,7 @@ GO(xsltGetSecurityPrefs, pFpi)
 GO(xsltGetUTF8Char, iFpp)
 GO(xsltGetXIncludeDefault, iFv)
 GO(xsltIf, vFpppp)
-GO(xsltInit, vFv)
+GOM(xsltInit, vFEv)
 GO(xsltInitAllDocKeys, iFp)
 GO(xsltInitCtxtExts, iFp)
 //GO(xsltInitCtxtKey, 
@@ -244,4 +244,6 @@ GO(xsltXPathCompile, pFpp)
 GO(xsltXPathCompileFlags, pFppi)
 GO(xsltXPathFunctionLookup, pFppp)
 GO(xsltXPathGetTransformContext, pFp)
-//GO(xsltXPathVariableLookup, 
\ No newline at end of file
+//GO(xsltXPathVariableLookup, 
+
+DATA(xsltMaxVars, 4)
\ No newline at end of file
