@@ -82,8 +82,8 @@ GO(FcFini, vFv)
 GOM(FcFontList, pFEppp)
 GO(FcFontMatch, pFppp)
 GO(FcFontRenderPrepare, pFppp)
-//GO(FcFontSetAdd, iFbiibp__p)
-//GO(FcFontSetCreate, biibp__Fv)
+GOM(FcFontSetAdd, iFEpp)
+GOM(FcFontSetCreate, pFEv)
 GOM(FcFontSetDestroy, vFEp)
 //GO(FcFontSetList, biibp__Fpbbiibp___ipbiibp__)
 //GO(FcFontSetMatch, pFpbbiibp___ipp)
