@@ -54,6 +54,7 @@ typedef void (*vFupupi_t)(uint32_t, void*, uint32_t, void*, int);
 typedef void (*vFuippp_t)(uint32_t, int, void*, void*, void*);
 typedef void (*vFuuippp_t)(uint32_t, uint32_t, int, void*, void*, void*);
 typedef void (*vFupupip_t)(uint32_t, void*, uint32_t, void*, int, void*);
+typedef void (*vFppupui_t)(void*, void*, uint32_t, void*, uint32_t, int);
 typedef void (*debugProc_t)(int32_t, int32_t, uint32_t, int32_t, int32_t, void*, void*);
 
 typedef struct gl_wrappers_s {
@@ -484,6 +485,35 @@ static void* find_glXChooseFBConfig_Fct(void* fct)
     printf_log(LOG_NONE, "Warning, no more slot for libGL glXChooseFBConfig callback\n");
     return NULL;
 }
+// glXChooseFBConfigSGIX ...
+#define GO(A)                                                                                               \
+static pFpipp_t my32_glXChooseFBConfigSGIX_fct_##A = NULL;                                                      \
+static void* my32_glXChooseFBConfigSGIX_##A(x64emu_t* emu, void* dpy, int screen, int* list, int* nelement)     \
+{                                                                                                           \
+    if(!my32_glXChooseFBConfigSGIX_fct_##A)                                                                     \
+        return NULL;                                                                                        \
+    void** res = my32_glXChooseFBConfigSGIX_fct_##A (dpy, screen, list, nelement);                              \
+    if(!res)                                                                                                \
+        return NULL;                                                                                        \
+    ptr_t* fbconfig = (ptr_t*)res;                                                                          \
+    for(int i=0; i<*nelement; ++i)                                                                          \
+        fbconfig[i] = to_ptrv(res[i]);                                                                      \
+    return res;                                                                                             \
+}
+SUPER()
+#undef GO
+static void* find_glXChooseFBConfigSGIX_Fct(void* fct)
+{
+    if(!fct) return fct;
+    #define GO(A) if(my32_glXChooseFBConfigSGIX_fct_##A == (pFpipp_t)fct) return my32_glXChooseFBConfigSGIX_##A;
+    SUPER()
+    #undef GO
+    #define GO(A) if(my32_glXChooseFBConfigSGIX_fct_##A == 0) {my32_glXChooseFBConfigSGIX_fct_##A = (pFpipp_t)fct; return my32_glXChooseFBConfigSGIX_##A; }
+    SUPER()
+    #undef GO
+    printf_log(LOG_NONE, "Warning, no more slot for libGL glXChooseFBConfigSGIX callback\n");
+    return NULL;
+}
 // glXGetVisualFromFBConfig ...
 #define GO(A)                                                                                               \
 static pFpp_t my32_glXGetVisualFromFBConfig_fct_##A = NULL;                                                 \
@@ -512,6 +542,34 @@ static void* find_glXGetVisualFromFBConfig_Fct(void* fct)
     printf_log(LOG_NONE, "Warning, no more slot for libGL glXGetVisualFromFBConfig callback\n");
     return NULL;
 }
+// glXGetVisualFromFBConfigSGIX ...
+#define GO(A)                                                                                           \
+static pFpp_t my32_glXGetVisualFromFBConfigSGIX_fct_##A = NULL;                                         \
+static void* my32_glXGetVisualFromFBConfigSGIX_##A(x64emu_t* emu, void* dpy, void* config)              \
+{                                                                                                       \
+    if(!my32_glXGetVisualFromFBConfigSGIX_fct_##A)                                                      \
+        return NULL;                                                                                    \
+    void* res = my32_glXGetVisualFromFBConfigSGIX_fct_##A (dpy, config);                                \
+    if(!res)                                                                                            \
+        return NULL;                                                                                    \
+    my_XVisualInfo_32_t* vinfo = (my_XVisualInfo_32_t*)res;                                             \
+    convert_XVisualInfo_to_32(dpy, vinfo, res);                                                         \
+    return vinfo;                                                                                       \
+}
+SUPER()
+#undef GO
+static void* find_glXGetVisualFromFBConfigSGIX_Fct(void* fct)
+{
+    if(!fct) return fct;
+    #define GO(A) if(my32_glXGetVisualFromFBConfigSGIX_fct_##A == (pFpp_t)fct) return my32_glXGetVisualFromFBConfigSGIX_##A;
+    SUPER()
+    #undef GO
+    #define GO(A) if(my32_glXGetVisualFromFBConfigSGIX_fct_##A == 0) {my32_glXGetVisualFromFBConfigSGIX_fct_##A = (pFpp_t)fct; return my32_glXGetVisualFromFBConfigSGIX_##A; }
+    SUPER()
+    #undef GO
+    printf_log(LOG_NONE, "Warning, no more slot for libGL glXGetVisualFromFBConfigSGIX callback\n");
+    return NULL;
+}
 // glXChooseVisual ...
 #define GO(A)                                                                                      \
 static pFpip_t my32_glXChooseVisual_fct_##A = NULL;                                                \
@@ -565,6 +623,31 @@ static void* find_glXCreateContext_Fct(void* fct)
     printf_log(LOG_NONE, "Warning, no more slot for libGL glXCreateContext callback\n");
     return NULL;
 }
+// glXGetFBConfigFromVisualSGIX ...
+#define GO(A)                                                                                           \
+static pFpp_t my32_glXGetFBConfigFromVisualSGIX_fct_##A = NULL;                                       \
+static void* my32_glXGetFBConfigFromVisualSGIX_##A(x64emu_t* emu, void* dpy, my_XVisualInfo_32_t* info) \
+{                                                                                                       \
+    if(!my32_glXGetFBConfigFromVisualSGIX_fct_##A)                                                      \
+        return NULL;                                                                                    \
+    my_XVisualInfo_t info_l = {0};                                                                      \
+    convert_XVisualInfo_to_64(dpy, &info_l, info);                                                      \
+    return my32_glXGetFBConfigFromVisualSGIX_fct_##A (dpy, &info_l);                                    \
+}
+SUPER()
+#undef GO
+static void* find_glXGetFBConfigFromVisualSGIX_Fct(void* fct)
+{
+    if(!fct) return fct;
+    #define GO(A) if(my32_glXGetFBConfigFromVisualSGIX_fct_##A == (pFpp_t)fct) return my32_glXGetFBConfigFromVisualSGIX_##A;
+    SUPER()
+    #undef GO
+    #define GO(A) if(my32_glXGetFBConfigFromVisualSGIX_fct_##A == 0) {my32_glXGetFBConfigFromVisualSGIX_fct_##A = (pFpp_t)fct; return my32_glXGetFBConfigFromVisualSGIX_##A; }
+    SUPER()
+    #undef GO
+    printf_log(LOG_NONE, "Warning, no more slot for libGL glXGetFBConfigFromVisualSGIX callback\n");
+    return NULL;
+}
 // glMultiDrawElements ...
 #define GO(A)                                                                                                                       \
 static vFupupi_t my32_glMultiDrawElements_fct_##A = NULL;                                                                           \
@@ -595,6 +678,36 @@ static void* find_glMultiDrawElementsEXT_Fct(void* fct)
 {
     return find_glMultiDrawElements_Fct(fct);
 }
+// glMultiModeDrawElementsIBM ...
+#define GO(A)                                                                                                                                               \
+static vFppupui_t my32_glMultiModeDrawElementsIBM_fct_##A = NULL;                                                                                           \
+static void my32_glMultiModeDrawElementsIBM_##A(x64emu_t* emu, void* mode, void* count, uint32_t type, ptr_t* indices, uint32_t primcount, int modestride)  \
+{                                                                                                                                                           \
+    if(!my32_glMultiModeDrawElementsIBM_fct_##A)                                                                                                            \
+        return;                                                                                                                                             \
+    void* indices_l[primcount];                                                                                                                             \
+    for(uint32_t i=0; i<primcount; ++i)                                                                                                                     \
+        indices_l[i] = from_ptrv(indices[i]);                                                                                                               \
+    my32_glMultiModeDrawElementsIBM_fct_##A (mode, count, type, indices_l, primcount, modestride);                                                          \
+}
+SUPER()
+#undef GO
+static void* find_glMultiModeDrawElementsIBM_Fct(void* fct)
+{
+    if(!fct) return fct;
+    #define GO(A) if(my32_glMultiModeDrawElementsIBM_fct_##A == (vFppupui_t)fct) return my32_glMultiModeDrawElementsIBM_##A;
+    SUPER()
+    #undef GO
+    #define GO(A) if(my32_glMultiModeDrawElementsIBM_fct_##A == 0) {my32_glMultiModeDrawElementsIBM_fct_##A = (vFppupui_t)fct; return my32_glMultiModeDrawElementsIBM_##A; }
+    SUPER()
+    #undef GO
+    printf_log(LOG_NONE, "Warning, no more slot for libGL glMultiModeDrawElementsIBM callback\n");
+    return NULL;
+}
+static void* find_glMultiModeDrawElementsIBMEXT_Fct(void* fct)
+{
+    return find_glMultiModeDrawElementsIBM_Fct(fct);
+}
 // glTransformFeedbackVaryings ...
 #define GO(A)                                                                                                               \
 static vFuipu_t my32_glTransformFeedbackVaryings_fct_##A = NULL;                                                            \
@@ -945,6 +1058,17 @@ EXPORT void* my32_glXChooseFBConfig(x64emu_t* emu, void* dpy, int screen, int* l
     return res;
 }
 
+EXPORT void* my32_glXChooseFBConfigSGIX(x64emu_t* emu, void* dpy, int screen, int* list, int* nelement)
+{
+    void** res = my->glXChooseFBConfigSGIX(dpy, screen, list, nelement);
+    if(!res)
+        return NULL;
+    ptr_t *fbconfig = (ptr_t*)res;
+    for(int i=0; i<*nelement; ++i)
+        fbconfig[i] = to_ptrv(res[i]);
+    return res;
+}
+
 EXPORT void* my32_glXGetVisualFromFBConfig(x64emu_t* emu, void* dpy, void* config)
 {
     void* res = my->glXGetVisualFromFBConfig(dpy, config);
@@ -954,6 +1078,15 @@ EXPORT void* my32_glXGetVisualFromFBConfig(x64emu_t* emu, void* dpy, void* confi
     return vinfo;
 }
 
+EXPORT void* my32_glXGetVisualFromFBConfigSGIX(x64emu_t* emu, void* dpy, void* config)
+{
+    void* res = my->glXGetVisualFromFBConfigSGIX(dpy, config);
+    if(!res) return NULL;
+    my_XVisualInfo_32_t* vinfo = (my_XVisualInfo_32_t*)res;
+    convert_XVisualInfo_to_32(dpy, vinfo, res);
+    return vinfo;
+}
+
 EXPORT void* my32_glXChooseVisual(x64emu_t* emu, void* dpy, int screen, int* attr)
 {
     void* res = my->glXChooseVisual(dpy, screen, attr);
@@ -970,6 +1103,13 @@ EXPORT void* my32_glXCreateContext(x64emu_t* emu, void* dpy, my_XVisualInfo_32_t
     return my->glXCreateContext(dpy, &info_l, shared, direct);
 }
 
+EXPORT void* my32_glXGetFBConfigFromVisualSGIX(x64emu_t* emu, void* dpy, my_XVisualInfo_32_t* info)
+{
+    my_XVisualInfo_t info_l = {0};
+    convert_XVisualInfo_to_64(dpy, &info_l, info);
+    return my->glXGetFBConfigFromVisualSGIX(dpy, &info_l);
+}
+
 EXPORT void my32_glMultiDrawElements(x64emu_t* emu, uint32_t mode, void* count, uint32_t type, ptr_t* indices, int drawcount)
 {
     void* indices_l[drawcount];
@@ -982,6 +1122,14 @@ EXPORT void my32_glMultiDrawElementsExt(x64emu_t* emu, uint32_t mode, void* coun
     my32_glMultiDrawElements(emu, mode, count, type, indices, drawcount);
 }
 
+EXPORT void my32_glMultiModeDrawElementsIBM(x64emu_t* emu, void* mode, void* count, uint32_t type, ptr_t* indices, uint32_t primcount, int modestride)
+{
+    void* indices_l[primcount];
+    for(uint32_t i=0; i<primcount; ++i)
+        indices_l[i] = from_ptrv(indices[i]);
+    my->glMultiModeDrawElementsIBM(mode, count, type, indices_l, primcount, modestride);
+}
+
 EXPORT void my32_glTransformFeedbackVaryings(x64emu_t* emu, uint32_t prog, int count, ptr_t* varyings, uint32_t mode)
 {
     void* varyings_l[count];
@@ -1079,10 +1227,14 @@ EXPORT void my32_glGetUniformIndices(x64emu_t* emu, uint32_t prog, int count, pt
  GO(vFuipp_t, glShaderSource)                   \
  GO(vFuipp_t, glShaderSourceARB)                \
  GO(pFpipp_t, glXChooseFBConfig)                \
+ GO(pFpipp_t, glXChooseFBConfigSGIX)            \
  GO(pFpp_t, glXGetVisualFromFBConfig)           \
+ GO(pFpp_t, glXGetVisualFromFBConfigSGIX)       \
  GO(pFpp_t, glXChooseVisual)                    \
- GO(pFpp_t, glXCreateContext)                   \
+ GO(pFpppi_t, glXCreateContext)                 \
+ GO(pFpp_t, glXGetFBConfigFromVisualSGIX)       \
  GO(vFupupi_t, glMultiDrawElements)             \
+ GO(vFppupui_t, glMultiModeDrawElementsIBM)     \
  GO(vFuipu_t, glTransformFeedbackVaryings)      \
  GO(vFuuippp_t, glBindBuffersRange)             \
  GO(vFuippp_t, glBindVertexBuffers)             \
@@ -1165,13 +1317,13 @@ void* getGLProcAddress32(x64emu_t* emu, glprocaddress_t procaddr, const char* rn
     } else
         symbol = procaddr(rname);
     if(!symbol) {
-        printf_dlsym(LOG_DEBUG, "%p\n", NULL);
+        printf_dlsym_prefix(0, LOG_DEBUG, "%p\n", NULL);
         return NULL;    // easy
     }
     // check if alread bridged
     uintptr_t ret = CheckBridged(emu->context->system, symbol);
     if(ret) {
-        printf_dlsym(LOG_DEBUG, "%p\n", (void*)ret);
+        printf_dlsym_prefix(0, LOG_DEBUG, "%p\n", (void*)ret);
         return (void*)ret; // already bridged
     }
     // get wrapper
@@ -1191,8 +1343,8 @@ void* getGLProcAddress32(x64emu_t* emu, glprocaddress_t procaddr, const char* rn
         k = kh_get(symbolmap, wrappers->glwrappers, tmp);
     }
     if(k==kh_end(wrappers->glwrappers)) {
-        printf_dlsym(LOG_DEBUG, "%p\n", NULL);
-        printf_dlsym(LOG_INFO, "Warning, no wrapper for %s\n", rname);
+        printf_dlsym_prefix(0, LOG_DEBUG, "%p\n", NULL);
+        printf_dlsym_prefix(2, LOG_INFO, "Warning, no wrapper for %s\n", rname);
         return NULL;
     }
     symbol1_t* s = &kh_value(wrappers->glwrappers, k);
@@ -1203,6 +1355,6 @@ void* getGLProcAddress32(x64emu_t* emu, glprocaddress_t procaddr, const char* rn
         s->resolved = 1;
     }
     ret = s->addr;
-    printf_dlsym(LOG_DEBUG, "%p\n", (void*)ret);
+    printf_dlsym_prefix(0, LOG_DEBUG, "%p\n", (void*)ret);
     return (void*)ret;
 }
