@@ -68,6 +68,6 @@ GO(XRRDeleteOutputMode, vFXLL)
 GOM(XRRGetOutputInfo, pFEXpL)
 GO(XRRConfigureProviderProperty, vFXLLiiibl_)
 //GO(XRRAllocateMonitor, bLiiiiiiiiibL__FXi)
-//GO(XRRGetMonitors, bLiiiiiiiiibL__FXLip)
+GOM(XRRGetMonitors, pFEXLip)
 //GO(XRRSetMonitor, vFXLbLiiiiiiiiibL__)
-//GO(XRRFreeMonitors, vFbLiiiiiiiiibL__)
+GOM(XRRFreeMonitors, vFEp)
