@@ -42,7 +42,7 @@ GO(XRenderFreeGlyphSet, vFXL)
 GO(XRenderFreePicture, vFXL)
 GO(XRenderParseColor, iFXpp)
 GO(XRenderQueryExtension, iFXpp)
-GO(XRenderQueryFilters, pFXL)
+GOM(XRenderQueryFilters, pFEXL)
 GO(XRenderQueryFormats, iFX)
 GO(XRenderQueryPictIndexValues, pFXrLiiwwwwwwwwL_p)
 GO(XRenderQuerySubpixelOrder, iFXi)
