@@ -122,7 +122,7 @@ GO(SDL_FreeAudioStream, vFp)
 GO(SDL_FreeCursor, vFp)
 //GO(SDL_FreeFormat, vFp)
 //GO(SDL_FreePalette, vFp)
-//GO(SDL_FreeRW, vFp)
+GO(SDL_FreeRW, vFp)
 GOM(SDL_FreeSurface, vFEp)
 //GO(SDL_FreeWAV, vFp)
 GO(SDL_GameControllerAddMapping, iFp)
@@ -197,7 +197,7 @@ GO(SDL_GetCurrentVideoDriver, pFv)
 // SDL_GetDefaultAssertionHandler
 //GO(SDL_GetDefaultAudioInfo, iFppi)
 //GO(SDL_GetDefaultCursor, pFv)
-GOM(SDL_GetDesktopDisplayMode, iFEip)
+GOM(SDL_GetDesktopDisplayMode, iFip)    //%noE
 GO(SDL_GetDisplayBounds, iFip)
 GO(SDL_GetDisplayDPI, iFippp)
 GOM(SDL_GetDisplayMode, iFiip)  //%noE
@@ -233,7 +233,7 @@ GO(SDL_GetNumVideoDrivers, iFv)
 //GOM(SDL_GetOriginalMemoryFunctions,
 GO(SDL_GetPerformanceCounter, UFv)
 GO(SDL_GetPerformanceFrequency, UFv)
-//GO(SDL_GetPixelFormatName, pFu)
+GO(SDL_GetPixelFormatName, pFu)
 //GO(SDL_GetPlatform, pFv)
 //GO(SDL_GetPointDisplayIndex, iFp)
 //GO(SDL_GetPowerInfo, uFpp)
@@ -273,7 +273,7 @@ GO(SDL_GetTicks, uFv)
 //GO(SDL_GetTouchDevice, lFi)
 //GO(SDL_GetTouchDeviceType, iFl)
 //GO(SDL_GetTouchFinger, pFli)
-//GO(SDL_GetVersion, vFp)
+GO(SDL_GetVersion, vFp)
 GO(SDL_GetVideoDriver, pFi)
 //GO(SDL_GetWindowBordersSize, iFppppp)
 //GO(SDL_GetWindowBrightness, fFp)
@@ -559,7 +559,7 @@ GO(SDL_RestoreWindow, vFp)
 //GOM(SDL_RWFromConstMem, pFEpi)
 //GOM(SDL_RWFromFP, pFEpi)
 GOM(SDL_RWFromFile, pFEpp)
-//GOM(SDL_RWFromMem, pFEpi)
+GOM(SDL_RWFromMem, pFEpi)
 //GOM(SDL_SaveAllDollarTemplates, iFEp)
 //GOM(SDL_SaveBMP_RW, iFEppi)
 //GOM(SDL_SaveDollarTemplate, iFEip)
